#!/usr/bin/env python

# change next line to 1 to debug script
debug=0

""" accounts.cgi this program provides functions for 
    simple LETS accounting 
    Richard Kay last changed: 7 Feb 2010
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import atab,btab,mtab,stab
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']
sec_vars=cgiutils.get_vars('secure.cfg')
tithe_vars=cgiutils.get_vars('tithes.cfg')

def ledger(session,ac_id):
  # prints table showing ledger of acknowledgements 
  # from selected archive or current table (admin only)
  if not cgiutils.auth(session,ac_id,"LDGRVIEW","view payments ledger"):
    return
  if "ackfile" not in woi.keys():
    # if archived file not selected display current table by default
    acks=tablcgi.table(atab,dir=dl)
    acks.sort(key="ack_id")
    acks.tab2html()
    cgiutils.html_end(session)
  else:
    ackfilename=woi.firstval("ackfile")
    if not cgiutils.is_valid(ackfilename,max_leng=14,
                  allowed='^acks(\.[0-9]+)?\.pkl$'):
      cgiutils.html_end(session,
            error="Invalid acknowledgements archive filename.")
      return
    import os.path
    full_ackfilename=os.path.join(dl,ackfilename)
    if not os.path.isfile(full_ackfilename):
      cgiutils.html_end(session,
                    error="Acknowledgements archive name is not a file.")
      return
    # clone the atab table definition object before
    # overriding the atab.file attribute, to avoid poisoning use of this
    # object attribute elsewhere
    cacks=tablcgi.table_def(atab)
    cacks.file=full_ackfilename
    aso=tablcgi.table(cacks,dir=dl)
    aso.sort(key="ack_id")
    aso.tab2html()
    cgiutils.html_end(session)

def strip_str(longst,newlen):
  ''' strips long string to newlen leaving out spaces.
  This ministatement requirement exists due to 
  the SMS message limit of 160 chars. To improve clarity slightly
  words starting after a stripped space are capitalised. '''
  outs="" # output string starts empty
  nextcap=False
  for char in longst:
    if char not in [" ","\t","\n"]: # ignore whitespace
      if nextcap:
        outs+=char.upper()
      else:
        outs+=char
      nextcap=False
    else:
      nextcap=True
    if len(outs) == newlen:
      return outs
  return outs

def mini_statement(ac_id,n=3):
  ''' generates mini statement for SMS user showing last
  n transactions for ac_id.'''
  # 
  # if transactions are tithed, small ministatements (n<6)
  # should not include automated tithe transactions.
  # default value for titheac which can't possibly match a real ac_id 
  titheac="this_string_cant_possibly_equal_an_ac_id_X6q28vG9Yiufv639713"
  if tithe_vars["tithes"] in ['y','Y']:
    tithe=True
  else:
    tithe=False
  if tithe:
    titheac=tithe_vars["to_ac"] # to_account will match this if tithe  
  acks=tablcgi.table(atab,dir=dl)
  balances=tablcgi.table(btab,dir=dl)
  balidx=balances.find(ac_id)
  fbalance=balances.data[balidx]["balance"]
  sbalance="%.2f" % fbalance
  acks.sort(key="ack_id")
  # create prefiltered acks view table object containing only 
  # non repudiated acks relating to ac_id
  ack_view_def=tablcgi.table_def(atab) # clone don't copy
  ack_view_def.file=None # memory object only
  ackv=tablcgi.table(ack_view_def) # ackv is view table 
  nacks_foruser=0 # count how many - we only want the last n
  for row in acks.data: # run through to get relevant acks 
    int_ackid=int(row["ack_id"])
    rep_id=str( -int_ackid ) # repudiation associated   
    if row["from_id"] == ac_id or row["to_id"] == ac_id:
      if int_ackid > 0 and acks.find(rep_id) == -1 :
        # only view non repudiated acks involving ack_id
        nacks_foruser+=1
        avrow={}
        for key in [ "ack_id","from_id","from_name","to_id",
                     "to_name","system","amount","details","time" ]:
          avrow[key]=row[key]
        ackv.addrow(avrow)
  # remove ackv rows relating to tithe payments if n < 6 
  # for small ministatement
  if nacks_foruser > n and n < 6:
    for row in ackv.data:
      if row["to_id"]==titheac: # payment of a tithe
        ackv.delrow(row["ack_id"])
        nacks_foruser-=1
        if nacks_foruser == n:
          break # stop when all ack records can be output
  # how many payments to skip ?
  if nacks_foruser <= n:
    nto_skip=0 # too few on file to skip any
  else:
    nto_skip=nacks_foruser - n 
  # process all acks for ac_id beyond the skip point
  i=nto_skip 
  print("With Name Paid Date Bal="+sbalance) # SMS statement heading 
  import time
  while i < nacks_foruser:
    # strip details to first 10 chars in string omitting spaces 
    details=strip_str(ackv.data[i]["details"],10) 
    if ackv.data[i]["from_id"] == ac_id:
      withac=ackv.data[i]["to_id"] 
      who=ackv.data[i]["to_name"]
      sign=-1.0 # negative amount, purchase
    else:
      withac=ackv.data[i]["from_id"] 
      who=ackv.data[i]["from_name"] 
      sign=1.0 # positive amount, earnings
    # shorten name if > 10 chars
    swho=strip_str(who,10) 
    amountstr="%.2f" %  (ackv.data[i]["amount"] * sign) 
    time_struct=time.localtime(ackv.data[i]["time"])
    # output time in DDMmmYY format e.g 12Jan10 for 12 January 2010
    strdate=time.strftime("%d%b%y",time_struct)
    print(withac+" "+swho+" "+amountstr+" "+strdate+" "+details)
    i+=1

def statement(session,ac_id): 
  # open balances table to get carry forward amount
  balances=tablcgi.table(btab,dir=dl)
  bal_indx=balances.find(ac_id)
  if bal_indx == -1:
    cgiutils.html_end(session,
        error="account has no balance. Please inform webmaster")
    return
  cfwd=balances.data[bal_indx]["cfwd"]
  
  # open acknowledgements table
  acks=tablcgi.table(atab,dir=dl)
  acks.sort(key="ack_id")
  # create statement view table object with new balance field
  st_view_def=tablcgi.table_def(atab)
  st_view_def.file=None
  st_view_def.meta["keys"].append("newbal")
  st_view_def.meta["colheads"].append("Balance")
  st_view_def.meta["formats"].append("%.2f")
  st_view=tablcgi.table(st_view_def)
  balance=cfwd
  for row in acks.data:
    # normal ack
    if int(row["ack_id"]) >= 0 :
      if row["from_id"] == ac_id:
        balance = balance - row["amount"]
        outp = 1
      elif row["to_id"] == ac_id:
        balance = balance + row["amount"]
        outp = 1
      else:
        outp = 0
    else:
      # repudiation ack with negative ack_id
      if row["from_id"] == ac_id:
        balance = balance + row["amount"]
        outp = 1
      elif row["to_id"] == ac_id:
        balance = balance - row["amount"]
        outp = 1
      else:
        outp = 0
    if outp:
      row["newbal"] = balance 
      st_view.addrow(row)
  # output to browser 
  carryforward=open("cfwd_date.txt").read()
  print(carryforward)
  st_view.tab2html(skip_cols=["system"])
  cgiutils.html_end(session)

def summary(session,option,members,ac_id):
  # checks whether requested summary file/table is old. If so
  # it is updated and saved. It is then displayed.
  import os,stat,time
  import os.path
  sfdo=tablcgi.table_def(stab) # clone summary file definition object
  if option == 'all':
    sfdo.file=os.path.join(dl,'summary_all.pkl')
  else: # option == active
    sfdo.file=os.path.join(dl,'summary_active.pkl')
  no_file=1 # true if file doesn't exist
  if os.path.isfile(sfdo.file):
    no_file=0 # file does exist
    statobj=os.stat(sfdo.file) # statobj is a file status tuple object
    # mtime is float seconds since summary file modified 
    mtime=time.time() - statobj[stat.ST_MTIME]
  # if no_file or mtime > (1): previously summaries were thought 
  # expensive to generate for some reason.
  if no_file or mtime > 1 :
    # create if nonexistent or recreate if more than 1 seconds old
    # recreate summary file from balance and ack data
    stabview=tablcgi.table_def(sfdo) # clone summary def object as view,
    # add key for bgcolor attribute used for selective row coloring
    stabview.meta["keys"].append("bgcolor")
    stabview.meta["colheads"].append("Bgc")
    stabview.meta["formats"].append("%s")
    # retaining appropriate file name in sfdo.file
    stabview.file=None # update view object in memory only until save
    sumry=tablcgi.table(stabview,dir=dl)
    # open acknowledgements table
    acks=tablcgi.table(atab,dir=dl)
    # open balances table 
    balances=tablcgi.table(btab,dir=dl)
    # copy balance records required to new summary table
    for row in balances.data:
      mindex=members.find(row["ac_id"])
      active=members.data[mindex]["active"] 
      name=members.data[mindex]["name"] 
      if (option == 'active' and active in ['Y','P']) or option == 'all': 
        srow={}
        for key in ["ac_id","cfwd","balance","in","out","turnover","oldturn"]:
          srow[key]=row[key]
        srow["name"]=name
        srow["bgcolor"]=cgiutils.get_color(active,
                           default_color='"#FFFFFF"')
        sumry.addrow(srow)
    # make sumry saveable by restoring filenames and save it
    sumry.file=sfdo.file
    sumry.lockfilename=sumry.file+'.lock'
    sumry.save()
  # end if summary file more than one second old 
  # reopen and read summary file
  sumary=tablcgi.table(sfdo,dir=dl)
  # output to browser
  carryforward=open("cfwd_date.txt").read()
  print(carryforward)
  import table_details
  sumary.sort(key="ac_id",cmpfunc=table_details.cmp_acid)
  if not cgiutils.auth(session,ac_id,"SUMRYVIEW","view accounts summary",
                       whom=ac_id,auditonly=True):
    return
  cgiutils.print_colortable() 
  sumary.tab2html(skip_cols=["bgcolor"],color_row_key="bgcolor")
  cgiutils.html_end(session)

def main():
  session=False
  if "ministate" not in woi.keys():
    cgiutils.html_header(title=
       "LETS Accounting System",bgcolor='"#FFFFDD"')
  else: 
    print("Content-type: text/plain\n")
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid or stale or logged out session.")
    return
  # which submit button was pressed ?
  valid,option=cgiutils.get_option(woi,
      buttons=["statement","balance","active","all","ack",
      "repudiate","ledger","ministate"])
  if not valid:
    return
 
  elif option == "ack":
    # send the ack1 form
    if ua != "admin": # default prefilled from account in form
      fromvalue=ac_id # presumably from account is user as logged in
    else:
      fromvalue="" # leave it blank. Don't yet know who payment is from 
    print(cgiutils.send_form("ack1.form",[v['registry'],ac_id,cookie,
                       ua,fromvalue]))
    cgiutils.html_end(session,want_form=1)
  elif option == "repudiate":
    # send the repud1 form
    print(cgiutils.send_form("repud1.form",[v['registry'],ac_id,cookie,ua]))
    cgiutils.html_end(session,want_form=1)
  elif option == "balance":
    # send the balance enquiry form
    print(cgiutils.send_form("balance.form",[v['registry'],ac_id,cookie,ua]))
  elif option == "statement":
    members=tablcgi.table(mtab,dir=dl)
    whom=woi.firstval("whom")
    if (not whom) or whom == ac_id : # request for own statement
      statement(session,ac_id)
    else: # request for someone elses statement must be authorised
      # check id for whom statement requested is an account
      if members.find(whom) == -1:
        cgiutils.html_end(session,
          error="Statement requested for non existent account.")
        return
      if ac_id == 'admin' and sec_vars["admin_has_superpowers"] == "yes":
        # implicitly authorised so just audit/log it.
        cgiutils.auth(session,ac_id,"STATEMENT","admin statement request",
                      whom=whom,auditonly=True)
        statement(session,whom)
        return
      elif not cgiutils.auth(session,
           ac_id,"STATEMENT","statement request",whom=whom):
        return
      statement(session,whom)
      return
  elif option == "active" or option == "all":
    members=tablcgi.table(mtab,dir=dl)
    summary(session,option,members,ac_id) 
  elif option == "ledger":
    ledger(session,ac_id) 
  elif option == "ministate":
    members=tablcgi.table(mtab,dir=dl)
    whom=woi.firstval("whom")
    # check whom has account
    if members.find(whom) == -1:
      # output error in plain text 
      print("error ministatement request for non existent account")
      return
    # check if user is authorised to obtain statement for whom a/c
    if not cgiutils.auth(session,
              ac_id,"STATEMENT","SMS ministatement",whom=whom):
      # stop it if not
      return
    # see if no. of transactions to report stated
    if "nacks" not in woi.keys():
      na=3 # default value, suitable for 160 char SMS
    else: 
      # validate nacks
      nas=woi.firstval("nacks")
      if not cgiutils.is_valid(nas,max_leng=6,
        allowed='^[0-9]+$'):
        print("error invalid value for no. of acks to report.")
        return
      na=int(nas) 
    mini_statement(whom,n=na)
  else:
    # should have trapped this one earlier ???
    cgiutils.html_end(session,error="Invalid option value (2nd trap).")
  return 

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in accounts.cgi main()")
    traceback.print_exc()

