#!/usr/bin/env python

# change 0 to 1 to debug script

debug=0

""" ack.cgi This file handles the 2 (ack1.form and ack2.form) 
    forms used to input and then confirm an acknowledgement. 

    Facilities include payment and repudiation. 

    Richard Kay 
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import btab,mtab,atab

v=cgiutils.get_vars('site.cfg')
dl=v['data_location']

# who_can_ack is controlled by editing secure.cfg file 
# 'ac_can_pay' to allow account holders to pay from own a/c [default]
# 'admin_plus' to allow only admin or authorised accounts to make payments
# 'all_can_record' to allow any account holders to pay from any a/c

sec_vars=cgiutils.get_vars('secure.cfg')
who_can_ack=sec_vars['who_can_ack']
if who_can_ack not in ['ac_can_pay','admin_plus','all_can_record']:
  raise ValueError("invalid value for who_can_ack configured: "+who_can_ack)

def conv_datetime(timeval):
  import time
  time_struct=time.localtime(float(timeval))
  return time.asctime(time_struct)

def main():
  session=False
  if woi.has_required(["textonly"]):
    cgiutils.htmloutput=False # turns off HTML header and footer for SMS
  cgiutils.html_header(title="LETS Accounting acknowledgement processing",
                       bgcolor='"#FFFFDD"')
  confirmation_needed=0
  repudiation=0
  if len(list(woi.keys())) == 0:
    # if no keys authentication data missing
    cgiutils.html_end(session,error="Missing authentication details.")
    return
  elif woi.has_required(["ac_id","pin","ua","ack_id","repudiate"]
      ) and not woi.has_required(["from_id","from_name","to_id",
      "to_name","amount","details"]):
    # first stage repudiation of a previous ack
    repudiation=1
  elif woi.has_required(["ac_id","pin","ua","ack_id","repudiate",
      "from_id","from_name","to_id","to_name","amount","details"]):
    # second stage repudiation of a previous ack
    repudiation=2
  elif woi.has_required(["ac_id","pin","ua","from_id",
        "to_id","amount","details"]) and not woi.has_required(
	["from_name","to_name"]):
    # input is from ack1.form, create and send ack2.form (confirmation) 
    confirmation_needed=1 
  elif not woi.has_required(["ac_id","pin","ua","from_id",
      "from_name","to_id","to_name","amount","details"]):
    cgiutils.html_end(session,
       error="Missing value/s. Complete all form fields.")
    return
  # check if has required keys and of correct format
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return 
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="Invalid or stale or logged out session.")
    return
  #
  acks=tablcgi.sq_table(atab,dir=dl,read_locked=True)
  # validate repudiation specific fields and send stage2 form if needed 
  if repudiation:
    # validate repudiate field
    if not woi.firstval("repudiate") == "on":
      cgiutils.html_end(session,error="Invalid repudiation request field.")
      return
    # validate ack_id field
    ack_id=cgiutils.stoi(woi,"ack_id")
    if type(ack_id) == type(None): # true if not an integer
      cgiutils.html_end(session,error="Invalid repudiation ack_id")
      return
    if ack_id not in acks.keys():
      cgiutils.html_end(session,error="repudiation ack_id not found.")
      return
    if -ack_id in acks.keys():
      cgiutils.html_end(session,
         error="repeated repudiation of same ack_id not allowed.")
      return
    if repudiation == 1:
      # send stage 2 form
      ackindex=acks.find(ack_id)
      from_name=acks.data[ackindex]["from_name"] 
      to_name=acks.data[ackindex]["to_name"]
      from_id=acks.data[ackindex]["from_id"] 
      to_id=acks.data[ackindex]["to_id"]
      system=acks.data[ackindex]["system"]
      details="REPUDIATION ack_id: "+str(ack_id)
      string_amount=str(acks.data[ackindex]["amount"])
      # send the ack2.form
      print(cgiutils.send_form("repud2.form",[v['registry'],ac_id,
        cookie,ua,str(ack_id),to_name,to_id,
        from_name,from_id,string_amount,details]))
      cgiutils.html_end(session,want_form=1)
      return
  
  # validate other fields
  from_id=woi.firstval("from_id")
  if not cgiutils.is_valid(from_id,max_leng=8,allowed='^[\w]+$'):
    cgiutils.html_end(session,error="Invalid from account id/no.")
    return
  members=tablcgi.table(mtab,dir=dl)
  fmember=from_id in members.keys()
  if not fmember:
    cgiutils.html_end(session,error="From Account no./id not found.")
    return
  balance=tablcgi.table(btab,dir=dl)
  if from_id not in balance.keys():
    error="""from a/c has member record but no balance record. 
      please inform webmaster."""
    cgiutils.html_end(session,error)
    return
  if (who_can_ack == 'ac_can_pay' and 
           (ua != "admin" ) and 
           from_id != ac_id ):
    cgiutils.html_end(session,
       error="You can only pay out of your own account.")
    return
  to_id=woi.firstval("to_id")
  findex=members.find(from_id)
  if members.data[findex]["active"] == 'N':
    cgiutils.html_end(session,error="From Account is inactive.")
    return
  if not cgiutils.is_valid(to_id,max_leng=8,allowed='^[\w]+$'):
    cgiutils.html_end(session,error="Invalid to account id/no.")
    return
  tmember=to_id in members.keys()
  if not tmember:
    cgiutils.html_end(session,error="To Account no./id not found.")
    return
  tindex=members.find(to_id)
  if from_id == to_id:
    cgiutils.html_end(session,error="You cant pay yourself.")
    return
  if members.data[tindex]["active"] == 'N':
    cgiutils.html_end(session,error="To Account is inactive.")
    return
  if to_id not in balance.keys():
    error="""To a/c has member record but no balance record. 
      please inform webmaster."""
    cgiutils.html_end(session,error)
    return
  amount=cgiutils.stof(woi,"amount")
  if type(amount) == type(None):
    cgiutils.html_end(session,error="Invalid amount, not numeric.")
    return
  if amount < 0.01 or amount > 1000.0:
    cgiutils.html_end(session,
       error="Invalid amount, zero, negative or too large.")
    return
  details=woi.firstval("details")
  if not cgiutils.is_valid(details,max_leng=60,
    allowed=r'''^[\w \'\-\.\,\!\"\'\$\%\^\&\*\(\)\+\=\;\:\?\/\\\@\|]+$'''):
    cgiutils.html_end(session,error="Invalid details.")
    return
  if confirmation_needed:
    # send ack2 confirmation form with names filled in
    from_name=members.data[findex]["name"] 
    to_name=members.data[tindex]["name"]
    string_amount=woi.firstval("amount") # need string not float type
    # send the ack2.form
    print(cgiutils.send_form("ack2.form",[v['registry'],
      ac_id,cookie,ua,to_name,to_id,
      from_name,from_id,string_amount,details]))
    cgiutils.html_end(session,want_form=1)
    return
  from_name=woi.firstval("from_name")
  if not cgiutils.is_valid(from_name,max_leng=60,
      allowed='^[A-Za-z][\w\'\- ]+$'):
    cgiutils.html_end(session,error="Invalid From name.")
    return
  to_name=woi.firstval("to_name")
  if not cgiutils.is_valid(to_name,max_leng=60,
      allowed='^[A-Za-z][\w\'\- ]+$'):
    cgiutils.html_end(session,error="Invalid To name.")
    return
  import time
  acktime=int(time.time())
  if not repudiation:
    try:
      ack_id=acks.get_nextnum()
    except ValueError :
      error="""Couldn't obtain next ack_id. File locking problem ? 
       Try again in a few minutes and if you have the same problem 
       repeatedly please inform the webmaster."""
      cgiutils.html_end(session,error)
      return
  else:
    ack_id = ack_id * -1   # prefix stored repudiation ack_id with - minus
  if not do_ack(session,
           ac_id,balance,acks,repudiation,
           ack_id,from_id,from_name,to_id,to_name,
           amount,details,acktime) : return 
  tvars=get_tithe_vars()
  if ( tvars['tithes'] in ['y','Y'] and      # Tithes must be set.
       ( not repudiation ) and               # Dont do this here. 
       (not to_id in tvars['untithed_to_acs'] ) and 
       (not from_id in tvars['untithed_from_acs'] )
     ) : # Some recipient and some from a/c are untithed e.g. a charity a/c.
    if not do_tithe(session,tvars,ac_id,balance,acks,repudiation,
         ack_id,from_id,from_name,to_id,to_name,
         amount,details,acktime) :
      error="""Acknowledgement: %s input but automatic tithe failed.
            Please inform webmaster.""" % str(ack_id) 
      cgiutils.html_end(session,error)
      return
  if cgiutils.htmloutput:
    print("<p> Acknowledgment: "+str(ack_id)+" successfully input.</p>") 
  else:
    print("Ack id: "+str(ack_id)+" successfully input", end=' ') 
  cgiutils.html_end(session,received=1)
  return 

def do_ack(session,ac_id,balance,acks,repudiation,
         ack_id,from_id,from_name,to_id,to_name,
         amount,details,acktime):
  ''' Perform the acknowledgement/payment. For auto tithed ack, this function
      will be called twice, firstly for the main transaction and secondly
      for the tithe . This function returns True or False depending upon
      whether payment was successful.''' 
  arow={"ack_id":ack_id,
        "from_id":from_id,
        "from_name":from_name,
        "to_id":to_id,
        "to_name":to_name,
        "amount":amount,
        "system":"main",
        "details":details,
        "time":acktime}
  # audit details should have human readable date/time
  auditrow=arow.copy()
  auditrow['time']=conv_datetime(arow['time']) 
  # transfer payment between accounts and add ack record to journal
  tindex=balance.find(to_id)
  findex=balance.find(from_id)
  if not repudiation:
    if who_can_ack == 'ac_can_pay' and ac_id == from_id : 
      # we don't need to authorise someone paying out of their own 
      # account further, so just audit log it.  
      cgiutils.auth(session,
            ac_id,"PAY",str(auditrow),whom=from_id,auditonly=True)
    elif who_can_ack == 'all_can_record' : 
      # we don't need to authorise this as with this setting anything goes
      # at least we can log who did what to whom 
      cgiutils.auth(session,
          ac_id,"PAY",str(auditrow),whom=from_id,auditonly=True)
    elif not cgiutils.auth(session,
          ac_id,"PAY",str(auditrow),whom=from_id):
      # else we have to check attempt to pay is authorised and
      # stop it if not
      return False
    funds=balance.data[findex]["balance"]
    limits=balance.data[findex]["limit"]
    if limits == 'None':
      haslimit=False
    else:
      try:
        flimit=float(limits)
      except: # should never occur if input validation correct
        cgiutils.html_end(session,
           error="Invalid limit stored data. Payment failed.")
        return False
      if funds - amount < flimit:
        # limited account with not enough funds to pay
        cgiutils.html_end(session,
           error="Payment blocked due to insufficient funds.")
        return False
    balance.data[findex]["balance"]-=amount
    balance.data[findex]["out"]+=amount
    balance.data[findex]["turnover"]+=amount
    balance.data[tindex]["balance"]+=amount
    balance.data[tindex]["in"]+=amount
    balance.data[tindex]["turnover"]+=amount
  else:
    if not cgiutils.auth(session,ac_id,"REPUD",cgiutils.mystr(auditrow)):
      return False
    balance.data[findex]["balance"]+=amount
    balance.data[findex]["out"]-=amount
    balance.data[findex]["turnover"]-=amount
    balance.data[tindex]["balance"]-=amount
    balance.data[tindex]["in"]-=amount
    balance.data[tindex]["turnover"]-=amount
  balance.save()
  acks.addrow(arow)
  acks.sort(key="ack_id") # puts repudiations in regular order
  return True 

def do_tithe(session,tvars,ac_id,balance,acks,repudiation,
         ack_id,from_id,from_name,to_id,to_name,
         amount,details,acktime):
  # when called, an automatic tithe is due
  input_by=tvars['input_by']
  from_id=to_id
  from_name=to_name
  # obtain tithe ack_id
  try:
    tack_id=acks.get_nextnum()
  except:
    error="""Couldn't obtain tithe ack_id due to locked file. Tithe 
     can not be paid by recipient. Please inform webmaster giving 
     acknowledgement ID if possible.  """
    cgiutils.html_end(session,error)
    return False
  to_id=tvars['to_ac']
  to_name=tvars['to_ac_name']
  # compute tithe amount
  tamount=amount*(float(tvars['percent'])/100.0)
  # round tithe amount, e.g. to nearest penny
  rtamount= ( int(tamount*float(tvars['rounding_divisor']))/
              float(tvars['rounding_divisor']) )
  details='Tithe from ack_id '+str(ack_id) 
  return do_ack(session,
           input_by,balance,acks,repudiation,
           tack_id,from_id,from_name,to_id,to_name,
           rtamount,details,acktime)

def get_tithe_vars():
  ''' returns variables controlling tithing. If you don't
  have tithing, set tithes.cfg tithes variable value to n (for no). ''' 
  tithe_vars=cgiutils.get_vars('tithes.cfg')
  if tithe_vars['tithes'] in ['n','N']: 
    # if tithes not paid in this system, all other tithe_vars are moot
    tithe_vars['to_ac']=False
    tithe_vars['to_ac_name']=False
    tithe_vars['untithed_to_acs']=False
    tithe_vars['untithed_from_acs']=False
    tithe_vars['percent']=False
    tithe_vars['rounding_divisor']=False
    tithe_vars['input_by']=False
  else:
    # listify untithed_to_acs and untithed_from_acs
    tithe_vars['untithed_to_acs']=tithe_vars['untithed_to_acs'].split(',')
    tithe_vars['untithed_from_acs']=tithe_vars['untithed_from_acs'].split(',')
  return tithe_vars

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in ack.cgi main()")
    traceback.print_exc()

