#!/usr/bin/env python

# change next line from 0 to 1 to debug script

debug=0

""" admin.cgi This file handles input from the account administrators
    form used to create/delete/view/mark inactive accounts.
    Also accesses login and audit details.
"""


if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import btab,mtab
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']
sec_vars=cgiutils.get_vars('secure.cfg')

def pnormalise(phone):
  ''' normalises phone number as on database by removing
  spaces and non numerics. this allows spaces in stored
  phone numbers but normalised comparisons to avoid 
  duplicating mobile phone numbers used for SMS authentication. '''
  nphone=''
  for char in phone:
    if '0' <= char <= '9':
      nphone=nphone+char
  return nphone

def get_email(session):
  email=woi.firstval("email")
  if email == '':
    return 'None'
  if not cgiutils.is_email(email):
    if not email.lower() == 'none':
      cgiutils.html_end(session,error="Invalid email address.")
      return False
    else: # normalise case for user with no email
      email='None'
  return email

def get_username(session):
  ''' get and return username or False if invalid '''
  user_name=woi.firstval("user_name")
  if user_name == '':
    return 'None'
  if not cgiutils.is_valid(user_name,max_leng=60,
      allowed='^[A-Za-z][\w\'\- ]+$'):
    cgiutils.html_end(session,error="Invalid name.")
    return False
  return user_name

def get_sbal(session):
  ''' get and return validity and starting balance tuple. 
      Validity and starting balance must be separate 
      values because a starting balance of 0 would be false.'''
  sbal=cgiutils.stof(woi,"start_bal")
  if type(sbal) == type(None): # wasn't valid float
    cgiutils.html_end(session,error="Invalid starting balance.")
    return False,None # i.e. invalid, no value
  if sbal > 1000000.0 or sbal < -1000000.0: # unreasonable balance
    cgiutils.html_end(session,error="starting balance too high or low.")
    return False,None
  return True,sbal #  i.e. valid, starting balance

def get_limit(session):
  ''' gets debit limit which may be string "None" or float. 
  Returns tuple: valid (true or false) 
                 has a limit (true if numeric or false if "None")
                 limit (float or "None" if valid or None if invalid)
  ''' 
  limit=woi.firstval("limit")
  flimit=0.0
  valid=True 
  haslimit=True 
  if limit == "None" or limit=="": # means no automatic limit on payments
    haslimit=False
    return valid,haslimit,limit
  else: 
    flimit=cgiutils.stof(woi,"limit")
    if type(flimit) == type(None): # wasn't valid float
      cgiutils.html_end(session,
         error="Invalid limit. Must be numeric or None .")
      haslimit=False
      valid=False
      return valid,haslimit,None
    if flimit > 1000000.0 or flimit < -1000000.0: # unreasonable limit 
      cgiutils.html_end(session,error="Debit limit is too high or low.")
      haslimit=False
      valid=False
      return valid,haslimit,None
    haslimit=True 
  return valid,haslimit,flimit

def get_phone(session,key):
  '''Returns a phone number, key might be "phone1" or "phone2" '''
  phone=woi.firstval(key)
  if phone == '':
    return 'None' 
  if not cgiutils.is_valid(phone,max_leng=15, allowed='^None$|^[0-9][\w\- ]+$'):
    cgiutils.html_end(session,error="Invalid "+key+".")
    return False
  return phone

def is_phone2_unique(session,phone2):
  ''' returns true if phone2 is unique within members table 
      or false if not ''' 
  nphone2=pnormalise(phone2)
  members=tablcgi.table(mtab,dir=dl)
  for row in members.data:
    if row["phone2"] != "None" and pnormalise(row["phone2"]) == nphone2 :
      owner=row["ac_id"]
      cgiutils.html_end(session,error="phone2 "+phone2+" belongs to "+owner )
      return False
  return True

def get_address(session):
  address=woi.firstval("address")
  if address == '':
    return 'None' 
  if not cgiutils.is_valid(address,max_leng=90,
      allowed='^[A-Za-z0-9][\w\'\-\,\.\r\n ]+$'):
    cgiutils.html_end(session,error="Invalid address.")
    return False
  return address

def get_postcode(session):
  postcode=woi.firstval("postcode")
  if postcode == '':
    return 'None' 
  if not cgiutils.is_valid(postcode,max_leng=10, allowed='^[A-Za-z0-9][\w ]+$'):
    cgiutils.html_end(session,error="Invalid postcode.")
    return False
  return postcode

def get_district(session):
  district=woi.firstval("district")
  if district == '':
    return 'None' 
  if not cgiutils.is_valid(district,max_leng=20, allowed='^[A-Za-z ]+$'):
    cgiutils.html_end(session,
       error="Invalid district use letters a-z and space only.")
    return False
  return district

def send_accept(email,account,pin,option):
  ''' send welcome message to address email on account being added 
  parameters:
  email - email address of applicant
  account - name of account created
  pin - PIN user will need to login
  option - type of membership granted, "provis" or "active"
  '''
  type="unknown"
  if option == "Y":
    type="active"
  elif option == "P":
    type="provisional"
  fromad=v['admin_e']
  regy=v['registry']
  url=v['homepage']
  message="From: "+regy+"  <"+v['admin_e']+">\n"
  to_line="To: %s\n" % email
  message+=to_line
  message+="Subject: "+regy+" application accepted \n\n"
  message+="Welcome to "+regy+". We hope you will enjoy membership.\n"
  message+="Your application to join "+regy+" has been \n"
  message+="accepted. Please keep these details for future reference.\n\n"
  message+="The type of membership granted is %s.\n\n" % type
  message+="Your account identifier is %s .\n\n" % account
  message+="Your personal identification number (PIN) is %s .\n\n" % pin
  message+="The accounts, membership and trading website is accessible\n"
  message+="at the following address:\n\n"+url
  cgiutils.send_mail(fromad,email,message)

def add(session,members,ac_id,user_ac_id):
  if not woi.has_required(["user_name","email","start_bal","limit",
         "phone1","phone2","address","postcode","district"]):
      cgiutils.html_end(session,error="Missing field for add member. Complete all fields.")
      return
  # validate fields
  # email
  email=get_email(session)
  if not email:
      return
  # user name
  user_name=get_username(session)
  if not user_name or user_name == 'None':
      cgiutils.html_end(session,
         error="Adding user with no name or None name is invalid. Can't add account.")
      return
  # check account doesn't exist
  if user_ac_id in members.keys():
      cgiutils.html_end(session,
         error="Account number/id exists. Can't add account.")
      return
  # check account name not in reserved list
  reserved_account_names=v['reserved_account_names'].split(',')
  if user_ac_id in reserved_account_names :
      cgiutils.html_end(session,
         error="Account id requested is a reserved name.")
      return
  # starting balance
  valid,sbal=get_sbal(session)
  if not valid or sbal=='None':
      return
  # limit
  valid,haslimit,limit=get_limit(session)
  if not valid:
      cgiutils.html_end(session,
      error="Invalid account limit.")
      return
  # phone numbers 
  phone1=get_phone(session,"phone1")
  if not phone1:
      return
  phone2=get_phone(session,"phone2")
  if not phone2:
      return
  # If user has a phone2 (not "None") it must be unique
  # within members table to be usable for SMS gateway authentication
  phone2uniq=is_phone2_unique(session,phone2)
  if not phone2uniq:
      return
  # street address, postcode and district (e.g. part of town)
  address=get_address(session)
  if not address:
      return
  postcode=get_postcode(session)
  if not postcode:
      return
  district=get_district(session)
  if not district:
      return
  if "pai" not in woi.keys():
    pai="active"
  else:
    pai=woi.firstval("pai")
  if not pai in ["provisional","active"]:
    cgiutils.html_end(session,
       error="new member status must be active or provisional.")
    return
  #
  # if still here, generate new PIN, create and add row to members
  # and balances tables
  newpin=cgiutils.make_pin(mini=10000000,maxi=99999999)
  row={}
  row["ac_id"]=user_ac_id
  row["name"]=user_name
  row["email"]=email
  row["pin"]=newpin
  if pai == 'active':
    row["active"]='Y'
  else:
    row["active"]='P'
  row["phone1"]=phone1
  row["phone2"]=phone2
  row["address"]=address
  row["postcode"]=postcode
  row["district"]=district
  row["url"]='None' # URL only configurable using mdetails.cgi
  row["pic"]='None' # URL only configurable using mdetails.cgi
  brow={}
  brow["ac_id"]=user_ac_id
  brow["balance"]=sbal
  brow["cfwd"]=sbal
  brow["in"]=0.0
  brow["out"]=0.0
  brow["turnover"]=0.0
  brow["oldturn"]=0.0
  # limit of None or string value of floating point
  if haslimit :
      brow["limit"]="%.2f" % limit
  else:
      brow["limit"]="None"
  balance=tablcgi.table(btab,dir=dl)
  if not cgiutils.auth(session,
            ac_id,"MEMBADD",str(row)+str(brow),whom=user_ac_id):
    return
  members.addrow(row)
  balance.addrow(brow)
  if not email == 'None': 
    # send an acceptance email message to the new member 
    send_accept(email,user_ac_id,newpin,'Y') 
  print("<p>new member added")
  cgiutils.html_end(session,received=1)
  return

def modify(session,members,ac_id,user_ac_id):
  modifications=0 # counts number of fields changed
  # check account does exist
  if user_ac_id not in members.keys():
      error="Account id/no. doesn't exist. Can't modify."
      cgiutils.html_end(session,error)
      return
  mindex=members.find(user_ac_id)
  maudetails='' # member audit details string
  baudetails='' # balance audit details string
  balance=tablcgi.table(btab,dir=dl)
  bindex=balance.find(user_ac_id)
  # check email address
  if "email" in woi.keys():
      email=get_email(session)
      if not email: 
        return # false if invalid
      if not email == 'None': 
        members.data[mindex]["email"]=email
        modifications+=1
        maudetails+="email: "+email+" "
  # check user name 
  if "user_name" in woi.keys():
      # validate user name 
      user_name=get_username(session)
      if not user_name: 
        return # false if invalid
      if not user_name == 'None': 
        members.data[mindex]["name"]=user_name
        modifications+=1
        maudetails+="user_name: "+user_name+" "
  # validate phone 1 
  if "phone1" in woi.keys():
      phone1=get_phone(session,"phone1")
      if not phone1:
        return
      if not phone1 == 'None': 
        members.data[mindex]["phone1"]=phone1
        modifications+=1
        maudetails+="phone1: "+phone1+" "
  if "phone2" in woi.keys():
      phone2=get_phone(session,"phone2")
      if not phone2:
        return
      phone2uniq=is_phone2_unique(session,phone2)
      if not phone2uniq:
        return
      if not phone2 == 'None': 
        members.data[mindex]["phone2"]=phone2
        modifications+=1
        maudetails+="phone2: "+phone2+" "
  if "address" in woi.keys(): 
      # validate field
      address=get_address(session)
      if not address:
        return
      if not address == 'None': 
        members.data[mindex]["address"]=address
        modifications+=1
        maudetails+="address: "+address+" "
  if "postcode" in woi.keys(): 
      postcode=get_postcode(session)
      if not postcode:
        return
      if not postcode == 'None': 
        members.data[mindex]["postcode"]=postcode
        modifications+=1
        maudetails+="postcode: "+postcode+" "
  if "district" in woi.keys(): 
      district=get_district(session)
      if not district:
        return
      if not district == 'None': 
        members.data[mindex]["district"]=district
        modifications+=1
        maudetails+="district: "+district+" "
  #
  if modifications: # check member record modification authorised 
      if not cgiutils.auth(session,ac_id,"MEMBMOD",
           maudetails, whom=user_ac_id):
        return
  # count balance table modifications
  bmodifications=0
  # check CFWD or start balance
  keyl=woi.keys()
  if "start_bal" in keyl or "limit" in keyl:
      balance=tablcgi.table(btab,dir=dl)
  if "start_bal" in keyl and woi.firstval("start_bal") != "":
      valid,sbal=get_sbal(session)
      if not valid:
        return
      old_cfwd=balance.data[bindex]["cfwd"]
      difference=sbal-old_cfwd
      balance.data[bindex]["cfwd"]=sbal
      balance.data[bindex]["balance"]+=difference
      baudetails=' adjusting starting balance by '+str(difference)+' '
      if not cgiutils.auth(session,ac_id,"SBALMOD",baudetails,
                            whom=user_ac_id):
        return
      bmodifications+=1
  if "limit" in keyl and woi.firstval("limit") != "":
      valid,haslimit,limit=get_limit(session)
      if not valid:
        return
      # limit of None or string value of floating point
      if haslimit:
        slimit="%.2f" % limit
      else:
        slimit="None"
      oldlimit=balance.data[bindex]["limit"]
      newlimit=slimit
      balance.data[bindex]["limit"]=slimit
      baudetails='changing limit from '+str(oldlimit)+' to '+str(newlimit) 
      if not cgiutils.auth(session,ac_id,"LIMITMOD",baudetails,
                            whom=user_ac_id):
        return
      bmodifications+=1
  # if we have got this far, any changes are validated
  if modifications: # save changes to members table
      members.save()
  if bmodifications: # save changes to balances table
      balance.save()
      modifications+=bmodifications # total number of fields modified
  if not modifications: # all changeable fields were blank 
      cgiutils.html_end(session,error="No modified field values submitted.")
      return
  else: 
      print("<p> %d field/s modified.</p>" % modifications)
      cgiutils.html_end(session,received=1)
      return

def view(session,members,ac_id,user_ac_id):
  if user_ac_id not in members.keys():
      error="Account id/no. doesn't exist. Can't view it."
      cgiutils.html_end(session,error)
      return
  mindex=members.find(user_ac_id)
  balance=tablcgi.table(btab,dir=dl)
  if user_ac_id not in balance.keys():
      error="""Account id/no. has member record but no balance record.
        please inform webmaster."""
      cgiutils.html_end(session,error)
      return
  bindex=balance.find(user_ac_id)
  bal=balance.data[bindex]["balance"]
  # create members table view object
  mtabvo=tablcgi.table_def(mtab)
  mtabvo.file=None
  mtabvo.meta["keys"].append("cfwd")
  mtabvo.meta["keys"].append("balance")
  mtabvo.meta["keys"].append("in")
  mtabvo.meta["keys"].append("out")
  mtabvo.meta["keys"].append("turnover")
  mtabvo.meta["keys"].append("limit")
  mtabvo.meta["colheads"].append("CFWD")
  mtabvo.meta["colheads"].append("Balance")
  mtabvo.meta["colheads"].append("In")
  mtabvo.meta["colheads"].append("Out")
  mtabvo.meta["colheads"].append("Turnover")
  mtabvo.meta["colheads"].append("Limit")
  mtabvo.meta["formats"].append("%.2f")
  mtabvo.meta["formats"].append("%.2f")
  mtabvo.meta["formats"].append("%.2f")
  mtabvo.meta["formats"].append("%.2f")
  mtabvo.meta["formats"].append("%.2f")
  mtabvo.meta["formats"].append("%s")
  mtv=tablcgi.table(mtabvo)
  row={}
  for key in ["ac_id","pin","name","email","active","phone1","phone2",
              "address","postcode","district","url","pic"]:
      row[key]=members.data[mindex][key]
  for key in ["cfwd","balance","in","out","turnover","limit"]:
      row[key]=balance.data[bindex][key]
  mtv.addrow(row)
  if not cgiutils.auth(session,ac_id,"MEMBVIEW",user_ac_id,whom=user_ac_id):
      return
  mtv.tab2html(skip_cols=["pin"])
  cgiutils.html_end(session)
  return

def delete(session,members,ac_id,user_ac_id):
    if user_ac_id not in members.keys():
      error="Account id/no. doesn't exist. Can't delete."
      cgiutils.html_end(session,error)
      return
    mindex=members.find(user_ac_id)
    balance=tablcgi.table(btab,dir=dl)
    if user_ac_id not in balance.keys():
      error="""Account id/no. has member record but no balance record.
        please inform webmaster."""
      cgiutils.html_end(session,error)
      return
    bindex=balance.find(user_ac_id)
    bal=balance.data[bindex]["balance"]
    # testing for exact zero on a floating point system where amounts
    # are only displayed to the nearest penny could lead to odd situations.
    # So we only test for an approximate zero here. 
    if bal > 0.009 or bal < -0.009:
      error="Account has non-zero balance. Unwise to delete it. Clear it first."
      cgiutils.html_end(session,error)
      return
    if not cgiutils.auth(session,ac_id,"MEMBDEL",user_ac_id,whom=user_ac_id):
      return
    # get rid of photo for member if one exists
    picfile=members.data[mindex]['pic']
    if picfile == 'None':
      pass # null value for field
    else:
      # unlink file for this member if it exists
      import os, os.path
      if os.path.isfile(picfile):
        os.unlink(picfile)
    #
    balance.delrow(user_ac_id)
    members.delrow(user_ac_id)
    cgiutils.html_end(session,received=1)
    return

def change_status(session,members,ac_id,user_ac_id,pai):
  # check account does exist
  if user_ac_id not in members.keys():
      error="Account id/no. doesn't exist. Can't make "+pai+"."
      cgiutils.html_end(session,error)
      return
  mindex=members.find(user_ac_id)
  paicodes={'active':'Y','inactive':'N','provisional':'P'}
  if not cgiutils.auth(session,ac_id,"MEMBSTATUS",str(members.data[mindex]),
                          whom=user_ac_id):
      return
  # if changing status to provisional, may need to remove photo/url
  # if changing status to inactive must remove photo/url
  newstatus=paicodes[pai]
  if ( ( v['provisional_can_photo'].upper() == 'N' and newstatus == 'P' ) or
       ( newstatus == 'N' ) ) :
    picfile=members.data[mindex]['pic']
    if picfile == 'None':
      pass # null value for field
    else:
      # unlink old file for this member if it exists
      import os, os.path
      if os.path.isfile(picfile):
        os.unlink(picfile)
    members.data[mindex]['pic']='None'
  if ( ( v['provisional_can_url'].upper() == 'N' and newstatus == 'P' ) or
       ( newstatus == 'N' ) ) :
    url=members.data[mindex]['url']
    if url == 'None':
      pass # null value for field
    else:
      # nullify url field
      members.data[mindex]['url']='None'
  # set member status 
  members.data[mindex]["active"]=paicodes[pai]
  members.save()
  cgiutils.html_end(session,received=1)
  return

def secure(session,ac_id,cookie,ua):
  if not cgiutils.auth(session,ac_id,"SECURITY",
        "view security form",whom=ac_id):
      return
  # create logfile select statement
  import authent
  logfiles=authent.list_logfiles("logins") # gets list of current logfiles
  html_select=cgiutils.make_file_select(logfiles,
                         instruction="choose current file or archive",
                         cgiparameter="logfilename") 
  # send the security form
  print(cgiutils.send_form("security.form",[v['registry'],ac_id,cookie,
      ua,html_select]))

def auth(session,ac_id,cookie,ua):
  if not cgiutils.auth(session,ac_id,"AUTHAUDIT",
         "view authorisation/audit form", whom=ac_id):
    return
  # create audit file select statement
  import authent
  logfiles=authent.list_logfiles("aactions") # gets list of auditfiles
  html_select=cgiutils.make_file_select(logfiles,
                         instruction="choose current file or archive",
                         cgiparameter="auditfilename") 
  # send the authorisation form
  print(cgiutils.send_form("authorisation.form",[v['registry'],ac_id,
    cookie,ua,html_select]))
  return

def yearend(session,ac_id,cookie,ua):
  if not cgiutils.auth(session,ac_id,"YEAREND",
       "view year end and writeoffs form", whom=ac_id):
    return
  # send the yearend form
  print(cgiutils.send_form("yearend.form",[v['registry'],ac_id, cookie,ua]))
  return

def approve(session,ac_id,cookie,ua):
  if not cgiutils.auth(session,ac_id,"ACCEPTREJECT",
        "accept or reject applications", whom=ac_id):
    return
  # create application selection statement to insert in approval form
  from table_details import apptab, cattab
  applications=tablcgi.table(apptab,dir=dl)
  # clone a auditfilenames categories type table 
  emailopts=tablcgi.table_def(cattab)
  emailopts.file=None # in memory only - don't save
  eopts=tablcgi.table(emailopts)
  i=0
  for arow in applications.data:
    if arow['confirmed']=='Y': # can only approve or reject once confirmed
      email=arow['email']
      fullname=arow['name']
      i+=1
      frow={"sort":i,"category":email,"desc":fullname+' &lt;'+email+'&gt;'}
      eopts.addrow(frow)
  html_select=cgiutils.make_select(eopts,
              instruction="choose application to process",
              cgiparameter="email") 

  # send the approval form
  print(cgiutils.send_form("approval.form",[v['registry'],ac_id,
    cookie,ua,html_select]))
  return

def send_newpin(email,newpin):
  ''' send new pin to email address 
  parameters:
  email - email address of applicant
  pin - PIN user will need to login
  '''
  fromad=v['admin_e']
  regy=v['registry']
  url=v['homepage']
  message="From: "+regy+"  <"+v['admin_e']+">\n"
  to_line="To: %s\n" % email
  message+=to_line
  message+="Subject: "+regy+" your PIN has been reset \n\n"
  message+="Your new personal identification number (PIN) is %s .\n\n" % newpin
  message+="The accounts, membership and trading website is accessible\n"
  message+="at the following address:\n\n"+url
  cgiutils.send_mail(fromad,email,message)

def reset_pin(session,members,ac_id,user_ac_id):
  ''' resets the user PIN and emails new PIN to address
  for the user '''
  if user_ac_id not in members.keys():
    error="Account id/no. doesn't exist. Can't reset PIN."
    cgiutils.html_end(session,error)
    return
  if not cgiutils.auth(session,ac_id,"PINMOD",user_ac_id,whom=user_ac_id):
    return
  mindex=members.find(user_ac_id)
  email=members.data[mindex]['email']
  newpin=cgiutils.make_pin(mini=10000000,maxi=99999999)
  members.data[mindex]['pin']=newpin
  members.save()
  if cgiutils.is_email(email):
    send_newpin(email,newpin)
    print("<p> PIN emailed for account: %s to address: %s </p>" % (user_ac_id,
         email))  
  cgiutils.html_end(session,received=1)
  return

def main():
  session=False
  cgiutils.html_header(title="LETS Accounting administration",
                       bgcolor='"#FFFFFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  valid,option=cgiutils.get_option(woi,buttons=["add","modify",
               "status","delete","view","rpin","secure",
               "auth","approve","yearend" ])
  if not valid:
    return # error message already sent
  if (option not in ["secure","auth","approve","yearend"] and not 
     woi.has_required(["user_ac_id"])):
    cgiutils.html_end(session,error="Missing user account identifier.")
    return

  # check if has required keys and of correct format
  user_ac_id='none' # default value - not needed to access security form 
  if option not in ["secure","auth","approve","yearend"]:
    user_ac_id=woi.firstval("user_ac_id").lower() # lower case ac_id's
  # check user account no./id
  if not cgiutils.is_valid(user_ac_id,max_leng=8,allowed='^[\w]+$'):
    cgiutils.html_end(session,error="Invalid user account id/no.")
    return
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid or stale or logged out session.")
    return
  # 
  if ua != 'admin':
    cgiutils.html_end(session,error="You must login as admin to do this.")
    return

  # If we've got this far, there must be a valid option and PIN OK
  members=tablcgi.table(mtab,dir=dl)
  if option == "approve": # send application approval/rejection form 
    approve(session,ac_id,cookie,ua)
    return
  elif option == "add": # account addition 
    add(session,members,ac_id,user_ac_id)
    return
  elif option == "modify": # modify details for account
    modify(session,members,ac_id,user_ac_id)
    return 
  elif option == "status":
    if "pai" not in woi.keys():
      cgiutils.html_end(session,
         error="no provisional/active/inactive selection.")
      return
    pai=woi.firstval("pai")
    if not pai in ["provisional","active","inactive"]:
      cgiutils.html_end(session,
         error="invalid provisional/active/inactive selection.")
      return
    change_status(session,members,ac_id,user_ac_id,pai)
    return 
  elif option == "delete":
    delete(session,members,ac_id,user_ac_id)
    return
  elif option == "view":
    view(session,members,ac_id,user_ac_id)
    return  
  elif option == "rpin":
    reset_pin(session,members,ac_id,user_ac_id)
    return  
  elif option == "secure": # send view security details form
    secure(session,ac_id,cookie,ua)
    return
  elif option == "auth": # send authorisation/audit form 
    auth(session,ac_id,cookie,ua)
    return
  elif option == "yearend": # send yearend and writeoff form 
    yearend(session,ac_id,cookie,ua)
    return

if __name__ == "__main__":
  try:
      main()
  except:
    import traceback
    print("error detected in admin.cgi main()")
    traceback.print_exc()

