#!/usr/bin/env python

# change next line from 0 to 1 to debug script

debug=0

""" application.cgi This file handles input from the application 
    form used to input data for a prospective account by the applicant .
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import apptab,mtab
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']
newm_cfgs=cgiutils.get_vars('newmember.cfg')

def pnormalise(phone):
  ''' normalises phone number as on database by removing
  spaces and non numerics. this allows spaces in stored
  phone numbers but normalised comparisons to avoid 
  duplicating mobile phone numbers used for SMS authentication. '''
  nphone=''
  for char in phone:
    if '0' <= char <= '9':
      nphone=nphone+char
  return nphone

def get_email(session,which):
  ''' which is which email field "email1" '''
  email=woi.firstval(which)
  if not cgiutils.is_email(email):
    cgiutils.html_end(session,error="Invalid email address.")
    return False
  return email.lower()

def unique_email(email):
  ''' returns True if email is not already on system or 
  False if it is already in use by another acccount '''
  members=tablcgi.table(mtab,dir=dl)
  for row in members.data:
    if row["email"] != "None" and row["email"].lower() == email.lower() :
      return False
  return True 

def get_username(session):
  ''' get and return username or False if invalid '''
  user_name=woi.firstval("user_name")
  if not cgiutils.is_valid(user_name,max_leng=60,
      allowed='^[A-Za-z][\w\'\- ]+$'):
    cgiutils.html_end(session,error="Invalid name.")
    return False
  return user_name

def get_phone(session,key):
  '''Returns a phone number, key might be "phone1" or "phone2" '''
  phone=woi.firstval(key)
  if not cgiutils.is_valid(phone,max_leng=15, allowed='^None$|^[0-9][\w\- ]+$'):
    cgiutils.html_end(session,error="Invalid "+key+".")
    return False
  return phone

def is_phone2_unique(session,phone2):
  ''' returns true if phone2 is unique within members table 
      or false if not ''' 
  nphone2=pnormalise(phone2)
  members=tablcgi.table(mtab,dir=dl)
  for row in members.data:
    if row["phone2"] != "None" and pnormalise(row["phone2"]) == nphone2 :
      owner=row["ac_id"]
      error1="Mobile phone number not allowed if this appears to belong\n"
      error2="to an existing account.\n"
      error3="phone2 "+phone2+" belongs to "+owner 
      cgiutils.html_end(session,error=error1+error2+error3)
      return False
  return True

def get_address(session):
  address=woi.firstval("address")
  if not cgiutils.is_valid(address,max_leng=90,
      allowed='^[A-Za-z0-9][\w\'\-\,\.\r\n ]+$'):
    cgiutils.html_end(session,error="Invalid address.")
    return False
  return address

def get_other(session):
  if "other" not in woi.keys():
    return True,'' # OK to leave blank
  other=woi.firstval("other")
  if not cgiutils.is_valid(other,max_leng=1200,
      allowed='^[A-Za-z0-9][\w\'\-\,\.\/\:\"\(\)\@\&\;\*\%\=\+\!\?\~\#\r\n ]+$'):
    cgiutils.html_end(session, error=
      "Invalid punctuation in additional information, or over 1200 chars.")
    return False,''
  return True,other

def get_postcode(session):
  postcode=woi.firstval("postcode")
  if not cgiutils.is_valid(postcode,max_leng=10, allowed='^[A-Za-z0-9][\w ]+$'):
    cgiutils.html_end(session,error="Invalid postcode.")
    return False
  return postcode

def get_district(session):
  district=woi.firstval("district")
  if not cgiutils.is_valid(district,max_leng=20, allowed='^[A-Za-z ]+$'):
    cgiutils.html_end(session,
       error="Invalid district use letters a-z and space only.")
    return False
  return district

def get_handle(session):
  if "handle" not in woi.keys():
    return True,"" # valid if no input
  handle=woi.firstval("handle")
  if not cgiutils.is_valid(handle,max_leng=40, allowed='^[A-Za-z0-9_ ]+$'):
    cgiutils.html_end(session,
      error="Invalid handle preferences use characters a-z,0-9 and underscore.")
    return False,"" # invalid if contains banned chars
  return True,handle 

def add(session):
  members=tablcgi.table(mtab,dir=dl)
  applications=tablcgi.table(apptab,dir=dl)
  if not woi.has_required(["user_name","email1",
         "phone1","phone2","address","postcode","district"]):
      cgiutils.html_end(session,
         error="Missing some input. Please complete all fields marked *")
      return
  # validate fields
  # email
  valid,handle=get_handle(session) # valid handle can be "" empty 
  if not valid:
      return
  email1=get_email(session,'email1')
  if not email1:
      return
  if applications.find(email1) != -1:
      errmes=''' 
    An application has already been filed for the email address which
    you entered. 2 applications cannot be filed for one email address. If you
    filed the earlier application in error please contact: '''+v['admin_e']
      cgiutils.html_end(session,error=errmes)
      return
  # check and process applicants_must_have_own_email setting 
  if newm_cfgs["applicants_must_have_own_email"] == 'Y':
    if not unique_email(email1):
      errmes=''' 
        An account with the same email address exists on the
        system. Reusing this address using this web application 
        form conflicts with system policy. You will need either to
        obtain and use your own email address or use another application
        procedure, e.g. involving identification of your system support 
        requirements and provision for you to continue.
      '''         
      cgiutils.html_end(session,error=errmes)
      return
  # user name
  user_name=get_username(session)
  if not user_name:
      return
  # phone numbers 
  phone1=get_phone(session,"phone1")
  if not phone1:
      return
  phone2=get_phone(session,"phone2")
  if not phone2:
      return
  # If user has a phone2 (not "None") it must be unique
  # within members table to be usable for SMS gateway authentication
  phone2uniq=is_phone2_unique(session,phone2)
  if not phone2uniq:
      return
  # street address, postcode and district (e.g. part of town)
  address=get_address(session)
  if not address:
      return
  postcode=get_postcode(session)
  if not postcode:
      return
  district=get_district(session)
  if not district:
      return
  valid,other=get_other(session)
  if not valid:
      return
  #
  # if still here, generate new PIN, create and add row to members
  # and balances tables
  token=cgiutils.make_pin(mini=1000000000,maxi=9999999999)
  row={}
  row["token"]=token
  row["confirmed"]='N' # first stage, email needs confirming using return of token
  row["handle"]=handle
  row["name"]=user_name
  row["email"]=email1
  row["phone1"]=phone1
  row["phone2"]=phone2
  row["address"]=address
  row["postcode"]=postcode
  row["district"]=district
  row["other"]=other
  import time
  apptime=int(time.time())
  row["time"]=apptime 
  applications.addrow(row)
  applications.save()
  try: 
    send_token(token,email1) # send the security token
    thankmess='''Thanks for applying. Your details are filed and a 
      confirmation code has been emailed.  When you have received 
      this code in your email please use this to confirm the account 
      application using the stage 2 form below.<p> '''
  except:
    # probably due to the email address of applicant being syntactically
    # correct but undeliverable, resulting in a SMTPrefused error
    errmes=''' An attempt was made to send a confirmation code to the email address
              which you provided, but this appears to be undeliverable. 
              Please confirm that the address supplied is deliverable to you. ''' 
    cgiutils.html_end(session,error=errmes)
    return
  print(cgiutils.send_form("confirmation.form",[v['registry'],
                                          v['membership_agreement_url'],
                                          email1 ] )) 
  cgiutils.html_end(session,received=0,want_form=1) 
  return

def send_token(token,email):
  fromad=v['admin_e']
  message="From: "+v['registry']+" Accounts <"+v['admin_e']+">\n"
  to_line="To: %s\n" % email
  message+=to_line
  message+="Subject: "+v['registry']+" application\n\n"
  message+="Someone (hopefully you) submitted an online application\n"
  message+="with your email address.\n"
  message+="\n"
  pin_line="The code needed for confirmation is: %s\n" % token 
  message+=pin_line
  message+="\n"
  message+="The confirmation form is linked from this page: \n"
  message+=v['homepage']+" \n"
  message+="A different PIN will be issued assuming your application\n"
  message+="is approved, so you will need to use this code once only.\n" 
  message+="\n"
  message+="If this message is in error please ignore it. However,\n"
  message+="if this error persists please contact "+v['abuse']+" .\n"
  message+="\n"
  cgiutils.send_mail(fromad,email,message)

def accept(session):
  if not woi.has_required(["token","email","read","accept"]):
    cgiutils.html_end(session,
      error="Missing fields for accept agreement. Complete all fields.")
    return
  applications=tablcgi.table(apptab,dir=dl)
  email=get_email(session,"email")
  if not email:
    return
  appidx=applications.find(email)
  if appidx == -1:
    cgiutils.html_end(session,
       error="Application for email address input is not on file")
    return
  if 'token' not in woi.keys():
    cgiutils.html_end(session,error="missing token")
    return
  token=woi.firstval("token")
  if not cgiutils.is_valid(token,max_leng=10,
      allowed='^[0-9]+$'):
    cgiutils.html_end(session,error="Invalid token, must have 10 numerals")
    return
  tokensent=applications.data[appidx]["token"]
  if int(token) != int(tokensent):
    errmes="You must input the token sent to your email to continue."
    cgiutils.html_end(session,error=errmes)
    return
  reads=woi.firstval("read")
  if not cgiutils.is_valid(reads, allowed='yes'):
    cgiutils.html_end(session,
       error="You must read the agreement before accepting it")
    return
  applications.data[appidx]["confirmed"]='Y'
  applications.save() 
  print("Your application has been input for consideration by")
  print("the membership administrator. You will receive an email")
  print("when your application is accepted or refused.\n")
  print("Thank you for your application.")
  cgiutils.html_end(session,received=1) 
  return  

def main():
  session=False
  system_name=v['registry']
  cgiutils.html_header(title=system_name+" Account Application")
  if len(list(woi.keys())) == 0:
    # if no keys send the form to the browser
    handle_option=cgiutils.get_handleopt()
    print(cgiutils.send_form("application.form",[system_name,handle_option]))
    cgiutils.html_end(session,want_form=1)
    return
  # which submit button was pressed ?
  option="none"
  for word in ["submit","accept","reject","confirm"]:
    if word in woi.keys():
      option=word
  if option=="none":
    cgiutils.html_header(title="PyLETS Account Application")
    cgiutils.html_end(session,error="Invalid submit button value.")
    return
  if option == "confirm":
    # send confirmation form
    print(cgiutils.send_form("confirmation.form",
      [v['registry'],v['membership_agreement_url'],''] )) 
    cgiutils.html_end(session,received=0,want_form=1) 
    return
  if option == "submit":
    # some form input so check the form and add if possible
    add(session)
    return
  elif option == "reject":
    print("You did not agree to the membership agreement.")
    print("</body></html>")
    return
  elif option == "accept":
    accept(session)
    return
  else:
    cgiutils.html_end(session,error="No valid option detected.")
    return
  return

if __name__ == "__main__":
  try:
    main()
  except:
    import traceback
    print("error detected in application.cgi main()")
    traceback.print_exc()

