#!/usr/bin/env python

# change next line from 0 to 1 to debug script

debug=0

""" approval.cgi This program accepts or rejects pending
    applications. 
"""


if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import btab,mtab,apptab
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']
sec_vars=cgiutils.get_vars('secure.cfg')
nm_vars=cgiutils.get_vars('newmember.cfg')

def expire_unconfirmed(applications):
  ''' expires applications left unconfirmed longer than
  period in hours set in newmember.cfg '''
  import time
  timenow=time.time()
  delrows=[] # list of keys to be deleted from applications table
  # confirmsecs is the number of seconds allowed for confirmation  
  confirmsecs=int(nm_vars["unconfirmed_application_deletion_period"])*3600 
  for row in applications.data:
    if row["confirmed"] == 'N' : 
      whenapplied=row["time"]
      if whenapplied+confirmsecs < timenow :
        delrows.append(row["email"])
  for email in delrows:     
    applications.delrow(email)
  applications.save()

def pnormalise(phone):
  ''' normalises phone number as on database by removing
  spaces and non numerics. this allows spaces in stored
  phone numbers but normalised comparisons to avoid 
  duplicating mobile phone numbers used for SMS authentication. '''
  nphone=''
  for char in phone:
    if '0' <= char <= '9':
      nphone=nphone+char
  return nphone

def get_email(session):
  email=woi.firstval("email")
  if not cgiutils.is_email(email):
    cgiutils.html_end(session,error="Invalid email address.")
    return False
  return email

def get_account(session):
  ''' get and return account or False if invalid 
      accounts must use numbers and lowercase letters. '''
  account=woi.firstval("account")
  if nm_vars['system_has_numeric_accounts'] == 'Y':
    if not cgiutils.is_valid(account,max_leng=5,allowed='^[0-9]+$'):
      cgiutils.html_end(session,error=
        "On this system account identifiers are numbers.")
      return False
  else:
    if not cgiutils.is_valid(account,max_leng=8,allowed='^[a-z0-9]+$'):
      cgiutils.html_end(session,error="Invalid account identifier.")
      return False
  # check account name not in reserved list
  reserved_account_names=v['reserved_account_names'].split(',')
  if account in reserved_account_names :
    cgiutils.html_end(session,error="Account id requested is a reserved name.")
    return False
  return account 

def get_reason(session):
  reason=woi.firstval("reason")
  if not cgiutils.is_valid(reason,max_leng=1500,
      allowed='^[A-Za-z0-9][\w\'\-\,\.\r\n ]+$'):
    cgiutils.html_end(session,
       error='''Invalid character in rejection reason.
      allowed chars are letters A-Za-z, numbers 0-9, single 
      quotes ('), commas(,) stops (.) and newlines.''')
    return False
  return reason 

def approve(session,ac_id,cookie,ua,members,applications,option):
  if not woi.has_required(["email","account"]):
      cgiutils.html_end(session,
         error="Missing input/s. Complete all fields.")
      return
  email=get_email(session)
  if not email:
      return
  account=get_account(session)
  if not account:
      return
  memidx=members.find(account)
  if memidx != -1:
    cgiutils.html_end(session,
       error="Account already exists, can't add it.")
    return
  appidx=applications.find(email)
  approw=applications.data[appidx]
  if approw["confirmed"] == 'N':
    cgiutils.html_end(session,
      error="Application is unconfirmed so you can't approve it.")
    return
  # if still here, generate new PIN, create and add row to members
  # and balances tables
  newpin=cgiutils.make_pin(mini=10000000,maxi=99999999)
  mrow={}
  mrow["ac_id"]=account
  mrow["name"]=approw['name']
  mrow["email"]=approw['email']
  mrow["pin"]=newpin
  mrow["phone1"]=approw['phone1']
  mrow["phone2"]=approw['phone2']
  mrow["address"]=approw['address']
  mrow["postcode"]=approw['postcode']
  mrow["district"]=approw['district']
  mrow["url"]='None' # URL only configurable using mdetails.cgi
  mrow["pic"]='None' # photo only configurable using mdetails.cgi
  brow={}
  if option == "active":
    mrow["active"]='Y' # active membership
    if nm_vars['new_active_has_limit'] == 'N': 
      brow["limit"]="None"
    else:
      brow["limit"]=nm_vars['new_active_limit']
  else:
    mrow["active"]='P' # provisional membership
    if nm_vars['new_provis_has_limit'] == 'N': 
      brow["limit"]="None"
    else:
      brow["limit"]=nm_vars['new_provis_limit']
  brow["ac_id"]=account
  brow["balance"]=0.0
  brow["cfwd"]=0.0
  brow["in"]=0.0
  brow["out"]=0.0
  brow["turnover"]=0.0
  brow["oldturn"]=0.0
  # limit of None or string value of floating point
  balance=tablcgi.table(btab,dir=dl)
  if not cgiutils.auth(session,ac_id,"ACCEPTREJECT",
        str(mrow)+str(brow),whom="admin"):
    return
  members.addrow(mrow)
  balance.addrow(brow) 
  applications.delrow(email)
  send_accept(email,account,newpin,option)
  print("<p>new member added")
  print(cgiutils.send_form("continue.form",["approval.cgi",ac_id,cookie,ua]))
  cgiutils.html_end(session, received=0,want_form=1)
  return

def reject(session,ac_id,cookie,ua,members,applications):
  if not woi.has_required(["email","reason"]):
     cgiutils.html_end(session, 
         error="Missing input/s. Please complete all fields.")
     return
  email=get_email(session)
  if not email:
     return
  reason=get_reason(session)
  if not reason:
     return
  if not cgiutils.auth(session,ac_id,"ACCEPTREJECT",
        "reject application "+email, whom="admin"):
     return
  appidx=applications.find(email)
  applications.delrow(email)
  send_reject(email,reason)
  print("The rejection reason as input has been emailed to: "+email)
  print(cgiutils.send_form("continue.form",["approval.cgi",ac_id,cookie,ua]))
  cgiutils.html_end(session, received=0,want_form=1)
  return

def send_reject(email,reason):
  # send message to address email confirmed on application
  fromad=v['admin_e']
  message="From: "+v['registry']+"  <"+v['admin_e']+">\n"
  to_line="To: %s\n" % email
  message+=to_line
  message+="Subject: "+v['registry']+" application rejection \n\n"
  message+="Sorry but it is not possible to accept your \n"
  message+="application at this time.\n\n"
  message+="The administrator who considered your application\n"
  message+="gave the following reason:\n"
  message+="\n"
  message+=reason
  message+="\n\n"
  message+="We are sorry your application has been unsuccessful."
  cgiutils.send_mail(fromad,email,message)

def send_accept(email,account,pin,option):
  ''' send message to address email confirmed on application
  parameters: 
  email - email address of applicant 
  account - name of account created
  pin - PIN user will need to login 
  option - type of membership granted, "provis" or "active"
  '''
  type="unknown"
  if option == "active":
    type="active"
  elif option == "provis":
    type="provisional"
  fromad=v['admin_e']
  regy=v['registry']
  url=v['homepage']
  message="From: "+regy+"  <"+v['admin_e']+">\n"
  to_line="To: %s\n" % email
  message+=to_line
  message+="Subject: "+regy+" application accepted \n\n"
  message+="Welcome to "+regy+". We hope you will enjoy membership.\n" 
  message+="Your application to join "+regy+" has been \n"
  message+="accepted. Please keep these details for future reference.\n\n"
  message+="The type of membership granted is %s.\n\n" % type
  message+="Your account identifier is %s .\n\n" % account 
  message+="Your personal identification number (PIN) is %s .\n\n" % pin
  message+="The accounts, membership and trading website is accessible\n"
  message+="at the following address:\n\n"+url+"\n\n"
  message+="Do please login there now and post your wants and offers\n"
  message+="using the trading directory option from the main menu.\n"
  cgiutils.send_mail(fromad,email,message)

def continu(session,ac_id,cookie,ua):
  if not cgiutils.auth(session,ac_id,"APPLICATIONSVIEW","view applications",
                         whom="admin"):
    return
  # create application selection statement to insert in approval form
  from table_details import cattab
  applications=tablcgi.table(apptab,dir=dl)
  # clone a auditfilenames categories type table 
  emailopts=tablcgi.table_def(cattab)
  emailopts.file=None # in memory only - don't save
  eopts=tablcgi.table(emailopts)
  i=0
  for arow in applications.data:
    if arow['confirmed']=='Y': # can only approve or reject once confirmed
      email=arow['email']
      fullname=arow['name']
      i+=1
      frow={"sort":i,"category":email,"desc":fullname+' &lt;'+email+'&gt;'}
      eopts.addrow(frow)
  html_select=cgiutils.make_select(eopts,
              instruction="choose application to process",
              cgiparameter="email") 
  # send the approval form
  print(cgiutils.send_form("approval.form",[v['registry'],ac_id,
    cookie,ua,html_select]))
  return

def main():
  session=False
  cgiutils.html_header(title="LETS Application Approval or Rejection",
                       bgcolor='"#FFFFFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="Invalid or stale or logged out session.")
    return
  # 
  if ua != 'admin':
    cgiutils.html_end(session,error="You must login as admin to do this.")
    return
  # which submit button was pressed ?
  valid,option=cgiutils.get_option(woi,buttons=[
       "active","provis","reject","view","continue"])
  if not valid:
    return # error message already sent

  # If we've got this far, there must be a valid option and PIN OK
  members=tablcgi.table(mtab,dir=dl)
  applications=tablcgi.table(apptab,dir=dl)
  expire_unconfirmed(applications) # expire accounts left unconfirmed too long
  if option == "continue":
    # regenerate approval form
    continu(session,ac_id,cookie,ua)
    return
  elif option in ["active","provis"] : # account approval active or provisional 
    approve(session,ac_id,cookie,ua,members,applications,option)
    return
  elif option == "reject":
    reject(session,ac_id,cookie,ua,members,applications)
    return
  elif option == "view":
    if not cgiutils.auth(session,ac_id,"APPLICATIONSVIEW",
          "view applications", whom="admin"):
      return
    applications.sort(key="name")
    applications.tab2html(skip_cols=["token"])
    return

if __name__ == "__main__":
  try:
      main()
  except:
    import traceback
    print("error detected in approval.cgi main()")
    traceback.print_exc()

