#!/usr/bin/env python

# change next line from 0 to 1 to debug script

debug=0

""" authorise.cgi This file handles input from the 
    form used to create/delete/view/modify authorisations
    and view audit log.
    Richard Kay 
    Jan 2010.
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import authtab,mtab
site_vars=cgiutils.get_vars('site.cfg')
dl=site_vars['data_location']
sec_vars=cgiutils.get_vars('secure.cfg')

def read_authcodes():
  import authcodes  
  return authcodes.codes()
 
def main():
  session=False
  cgiutils.html_header(title="LETS Authorisation and Audit",
                       bgcolor='"#FFFFFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  valid,option=cgiutils.get_option(woi,buttons=
      ["add","delete","view","codes","audit"])
  if not valid:
    return # error message already sent
  if ua != 'admin':
    cgiutils.html_end(session,error="Login as admin to do this.")
    return
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="Invalid or stale or logged out session.")
    return
  # 
  # If we've got this far, there should be a valid option and PIN OK
  auths=tablcgi.table(authtab,dir=dl)
  if option in ["add","delete"]: # need details for authorisation 
    if not woi.has_required(["auth","acbe","whom"]):
      cgiutils.html_end(session,error=
        "Missing field for add or del authorisation. Complete all fields.")
      return
    # validate fields
    members=tablcgi.table(mtab,dir=dl)
    acbe=woi.firstval("acbe") # account being authorised
    if not cgiutils.is_valid(acbe,max_leng=8,
        allowed='^\w+$'):
      cgiutils.html_end(session,error="Invalid account format.")
      return
    if acbe not in members.keys():
      cgiutils.html_end(session,
         error="Attempt to authorise non-existent account.")
      return
    whom=woi.firstval("whom") # account acbe is acting on behalf of 
    if not cgiutils.is_valid(whom,max_leng=8,
        allowed='^\w+$'):
      cgiutils.html_end(session,error="Invalid account format.")
      return
    if whom not in members.keys():
      errmes="Attempt to authorise on behalf of non-existent account."
      cgiutils.html_end(session,error=errmes)
      return
    # 
    auth=woi.firstval("auth") # name of authority to be granted
    if not cgiutils.is_valid(auth,max_leng=25,
        allowed='^[A-Z]+$'):
      cgiutils.html_end(session,
         error="Invalid authority code. Letters A-Z only.")
      return
    codes=read_authcodes()
    if not auth in codes:
      cgiutils.html_end(session,error="Non existent authority code.")
      return
    # 
    authkey=auth+','+acbe+','+whom
  if option == "add": 
    # check combination of auth and acbe doesn't exist
    if authkey in auths.keys():
      cgiutils.html_end(session,error="authority exists. Can't add it.")
      return

    # if still here add row to auths table
    row={}
    row["authac"]=authkey
    row["auth"]=auth
    row["acbe"]=acbe
    row["whom"]=whom
    if not cgiutils.auth(session,ac_id,"AUTHADD",cgiutils.mystr(row),whom=acbe):
      return
    auths.addrow(row)
    print("<p>New authorisation "+auth+" added for "+acbe+"</p>")
    cgiutils.html_end(session,received=1)
    return
  elif option == "delete":
    if authkey not in auths:
      cgiutils.html_end(session,
         error="Authority does not exist. Can't delete it.")
      return
    aindex=auths.find(authkey)
    if not cgiutils.auth(session,ac_id,"AUTHDEL",authkey,whom=acbe):
      return
    auths.delrow(authkey)
    print("<p>Authorisation "+auth+" removed from "+acbe+"</p>")
    cgiutils.html_end(session, received=1)
    return
  elif option == "view":
    # view auths table
    auths.sort()
    if not cgiutils.auth(session,ac_id,"AUTHVIEW",'view authorisations'):
      return
    auths.tab2html()
    cgiutils.html_end(session)
    return
  elif option == "codes":
    # list authorisation codes and descriptions
    import authcodes
    authcodes.htmltab()    
    cgiutils.html_end(session,received=1)
    return
  elif option == "audit": # view audit log
    from table_details import aactions
    if "auditfilename" not in woi.keys():
      # if archived log not selected display current log by default
      aact=tablcgi.table(aactions,dir=dl)
      if not cgiutils.auth(session,ac_id,"AUDITVIEW",
          'view current audit records'):
        return
      aact.tab2html()
    else:
      auditfilename=woi.firstval("auditfilename")
      if not cgiutils.is_valid(auditfilename,max_leng=17,
                    allowed='^aactions(\.[0-9]+)?\.pkl$'):
        cgiutils.html_end(session,error="Invalid audit archive filename.")
        return
      import os.path
      full_auditfilename=os.path.join(dl,auditfilename)
      if not os.path.isfile(full_auditfilename):
        cgiutils.html_end(session,error="Audit archive filename is not a file.")
        return
      # clone the aactions table definition object before
      # overriding the audit.file attribute, to avoid poisoning use of this
      # object attribute elsewhere
      caactions=tablcgi.table_def(aactions)
      caactions.file=full_auditfilename
      aact=tablcgi.table(caactions,dir=dl)
      if not cgiutils.auth(session,ac_id,"AUDITVIEW",
          'view audit archive: '+full_auditfilename):
        return
      aact.tab2html()
    cgiutils.html_end(session)
    return

if __name__ == "__main__":
  try:
      main()
  except:
    import traceback
    print("error detected in authorise.cgi main()")
    traceback.print_exc()

