#!/usr/bin/env python

# change next line to debug=1 to debug

debug=0

""" balance.cgi This file handles input from the balance 
    form used to request balance details. 
    Richard Kay last changed: 21 Mar 2008
"""

# uncomment next 3 lines to debug script

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import btab,mtab
site_vars=cgiutils.get_vars('site.cfg')
dl=site_vars['data_location']

def main():
  session=False
  if not woi.has_required(["sms_minibal"]):
    # This balance enquiry requires HTML 
    cgiutils.html_header(title="LETS Accounting balance request",
                        bgcolor='"#FFFFDD"')
  else:
    print("Content-type: text/plain\n") # client still using HTTP
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid or stale or logged out session.")
    return
  #  
  # validate other fields
  if not woi.has_required(["enquiry_id"]):
    if ua == 'admin':
      cgiutils.html_end(session,
        error="Missing value/s. Complete all form fields.")
      return
    else:
      enquiry_id=ac_id
  else:
    enquiry_id=woi.firstval("enquiry_id")
  if not cgiutils.is_valid(enquiry_id,max_leng=8,allowed='^[\w]+$'):
    cgiutils.html_end(session,error="Invalid enquiry account id/no.")
    return
  members=tablcgi.table(mtab,dir=dl)
  enq_member=enquiry_id in members.keys()
  if not enq_member:
    cgiutils.html_end(session,error="Enquiry Account no./id not found.")
    return
  balance=tablcgi.table(btab,dir=dl)
  if enquiry_id not in balance.keys():
    error="""enquiry a/c has member record but no balance record. 
      please inform webmaster."""
    cgiutils.html_end(session,error)
    return
  # display details
  cfwddate=open("cfwd_date.txt").read()
  if not woi.has_required(["sms_minibal"]): # HTML or SMS output
    print(cfwddate)
  else: 
    print("cfwd date: "+cfwddate[25:36])
  bindex=balance.find(enquiry_id)
  bal=balance.data[bindex]["balance"]
  # audit event - but whenever someone can login this action is allowed
  cgiutils.auth(session,ac_id,"BALVIEW","balance enquiry on: "+enquiry_id,
                      whom=enquiry_id, auditonly=True)
  if not woi.has_required(["sms_minibal"]):
    # This balance enquiry requires HTML 
    # create balance table view object
    bvd=tablcgi.table_def(btab)
    bvd.file=None
    btv=tablcgi.table(bvd)
    row={}
    for key in ["ac_id","cfwd","balance","in","out",
                "turnover","oldturn","limit"]:
      row[key]=balance.data[bindex][key]
    btv.addrow(row)
    btv.tab2html()
    cgiutils.html_end(session)
    return
  else: 
    # This balance enquiry requires SMS text formatted output 
    print("A/C   CFWD IN  OUT  BAL")
    print("%s %.2f %.2f %.2f %.2f" % ( 
          balance.data[bindex]["ac_id"],
          balance.data[bindex]["cfwd"],
          balance.data[bindex]["in"],
          balance.data[bindex]["out"],
          balance.data[bindex]["balance"]))
    print("TRNOVR OLD_TRNOVR LIMIT")
    print("%.2f %.2f %s" % ( 
          balance.data[bindex]["turnover"],
          balance.data[bindex]["oldturn"],
          balance.data[bindex]["limit"]))
    return

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in balance.cgi main()")
    traceback.print_exc()

