#!/usr/bin/env python

# change next line to debug=1 to debug script

debug=0

""" cdirectory.cgi This file handles membership contact directory options
    and address label printing suitable for cut/paste into spreadsheet. 
    Richard Kay last changed: 21 Mar 2008
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import mtab,ctab,latab

site_vars=cgiutils.get_vars('site.cfg')
dl=site_vars['data_location']

sec_vars=cgiutils.get_vars('secure.cfg')

def vlabels(session,members,activ,lcols,noemail,ac_id):
  ''' outputs address labels in number of columns given by lcols '''
  members=tablcgi.table(mtab,dir=dl)
  labels=tablcgi.table(latab)
  col=lcols-1 # it seems we have to initialise col to something to
  # avoid a very infrequent uninitialised local variable bug, presumably
  # when the for loop below iterates zero times 
  i=-1
  # initialise next row with blank labels
  lrow={"0":"","1":"","2":"","3":""}
  reserved_acs=site_vars['reserved_account_names'].split(',')
  for row in members.data:
    if (( activ == "all" or row["active"] in ['Y','P']) and
        ( noemail == "include" or row["email"] == 'None' )) :
      if row["ac_id"] not in reserved_acs:
        i+=1
        col=i%lcols # gives column position in range 0 .. lcols-1 
        name=row["name"]
        address=row["address"]
        postcode=row["postcode"]
        label='<pre>'+name+'\n'+address+'\n'+postcode+'</pre>'
        # Dictionary key is one strings in set: "0","1","2","3"
        lrow[str(col)]=label
        if col + 1 == lcols:
          # add complete row to labels table
          labels.addrow(lrow)
          # initialise next row
          lrow={"0":"","1":"","2":"","3":""}
  # add last row if incomplete 
  if col + 1 != lcols: 
    labels.addrow(lrow)
  # print labels out omitting blank columns
  if not cgiutils.auth(session,ac_id,"LABELS","generate address labels"):
    return
  if lcols == 4:
    labels.tab2html() 
  if lcols == 3:
    labels.tab2html(skip_cols=["3"]) 
  if lcols == 2:
    labels.tab2html(skip_cols=["2","3"]) 
  if lcols == 1:
    labels.tab2html(skip_cols=["1","2","3"]) 
  cgiutils.html_end(session)

def show_cdirectory(session,option,members):
  # Outputs selected columns of contact directory as HTML table 
  contacts=tablcgi.table(ctab) # contacts file definition object
  # adjust contact view meta data to allow for row coloration
  contacts.meta["keys"].append("bgcolor")
  contacts.meta["colheads"].append("Bgc")
  contacts.meta["formats"].append("%s")
  # 
  # copy records required to new contacts table
  for row in members.data:
    mindex=members.find(row["ac_id"])
    active=members.data[mindex]["active"] 
    if ((option == 'active' 
        and active in ['Y','P'] 
        and row["ac_id"] != "admin" 
        and row["ac_id"] != "1" ) 
        or option == 'all'): 
      crow={}
      for key in [
          "ac_id","active","name","email","phone1","phone2","url","pic"]:
        crow[key]=row[key]
      crow["bgcolor"]=cgiutils.get_color(active,
                          default_color='"#FFFFFF"')
      contacts.addrow(crow)
  # output to browser
  cgiutils.print_colortable()
  contacts.sort(key="ac_id")
  contacts.tab2html(skip_cols=["bgcolor"],color_row_key="bgcolor")
  cgiutils.html_end(session)

def main():
  session=False
  cgiutils.html_header(title="LETS Member Contact Directory Access",
                       bgcolor='"#FFDDFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  valid,option=cgiutils.get_option(woi,buttons=
     ["active","all","labels","addr"])
  if not valid:
    return # error message already sent
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="Invalid or stale or logged out session.")
    return
  # 
  members=tablcgi.table(mtab,dir=dl)
  if option == "active":
    show_cdirectory(session,"active",members)
  elif option == "all":
    show_cdirectory(session,"all",members)
  elif option == "addr":
    if not cgiutils.auth(session,ac_id,"FULLADDRVIEW",
           "view full contact details"):
      return
    members.tab2html(skip_cols=["pin"])
    cgiutils.html_end(session)
  elif option == "labels":
    if not ua == "admin":
      cgiutils.html_end(session,error="Only admin may view address labels.")
      return
    if not woi.has_required(["labelsa","lcols"]):
      cgiutils.html_end(session,
         error="Missing value/s. Complete all form fields.")
      return
    activ=woi.firstval("labelsa")
    if activ not in ["active","all"]:
      cgiutils.html_end(session,error="Incorrect value for active or all.")
      return
    cols=woi.firstval("lcols")
    if cols not in ["1","2","3","4"]:
      cgiutils.html_end(session,error="Incorrect value for label columns.")
      return
    lcols=int(cols)   
    # labels only for members without email ? 
    noemail=woi.firstval("noemail") 
    if noemail not in ["without","include"]:
      errmes="Incorrect value for members without email only."
      cgiutils.html_end(session,error=errmes)
      return
    vlabels(session,members,activ,lcols,noemail,ac_id)
  return

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in cdirectory.cgi main()")
    traceback.print_exc()

