#!/usr/bin/env python

# change next line to 1 to debug script
debug=0

""" entry.cgi This file is the gateway program for a
    simple LETS accounting and directory system. It provides
    Initial authentication, PIN request and access to main menu 
    for accounts, contact directory, trading directory
    and membership admin.  
    Richard Kay last changed: 14 Nov 2023
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput() 
import tablcgi
from table_details import atab,btab,mtab,stab,authtab
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']

def send_pin(session,ac_id,email,members):
  if email=="none":
    cgiutils.html_header(title="LETS Accounting and Directory System")
    errmes='''<p>
           Account does not have an email address recorded. Pylets Access 
           is not supported without an email address. Please contact the 
           membership administrator with details of your email address 
           when you have one.<p>'''
    cgiutils.html_end(session,error=errmes)
    return
  # need to access members table to check if user is a member
  index=members.find(ac_id)
  pin=members.data[index]["pin"]
  # send PIN to address entered
  fromad=v['admin_e']
  message="From: "+v['registry']+" Accounts <"+v['admin_e']+">\n"
  to_line="To: %s\n" % email
  message+=to_line
  message+="Subject: "+v['registry']+" PIN request \n\n"
  message+="Someone (hopefully you) entered your account number/id\n"
  message+="and email address requesting a PIN for your account.\n"
  message+="\n"
  pin_line="The PIN you will need for access is: %s\n" % pin
  message+=pin_line
  message+="\n"
  message+="If you didn't request your PIN, please ignore this message.\n"
  message+="However, if this error persists please contact\n"
  message+=v['abuse']+" .\n"
  message+="\n"
  cgiutils.send_mail(fromad,email,message)
  cgiutils.html_header(title="LETS Accounting and Directory System")
  print("<p>The PIN for the account has been sent as requested.</p>")
  cgiutils.html_end(session,received=1)

def main():
# cgiutils.html_header(title="LETS Accounting and Directory System")
  session=False 
  if len(list(woi.keys())) == 0:
    # if no keys send the form to the browser
    # include a logout which clears the form 
    cgiutils.html_header(title="LETS Accounting and Directory System")
    system_name=v['registry']
    print(cgiutils.send_form("entry.form",[system_name]))
    cgiutils.html_end(session,want_form=1)
    return
  elif not woi.has_required(["account"]):
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="You havn't input your account no. or identifier.")
    return

  # has required keys - process form
  ac_id=woi.firstval("account")
  if not cgiutils.is_valid(ac_id,allowed='^\w+$'):
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid account number/identifier.")
    return 
  # which submit button was pressed ?
  option="none"
  for word in ["sendpin","user","admin"]:
    if word in woi.keys():
      option=word

  if option=="none": 
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid submit button value.")
    return
  
  # check if a PIN was entered and validate it
  pinentry=woi.firstval("pin",default="")
  if pinentry and not cgiutils.is_valid(pinentry,allowed='^[1-9][0-9]{7,7}$'):
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid PIN format.")
    return
  if pinentry:
    if option not in ["user","admin"]: # PIN entered but wrong button 
      cgiutils.html_header(title="LETS Accounting and Directory System")
      cgiutils.html_end(session,error=
        "A PIN was entered so you must login as account holder or as admin.")
      return
    # check PIN and account
    session=cgiutils.login(ac_id,pinentry,option)
    valid=session.login()
    cookie=session.cookie
    if not valid: 
      cgiutils.html_header(title="LETS Accounting and Directory System")
      cgiutils.html_end(session,error="Incorrect PIN and/or account entered, or account inactive.")
      return
  elif not option == "sendpin":
    # Without a valid pin actions other than sending pin are prohibited
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="No PIN entered.")
    return
  # If we've got this far, there should either be a valid option and PIN
  # or we have a PIN request.
  if option == "sendpin":
    # check correct email registered for account entered
    members=tablcgi.table(mtab,dir=dl)
    index=members.find(ac_id)
    email_entered=woi.firstval("EMAIL",default="")
    email_entered=email_entered.lower()
    email=members.data[index]["email"]
    email=email.lower()
    if email_entered != email:
      cgiutils.html_header(title="LETS Accounting and Directory System")
      cgiutils.html_end(session,
         error="Incorrect email address, can't send PIN.")
      return
    status=members.data[index]["active"]
    if status == 'N': # N for inactive, Y for active, P for provisional
      cgiutils.html_header(title="LETS Accounting and Directory System")
      cgiutils.html_end(session, error=
        "Inactive account, PIN not sent. Contact your LETS administrator to reactivate account.")
      return
    send_pin(session,ac_id,email,members)
    return
  # If we've got this far, there should be a valid option and cookie 
   
  elif option in ["user","admin"]:
    if option == "admin":
      # prevent users without administrative authorisations from
      # getting further with the normal browser interface
      # Others will be excluded prior to the specific unauthorised
      # action, but we dont want to scare users who make mistakes
      # by logging unneccessarily.
      can_admin=False # until proven otherwise
      if ac_id == 'admin':
        can_admin=True
      else:
        auths=tablcgi.table(authtab,dir=dl)
        for arow in auths.data :
          if arow['acbe'] == ac_id :
            can_admin=True # well maybe - specific authorisation TBC later 
      if not can_admin :
        cgiutils.html_header(title="LETS Accounting and Directory System")
        cgiutils.html_end(session,error=
            "No administrative authorisations. Please login as user.")
        return
    # All OK sofar. Send the main menu form 
    cgiutils.html_header(title="LETS Accounting and Directory System",
                         bgcolor='"#FFFFFF"')
    print(cgiutils.send_form("mainmenu.form",[v['registry'],ac_id,
        cookie,option,None],ua=option))
    return 
  else:
    # should have trapped this one earlier ???
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid option value (2nd trap).")
  return 

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in entry.cgi main()")
    traceback.print_exc()

