#!/usr/bin/env python

# change next line to 1 to debug script
debug=0

""" mainmenu.cgi This file is the main menu program for a
    simple LETS accounting and directory system. It provides
    forms for accounts, contact directory, trading directory
    and membership admin.  
    Richard Kay last changed: 08 Apr 2008
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import atab,btab,mtab,stab,cattab

sec_vars=cgiutils.get_vars('secure.cfg')
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']

def main():
  ''' This cgi main() function differs from others in the sense
      that the HTML header is customised to suit the 
      subsystem the main menu option directs the user into.
      This currently prevents reuse of cgiutils.get_options
      and cgiutils.get_hidden functions. 
  '''
  session=False
  if len(list(woi.keys())) == 0:
    cgiutils.html_header(title="LETS Accounting and Directory System",
                         bgcolor='"#FFFFFF"')
    # if no keys authentication data missing
    cgiutils.html_end(session,error="Missing authentication details.")
    return
  elif not woi.has_required(["ac_id","pin","ua"]):
    cgiutils.html_header(title="LETS Accounting and Directory System",
                         bgcolor='"#FFFFFF"')
    cgiutils.html_end(session,
       error="Missing value/s. Complete all form fields.")
    return

  # has required keys - process form
  ac_id=woi.firstval("ac_id")
  if not cgiutils.is_valid(ac_id,allowed='^\w+$'):
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid account number/identifier.")
    return 
  # which submit button was pressed ?
  option="none"
  for word in ["accounts","tdirectory","cdirectory","admin","main",
               "mydetails","logout"]:
    if word in woi.keys():
      option=word

  if option=="none": 
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid submit button value.")
    return
  
  # user/admin option
  ua=woi.firstval("ua")
  if not ua in ["user","admin"]:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Incorrect user/admin option entered.")
    return
  
  # check if a PIN was entered and validate it
  pincookie=woi.firstval("pin",default="")
  if not cgiutils.is_pincookie(pincookie):
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid PIN format.")
    return
  # check PIN and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie 
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid or stale or logged out session.")
    return

  # If we've got this far, there should be a valid option 
   
  if option == "cdirectory":
    # send the contact directory form 
    cgiutils.html_header(title="LETS Contact Directory System",
                         bgcolor='"#FFDDFF"')
    print(cgiutils.send_form("cdirectory.form",[v['registry'],
       ac_id,cookie,ua,None],ua=ua))
  elif option == "accounts":
    # send the accounts form
    cgiutils.html_header(title="LETS Accounting System",
                         bgcolor='"#FFFFDD"')
    if ua=='admin':
      # create ack current and archived files selection statement
      ackfiles=cgiutils.list_ackfiles()
      html_select=cgiutils.make_file_select(ackfiles,
         instruction="choose current file or archive",
         cgiparameter="ackfile")
    else: # users don't see this
      html_select=''
    # send the accounts form
    print(cgiutils.send_form("accounts.form",[v['registry'],ac_id,cookie,ua,
                          None,None,html_select ],ua=ua))
  elif option == "tdirectory":
    # send the trading directory form
    cgiutils.html_header(title="LETS Trading Directory System",
                         bgcolor='"#DDFFFF"')

    # make category selection HTML string
    categtab=tablcgi.table(cattab,dir=dl)
    selectcategories=cgiutils.make_select(categtab,instruction=
      "all",
      cgiparameter="category",
      default_value="all")
    print(cgiutils.send_form("tdirectory.form",[v['registry'],ac_id,cookie,ua,
        selectcategories,None],ua=ua))
  elif option == "admin":
    # while the admin form does its own check, no need for anyone
    # not admin to see it
    if not ua == "admin":
      cgiutils.html_header(title="LETS Accounting and Directory System")
      cgiutils.html_end(session,
         error="You must login as admin to use the admin form.")
      return
    # send the administration form
    cgiutils.html_header(title="LETS Accounting and Directory System")
    print(cgiutils.send_form("admin.form",[v['registry'],ac_id,cookie,ua]))
  elif option == "mydetails":
    # user wants to change membership details or reset PIN
    members=tablcgi.table(mtab,dir=dl)
    midx=members.find(ac_id)
    status=members.data[midx]['active']
    # provisionals may not have URL form option by site policy
    # default is the relevant form part
    #
    mayurl='''
<p>Homepage.<INPUT TYPE="text" NAME="url" SIZE="40" MAXLENGTH="70">
Input a URL link to your homepage or blog e.g. http://fred.example.com/ .</p>
''' 
    #
    # provisionals may not upload photo by site policy
    mayphoto=( '''
<h3> Upload my Photo </h3>
<p>You may upload a digital photo as part of your member
profile, and this will be visible from the contacts
list. The photo must be in JPEG or PNG format and less than
'''+v['maximagesize']+''' KBytes in size. <i> 100x120 pixels are 
displayed. Different formats or larger size photos will be 
rejected. On most computers you can right mouse click a file 
to view file properties</i>.</p>
<p>Select photo <input type="file" name="filename" />
Upload photo <input TYPE="submit" NAME="Upload" VALUE="Upload"><br>
</p> ''' )   
    #
    if v['provisional_can_url'] == 'N' and status.upper() == 'P' :
      mayurl='' # provisionals dont see this part of form 
    if v['provisional_can_photo'] == 'N' and status.upper() == 'P' :
      mayphoto='' # provisionals dont see this part of form
    # edit RK 18 Nov 23 due to python3 multipart processing 
    # breakage requiring further development. When done, restore
    # mayphoto to include form fragment coded above
    # by commenting out following line. 
    mayphoto=''
    cgiutils.html_header(title="LETS Accounting and Directory System")
    print(cgiutils.send_form("mdetails.form", [v['registry'],
          ac_id,cookie,ua, mayurl, v['admin_e'], ac_id,cookie,ua,
          mayphoto ], ua=ua))
  elif option == "logout":
    cgiutils.html_header(title="LETS Accounting and Directory System")
    session.logout()
    print(cgiutils.send_form("entry.form",[v['registry']]))
  elif option == "main": # regenerate main menu
    cgiutils.html_header(title="LETS Accounting and Directory System")
    print(cgiutils.send_form(
         "mainmenu.form",[v['registry'],ac_id,cookie,ua,None], ua=ua))
  else:
    # should have trapped this one earlier ???
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid option value (2nd trap).")
  return 

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in mainmenu.cgi main()")
    traceback.print_exc()

