#!/usr/bin/env python

# uncomment next 2 lines to debug script
#import sys
#sys.stderr=sys.stdout

''' make_clean.py Richard Kay 2002 - 2010
# script to start a PyLETS CGI database clean with just an
# admin account 
# DO NOT RUN ON A WORKING SYSTEM !!! make it non executable.
# 
# This script can be run in one of 2 modes. 
# 
# 1. testing mode: controlled either by adding a testing attribute
# when run from make_clean.cgi, or by using a 'testing' argument in sys.argv . 
# This makes it easier for the admin account to get into the 
# test system using a default pin of 12345678
# 
# 2. secure mode: default behaviour. In this mode the
# script generates a secure admin PIN and leaves the
# files with more secure permissions.  
'''

import tablcgi,cgiutils
import sys
import os,os.path,pickle

def main(tsmode='secure'):
  print("Content-type: text/plain\n")
  # check if in testing or secure mode
  if 'testing' in sys.argv:
    mode='testing'
  else:
    mode=tsmode
  
  # read site configuration into dictionary variable v
  v=cgiutils.get_vars('site.cfg')
  tv=cgiutils.get_vars('test.cfg')
  dl=v['data_location']
  from table_details import mtab,atab
  
  # get rid of unwanted .pkl data files
  for filename in ['summary_all.pkl','summary_active.pkl',"members.pkl",
      "balances.pkl","acks.pkl","turnover.pkl","acks_sqk.pkl","LockFile.log",
      "wantsoffers_sqk.pkl","overrides_sqk.pkl","temp.pkl","overrides.pkl",
      "wantsoffers.pkl","email_accounts.txt","just_email.txt",
      "logins.pkl","logins_sqk.pkl","lacs.pkl","gbips.pkl","categories.pkl",
      "aactions_sqk.pkl","aactions.pkl","authorisations.pkl",
      "applications.pkl"]:
    fullname=os.path.join(dl,filename)
    if os.path.isfile(fullname):
      os.unlink(fullname)
  
  # more files to be removed matched using wildcards
  import glob
  # glob file pattern matching to get rid of more files
  delfiles=[]
  for delfile in glob.glob('*~'): # edit backups
    delfiles.append(delfile) 
  for delfile in glob.glob('*.pyc'): # compiled python bytecodes 
    delfiles.append(delfile) 
  for delfile in glob.glob(os.path.join(dl,'logins.*.pkl')):
    # rotated login logfiles 
    delfiles.append(delfile)
  for delfile in glob.glob(os.path.join(dl,'aactions.*.pkl')): 
    # rotated audit logfiles 
    delfiles.append(delfile)
  for delfile in glob.glob(os.path.join(dl,'acks.*.pkl')): 
    # rotated ack archives 
    delfiles.append(delfile)
  if mode == 'secure': 
    for delfile in glob.glob(os.path.join('testsuite','*.pyc')): 
      # testsuite bytecodes 
      delfiles.append(delfile) 
  
  for delfile in delfiles:
    #print "file to be deleted: "+delfile
    os.unlink(delfile)
  
  # initialise admin account details 
  admins={}
  
  # database administrator
  admin_email=v['admin_e'] 
  admin_name=v['admin_n']
  minpin=int(v['minpin']) 
  maxpin=int(v['maxpin']) 
  if mode == 'secure': # create random PIN 
    apin=cgiutils.make_pin(mini=minpin,maxi=maxpin)
    # add entries like that commented below for other admin accounts
    # smspin=cgiutils.make_pin(min=minpin,max=maxpin)
  else: # testing so use default PIN 
    apin=str(tv['test_pin'])
  # add similar entries for other admin accounts which have no balances
  admins['admin']=[apin,admin_email,admin_name]
  # e.g.  admins['smsgw']=[smspin,'smsgw@example.com','SMS Gateway']
 
  print('admin PIN is: '+str(apin))
  
  # create members file with admin accounts
  members=tablcgi.table(mtab,dir=dl)
  for user in list(admins.keys()):
    mrow={"ac_id":user,
        "pin":admins[user][0],
        "email":admins[user][1],
        "name":admins[user][2],
        "active":'Y',
        "phone1":'0',
        "phone2":'0',
        "address":'not yet input',
        "postcode":'UNKNOWN',
        "district":'unknown',
        "url":'None',
        "pic":'None'
    }
    members.addrow(mrow)
    members.save()
  
  # inactivate further use of this and other test scripts
  # unless testing parameter is supplied. Use with testing parameter
  # if further testing is required, the default is to leave the
  # system safe. 
  
  if mode == 'secure':
    os.system("chmod -x testsuite/test*.py master_key.py")
    os.system("chmod -x test.cgi")
    os.system("chmod -x init_smsgwac.py")
    os.system("chmod ugo=  testsuite")
    os.system("chmod -R o=  "+dl)
    print('make_clean.py run in secure mode')
  else:
    print('make_clean.py run in testing mode')

if __name__ == "__main__":
  try:
      main()
  except:
    import traceback
    print("error detected in make_clean.py main()")
    traceback.print_exc()
