#!/usr/bin/env python

# uncomment next 2 lines to debug script
#import sys
#sys.stderr=sys.stdout

''' make_configure.py Richard Kay  2010
# This script is used to modify configuration variables
# needed for a new installation. 
# Run it after make_install.py or make_test.py
'''

import sys
import os,os.path,pickle
import cgiutils
from default_paths import cfg_directory

class md:
  ''' module data namespace class
  # md.cv contains data useful in assisting setting
  # of configured site variables likely to need changing.
  # 
  # Each file requires entries for filname, keys and information
  # for each variable as below.
  #  
  # Each variable requires:
  # 'default': A default value 
  # 'prompt': a user prompt to change the value 
  # 'help': further helpful information for the user
  '''
  # configuration files listed here so these can be processed in this order 
  cfgflist=['site_vars'] # configuration file list stored so these 
  cv={  
      # Config file: Site Variables 

      "site_vars":{
        "filename":"site.cfg", 
        "keys":['abuse','homepage','registry','admin_e',
               'domain','testdir','ipv4'], 
        "vars":{
      'abuse':{
      'default':'abuse@example.com',
      'prompt':'PIN sending abuse complaint address', 
      'help':'Email address for PIN abuse complaints.' },

      'homepage':{
      'default':'"http://example.com/pylets/entry.cgi"',
      'prompt':'Homepage URL of server',
      'help':'Users will be given a link to this "double quoted" URL '+
             'after successful input e.g.'+
             '"http://example.com/pylets/mainmenu.cgi"' },

      'registry':{
      'default':'Example Town LETS',
      'prompt':'name of community currency', 
      'help':'Name of your Community Currency as shown on HTML headings.' },

      'admin_e':{
      'default':'dbadmin@example.com',
      'prompt':'email to which admin account PIN is sent', 
      'help':'email address of database admininstrator ' },

      'domain':{
      'default':'example.com',
      'prompt':'domain name of server needed for testing',
      'help':'e.g. if your server will be accessed as '+
             'http://example.com/pylets/entry.cgi enter example.com' },

      'testdir':{
      'default':'/',
      'prompt':'URL directory of server needed for testing',
      'help':'e.g. if your server will be accessed as '+
             'http://example.com/pylets/entry.cgi enter /pylets/' },

      'ipv4':{
      'default':'127.0.0.1',
      'prompt':'IP address of test client computer', 
      'help':'This address will be matched by test software from login logs.'+
             'or these tests will fail' },
      } # end vars 
      } # end site_vars 
      # End of Config file : Site Variables 
      } # end cv 

def modify_config_var(filename,varname,newval):
  ''' rewrites a variable value in a
  live or testing .cfg file to configure some options likely to
  need changing.

  Parameters: filename - name of configuration file
              varname - name of variable to be substituted
              newval - new value for variable
  '''
  # regex to substitute value for config_dir
  # portability note: might need to handle windows pathnames as well as Unix
  # example of a matching line: ip4addr:1.2.3.4
  regex=r'^'+varname+r'\:.+$'
  import re
  cfh=open(filename,'r')
  cff=cfh.read() # read config file as string
  cfh.close()
  cre=re.compile(regex,re.MULTILINE)
  subtext=varname+':'+newval
  mo=cre.search(cff)
  if mo: # substitute found instances with subtext
    cff=re.sub(cre,subtext,cff)
  else:
    raise ValueError("Can't substitute invalid contents of "+filename)
  cfh=open(filename,'w')
  cfh.write(cff)
  cfh.close()

def main():
  # loop once for each config file in md.cv
  for cfgf in md.cfgflist:
    filename=md.cv[cfgf]["filename"]
    # read configuration file into dictionary variable v
    v=cgiutils.get_vars(filename)
    import os
    full_fn=os.path.join(cfg_directory,filename) # full config filename
    # prompt the user for each variable in the keylist
    for varn in md.cv[cfgf]["keys"]:
      cvalue=v[varn]
      print()
      print(md.cv[cfgf]["vars"][varn]['help'])
      print('\nenter d for default value: '+md.cv[cfgf]["vars"][varn]['default'])
      print('enter c for current value: '+cvalue+' or input new value')
      print(md.cv[cfgf]["vars"][varn]['prompt'])
      nvalue=input('[d, c or new value] :')
      if nvalue == 'd':
        # reset value to default
        modify_config_var(full_fn,varn,md.cv[cfgf]["vars"][varn]['default'])
      elif nvalue == '' or nvalue == 'c':
        pass # nowt to do, keep current value 
      else: # new value
        modify_config_var(full_fn,varn,nvalue)

if __name__ == "__main__":
  try:
      main()
  except:
    import traceback
    print("error detected in make_configure.py main()")
    traceback.print_exc()
