#!/usr/bin/env python

# installation script for pylets

import os, os.path
import cgiutils

cwd=os.getcwd() # current working directory
# users home directory - good place for pylets data and configs if not root
home=os.getenv('HOME') 
# users login name - helps guess user-mode software install location ?
user=os.getenv('USER')

# default target config, data and software directories for root installs
ri_default_configdir='/etc/pylets' 
ri_default_datadir='/home/pylets' 
ri_default_softdir='/var/www/pylets' 
ri_default_user='www-data'
ri_default_group='www-data'

# default source directories relative to cwd
# (Python program sources and HTML are stored in cwd) 

testsuite='testsuite' # relative path of test suite
test_expected='testsuite/expected_results' # relative path of test expected results
test_data='testsuite/test_data' # relative path of test data
test_configs='testsuite/test_configs' # relative path of test data
package_defaults='package_defaults' # initial and build configuration files and data
forms='forms' # HTML form files

# create namespace class for File Overwrite Policies

class ovpol:
  # default file replacement policies
  always_replace=False # always replace target with source
  always_ask=False # always ask if new file exists 
  ask_if_older=False # only replace if target older than source

def want_user_install():
    print('running make_install.py in non root/admin mode')
    print('PyLETS can be run more securely if you install as root')
    wantuser=input('Do you still want to install without root/admin access ? [n]')
    if not wantuser:
      wantuser='n' 
    if wantuser.lower() == 'n':
      return False
    else:  
      return True

# installation of files, directories and permissions

def install_file(filename,sourcedir,installdir,ovpol,mode,uid,gid,root_install):
  ''' installs single file from localdir into
  installdir. If file exists, user is asked whether
  to override old with new
  
  parameters: filename - basename of file to be copied
              installdir - target installation directory
              sourcedir - source directory for installation file
              ovpol - policies determining when to overwrite
                      existing target
                  always_ask,always_replace,ask_if_older
              mode - octal file creation mode
              uid - numeric UID of file owner
              gid - numeric GID of file owner
  '''
  import time,stat
  # use seconds since epoch as default seconds by which new file
  # is younger than old if old does not exist. 
  now=time.time()
  newness=now
  replace=False 
  sourcef=os.path.join(sourcedir,filename)
  if os.path.isfile(sourcef):
    sfex=True # source file exists
    sfso=os.stat(sourcef) # sfso is source file status tuple object
    # smtime is float seconds since source file modified
    smtime=sfso[stat.ST_MTIME]
    smtime_struct=time.localtime(smtime)
    #print time.asctime(smtime_struct)
  else:
    sfex=False
    errmes="source file "+sourcef+" does not exist so cant install it"
    raise ValueError(errmes) # no more to do 
  targetf=os.path.join(installdir,filename)
  # print targetf+' '+targetf # debug
  if os.path.isfile(targetf):
    tfex=True # target file exists
    tfso=os.stat(targetf) # sfso is source file status tuple object
    # tmtime is float seconds since target file modified
    tmtime=tfso[stat.ST_MTIME]
    tmtime_struct=time.localtime(tmtime)
    #print time.asctime(tmtime_struct)
  else:
    tfex=False
  if tfex:
    newness=smtime-tmtime 
    # seconds by which source file is younger than target
  # print 'newness: '+str(newness) # debug
  if targetf == sourcef: # check if full pathnames are the same
    print("source and target files are identical - not copied")
    return 
  # if targe exists decide whether to replace with source
  replace=True
  if ( (tfex and ovpol.always_ask) or
     (tfex and newness < 0.0 and ovpol.ask_if_older) ):
    print('\nsource file: '+filename+'\n')
    if ovpol.always_ask:
      print('Are you sure you want to replace target file ??')
    if ovpol.ask_if_older:
      print('target file newer than source file.')
      print('target time: '+ time.asctime(tmtime_struct))
      print('source time: '+ time.asctime(smtime_struct))
      print('Are you sure you want to replace target file ??')
    force_replace=input(
        'a - all, y - yes, n - no [n]').lower()
    if not force_replace:
      force_replace='n'
    if force_replace in ['y', 'a']:
      if force_replace == 'a':
        ovpol.always_replace = True
      replace=True
    else:
      replace=False
  # end of while loop      
  if not replace:
    # if don't want to replace - nothing more to do.
    return
  # copy source file to target file
  print('copying source: '+sourcef+' to target: '+targetf) # debug
  rfh=open(sourcef,'r')
  wfh=open(targetf,'w')
  for line in rfh:
    wfh.write(line)
  rfh.close()
  wfh.close()
  os.chmod(targetf,mode) # change user/group/other read/write/x permissions to mode
  if root_install: # change ownership of file as required for root installation
    os.chown(targetf,uid,gid)
  # modify access/modify times of target to match source
  os.utime(targetf,(smtime,smtime))

def rewrite_default_path(ddir,varname):
  ''' rewrites a path in default_paths.py so the
  new path can be easily imported by other modules.

  Parameters: ddir - value_of_new_default_directory_path
              varname - name of variable to be substituted 
                        with ddir as value
  '''
  # regex to substitute value for config_dir
  # portability note: might need to handle windows pathnames as well as Unix
  # example of a matching line: cfg_directory='/etc/pylets'
  regex=varname+r"""\=                  # match start
          \'                            # literal single quote
          (/[a-zA-Z0-9\.\ \-\_]+)+      # e.g. /etc/a/b/c
          \'"""                         # end with single quote
  
  import re
  cfh=open('default_paths.py','r')
  dpp=cfh.read()
  cfh.close()
  cre=re.compile(regex,re.VERBOSE)
  subtext=varname+"='"+ddir+"'"
  mo=cre.search(dpp)
  if mo: # substitute found instances with subtext
    dpp=re.sub(cre,subtext,dpp)
  else:
    raise ValueError("Can't substitute invalid contents of default_paths.py")
  cfh=open('default_paths.py','w')
  cfh.write(dpp)
  cfh.close()

def getmakedir(root_install,helpinfo,rdl,udl,uid,gid,dcmode=0o770):
  ''' generic directory location prompt and creation tool 
      parameters: root_install True if root, False if user mode
      helpinfo: dictionary containing help and prompt text
      rdl: root mode default location for directory
      udl: user mode default location for directory
      dcmode: octal directory creation mode, default drwxrwxr-x
      uid: directory user owner
      gid: directory group owner
  '''
  print('\n'+helpinfo['help']+'\n') 
  while(True):
    if root_install:
      direc=input(helpinfo['prompt']+" ["+rdl+"] ")
      if not direc: # use default if user just pressed enter
        direc=rdl
    else:
      direc=input(helpinfo['prompt']+" ["+udl+"] ")
      if not direc:
        direc=udl
    if not os.path.isdir(direc):
      try:
        print('attempting to create: '+direc)
        os.makedirs(direc,dcmode)
      except:
        print('unable to create: '+direc)
    if root_install:
      os.chown(direc,uid,gid)
    if not os.access(direc,os.W_OK):
      print(direc+' is not writeable')
    elif direc == cwd:
      print("Installation target same as source directory not supported.")
      print("Either target installation to a different directory")
      print("or if you really want to install in the current directory,") 
      print("please unpack installation source files elsewhere.")
      elsewhere=input(
                "different target - d, unpack elsewhere [e]").lower()
      if not elsewhere or elsewhere == 'e':
        raise EnvironmentError("install source and target locations identical") 
    else: # should be OK
      return direc # way out of while loop 

def file_overwrite_policy(ovpol):  
  print('\n')
  print('if a file exists in the target installation location')
  print('do you want to be asked before it is overwritten by the') 
  print('source file being installed ?')
  print('v - never. always replace with source')
  print('o - only if source file is older than target')
  print('a - always ask regardless of file dates/times')
   
  overwrite='x'
  while(overwrite not in ['v','o','a']):
    overwrite=input('enter v,o or a [o]').lower()
    if not overwrite:
      overwrite='o'
  if overwrite == 'v':
    ovpol.always_ask=False
    ovpol.always_replace=True
    ovpol.ask_if_older=False 
  elif overwrite == 'o':
    ovpol.always_ask=False
    ovpol.always_replace=False
    ovpol.ask_if_older=True 
  elif overwrite == 'a':
    ovpol.always_ask=True
    ovpol.always_replace=False
    ovpol.ask_if_older=False 

def get_usergroup():
  import pwd # Unix password database
  import grp # Unix group database
  while(True):
    user=input(
      'which user pylets will run as ['+ri_default_user+'] ')
    if not user:
      user=ri_default_user
    try:
      uid=pwd.getpwnam(user)[2] # lookup uid based on user name
    except KeyError:
      print('user: '+user+' is not a valid user')
    else:
      break
  while(True): 
    group=input(
      'which group will pylets run as ['+ri_default_group+'] ')
    if not group:
      group=ri_default_group
    try:
      gid=grp.getgrnam(group)[2] # lookup gid based on group name
    except KeyError:
      print('group: '+group+' is not a valid group')
    else:
      break
  return (uid,gid)

def clean_junk(): # gets rid of .pyc files created within source directory
  hitlist=[]
  import glob
  hitlist+=glob.glob(os.path.join(cwd,'*.pyc')) # compiled python files
  for filen in hitlist:
    print('removing: '+filen)
    os.unlink(filen)

def main():
  ''' main function for make_install.py '''
  print('\nmake_install.py\n')
  print('Default values for various prompts are given in [brackets]')
  print('To accept the defaults press enter.')
  if not os.environ['USER'] == 'root':
    if not want_user_install():
      print('OK rerun this program again as root')
      return # no more to do
    else:
      root_install=False
      uid=0 # uid/gid will not be used for user install
      gid=0
  else:
    root_install=True
    (uid,gid)=get_usergroup()

  # configuration directory 

  helpinfo={
  'help':'''The configuration directory is used to store site
            and security variables. This should not be directly
            accessible within the web server document root.''',
  'prompt':'configuration directory path'}
  rdl=ri_default_configdir
  udl=os.path.join(home,'pylets_configs') 
  cdir=getmakedir(root_install,helpinfo,rdl,udl,uid,gid,dcmode=0o770)
  print('config dir: '+cdir)
  rewrite_default_path(cdir,'cfg_directory')

  # data directory

  helpinfo={
  'help':'''The data directory is used to store the database 
            tables containing accounting, member and trading records. 
            This should not be directly accessible within the web server 
            document root.''',
  'prompt':'data directory path'}
  rdl=ri_default_datadir
  udl=os.path.join(home,'pylets_data') 
  ddir=getmakedir(root_install,helpinfo,rdl,udl,uid,gid,dcmode=0o770)
  print('data dir: '+ddir)
  rewrite_default_path(ddir,'data_directory')
  
  # software directory

  helpinfo={
  'help':'''
The software directory is used to store the Python 
programs which are run as modules or directly by the
webserver using the CGI (Common Gateway Interface). 
This directory must be within the part of the  
web server document root to which you have write
and upload access, i.e. to upload webpages and other 
files for your website. 
 
If for example the document root for your website is: /var/www/youruserid 
and the URL at which you want the software to be accessed is: 
http://example.com/pylets , then you would enter: 
/var/www/youruserid/pylets for the software directory path. ''',
  'prompt':'software directory path'}
  rdl=ri_default_softdir
  udl=os.path.join('/var/www/'+user,'pylets') 
  sdir=getmakedir(root_install,helpinfo,rdl,udl,uid,gid,dcmode=0o775)
  print('software dir: '+sdir)
  rewrite_default_path(sdir,'sw_directory')

  # backup configured default_paths.py to default_paths_live.py
  # Note that default_paths will be reconfigured when going
  # between test and live operational modes. 
  def_paths_fn='default_paths.py'
  def_paths_live_fn='default_paths_live.py'
  backupcmd="cp "+def_paths_fn+" "+def_paths_live_fn
  os.system(backupcmd)  

  # create directories relative to sdir for testsuite, forms and package_defaults
  
  for subdir in (testsuite,forms,test_data,test_expected,test_configs,package_defaults):
    fullpath=os.path.join(sdir,subdir) 
    if not os.path.isdir(fullpath):
      os.makedirs(fullpath,0o770) 
    os.chown(fullpath,uid,gid)

  # set file overwrite policies
  file_overwrite_policy(ovpol)

  print('ovpol.always_ask: '+ str(ovpol.always_ask)) 
  print('ovpol.always_replace: '+str(ovpol.always_replace))
  print('ovpol.ask_if_older: '+str(ovpol.ask_if_older))

  # install files in software directory

  import glob
  paths=[]
  paths+=glob.glob(os.path.join('.','*.py')) # executables 
  paths+=glob.glob(os.path.join('.','*.cgi')) # executables 
  for filen in paths: 
    basename=os.path.basename(filen)  
    install_file(basename,cwd,sdir,ovpol,0o750,uid,gid,root_install)

  paths=[]
  paths+=glob.glob(os.path.join('.','*.html')) # non-executables 
  paths+=glob.glob(os.path.join('.','*.txt')) 
  paths+=glob.glob(os.path.join('.','README')) 
  paths+=glob.glob(os.path.join('.','CHANGE*')) 
  for filen in paths: 
    basename=os.path.basename(filen)  
    install_file(basename,cwd,sdir,ovpol,0o640,uid,gid,root_install)

  # install files into configs directory

  for filen in glob.glob(os.path.join(package_defaults,'*.cfg')):
    basename=os.path.basename(filen)  
    install_file(
          basename,os.path.join(cwd,package_defaults),cdir,ovpol,0o640,uid,gid,root_install)

  # install files in forms directory

  tformsdir=os.path.join(sdir,forms) # target forms directory
  for filen in glob.glob(os.path.join(forms,'*.form')):
    basename=os.path.basename(filen)  
    install_file(
      basename,os.path.join(cwd,forms),tformsdir,ovpol,0o640,uid,gid,root_install)
  for filen in glob.glob(os.path.join(forms,'*.form.a*')):
    basename=os.path.basename(filen)  
    install_file(
      basename,os.path.join(cwd,forms),tformsdir,ovpol,0o640,uid,gid,root_install)

  # install files into data directory needed for starting a clean system

  for filen in glob.glob(os.path.join(package_defaults,'*.pkl')):
    basename=os.path.basename(filen)  
    install_file(
      basename,os.path.join(cwd,package_defaults),ddir,ovpol,0o640,uid,gid,root_install)

  # install files into package_defaults directory

  pdefdir=os.path.join(sdir,package_defaults) # target package_defaults directory
  paths=[]
  paths+=glob.glob(os.path.join(package_defaults,'*.cfg')) # default configs 
  paths+=glob.glob(os.path.join(package_defaults,'*.pkl')) # default data 
  paths+=glob.glob(os.path.join(package_defaults,'README')) # directory README 
  for filen in paths:
    basename=os.path.basename(filen)  
    install_file(
      basename,os.path.join(cwd,package_defaults),
      pdefdir,ovpol,0o640,uid,gid,root_install)

  # install files in testsuite directory

  tsopt=input('do you want to install the test suite ? y/n').lower()
  if tsopt == 'y':
    # install test case python files e.g. test1.py
    tsuitedir=os.path.join(sdir,testsuite) # target testsuite directory
    paths=[]
    paths+=glob.glob(os.path.join(testsuite,'*.py')) # executables 
    for filen in paths:
      basename=os.path.basename(filen) 
      #print 'basename: '+basename # debug 
      install_file(
        basename,os.path.join(cwd,testsuite),tsuitedir,ovpol,0o750,uid,gid,root_install)
    # install test configuration files
    paths=[]
    paths+=glob.glob(os.path.join(testsuite,'*.cfg')) #  configurations 
    paths+=glob.glob(os.path.join(testsuite,'README')) 
    for filen in paths:
      basename=os.path.basename(filen) 
      #print 'basename: '+basename # debug 
      install_file(
        basename,os.path.join(cwd,testsuite),tsuitedir,ovpol,0o640,uid,gid,root_install)
    # install expected test results files
    expecteddir=os.path.join(sdir,test_expected) # target test expected results directory
    paths=[]
    paths+=glob.glob(os.path.join(test_expected,'*.tst')) #  expected results 
    for filen in paths:
      basename=os.path.basename(filen) 
      #print 'basename: '+basename # debug 
      install_file(basename,os.path.join(cwd,test_expected),
                   expecteddir,ovpol,0o640,uid,gid,root_install)
    # install test data files - different location than for live data 
    testdatadir=os.path.join(sdir,test_data) # target test data directory
    for filen in glob.glob(os.path.join(package_defaults,'*.pkl')):
      basename=os.path.basename(filen)  
      install_file(
        basename,os.path.join(cwd,package_defaults),testdatadir,
        ovpol,0o640,uid,gid,root_install)
    # install test config files - different location than for live configs 
    testcfgdir=os.path.join(sdir,test_configs) # target test configs directory
    for filen in glob.glob(os.path.join(package_defaults,'*.cfg')):
      basename=os.path.basename(filen)  
      install_file(
        basename,os.path.join(cwd,package_defaults),testcfgdir,
        ovpol,0o640,uid,gid,root_install)
  clean_junk() # get rid of .pyc files created within source directory

if __name__ == "__main__":
  try:
      main()
  except:
    import traceback
    print("error detected in make_install.py main()")
    traceback.print_exc()
