#!/usr/bin/env python

'''
routine.py - does maintenance jobs on a regular basis
this is called using scripts in /etc/cron.hourly, 
/etc/cron.daily and cron.monthly . It is questionable 
whether jobs done less frequently than that should be automated.
'''

import cgiutils
import tablcgi
import authent
import time
import sys
import os 
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']

def inform_waiting_applications():
  ''' Checks if any applicants are awaiting approval/rejection
  and emails designated address if any are. '''
  
  # first check if an email approval reminder address is configured
  to_ad=v['approval_e']
  if not cgiutils.is_email(to_ad):
    # a valid email address has not been configured so no more to do
    return
  from table_details import apptab
  applications=tablcgi.table(apptab,dir=dl)
  confirmed='' # list of applicants names with confirmed email addresses
  nconfirmed=0
  for row in applications.data :
    if row['confirmed'] == 'Y':
      nconfirmed+=1
      name=row['name']
      email=row['email']
      applicant=name+' <'+email+'>\n'
      confirmed+=applicant
  if nconfirmed == 0:
    # no applications, nothing to do
    return 
  # applications found so inform administrator
  from_ad=v['admin_e']
  message="From: "+v['registry']+" Applications <"+from_ad+">\n"
  to_line="To: %s\n" % to_ad
  message+=to_line
  message+="Subject: "+v['registry']+" applications awaiting approval \n\n"
  if nconfirmed == 1:
    message+="This is a daily reminder that 1 application \n" 
    message+="is awaiting your approval. The site address is: \n"
    message+=v['homepage']+"\n"
    message+="\nThe name and email address of the applicant is :\n"
    message+="\n"
  else:
    message+="This is a daily reminder that %d applications \n" % nconfirmed 
    message+="are awaiting your approval. The site address is: \n"
    message+=v['homepage']+"\n"
    message+="\nThe names and email addresses of the applicants are :\n"
    message+="\n"
  message+=confirmed
  message+="\n"
  cgiutils.send_mail(from_ad,to_ad,message)
  return

def do_log_maintenance():
  mode=open("mode.txt").read()
  if mode[-1] == '\n':
    mode=mode[:-1]
  if mode not in ['test','live']:
    raise ValueError("invalid mode.txt value must be test or live")
  if mode == 'live': # don't want unexpected changes while testing
    authent.maintenance()

def do_hourly():
  do_log_maintenance()

def do_daily():
  inform_waiting_applications()

def do_monthly():
  pass

def main():
  if len(sys.argv) < 2:
    print("usage:\nroutine.py hourly|daily|monthly")
    return 
  option=sys.argv[1]
  if option.lower() == 'hourly':
    do_hourly()
  elif option.lower() == 'daily':
    do_daily()
  elif option == 'monthly':
    do_monthly()
  return
 
if __name__ == "__main__":
  try:
    main()
  except:
    import traceback
    print("error detected in routine.main()")
    traceback.print_exc()

