#!/usr/bin/env python

# change next line to debug=1 to debug script

debug=0

''' security.cgi used to view login security records. 
    These come in 4 flavours:
    a. log records for each login attempt
    b. summaries of attempts per IP record  
    c. summaries of attempts per account
    d. overrides showing which account/address pairs are overridden  
    
    The security admin may add override records.

    Richard Kay last changed: 3 Apr 2008
'''

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import mtab,logins,gbips,lacs,ovrid,smaint
site_vars=cgiutils.get_vars('site.cfg')
dl=site_vars['data_location']
sec_vars=cgiutils.get_vars('secure.cfg')

def main():
  session=False
  cgiutils.html_header(title="LETS Member Security Records",
                       bgcolor='"#FFFFFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  valid,option=cgiutils.get_option(woi,buttons=[
      "logins","gbips","lacs","vover","aover","delete","vmaint"])
  if not valid:
    return # error message already sent
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,error="Invalid or stale or logged out session.")
    return
  #
  # only do work for authorised accounts
  if not cgiutils.auth(session,ac_id,"SECURITY",
        "viewing login security form"):
    return False
  #  
  if option == "logins":
    if "logfilename" not in woi.keys():
      # if archived log not selected display current log by default
      lins=tablcgi.table(logins,dir=dl)
      lins.tab2html(skip_cols=['cookie'])
    else:
      logfilename=woi.firstval("logfilename")
      if not cgiutils.is_valid(logfilename,max_leng=14,
                    allowed='^logins(\.[0-9]+)?\.pkl$'):
        cgiutils.html_end(session,error="Invalid login archive filename.")
        return
      import os.path
      full_logfilename=os.path.join(dl,logfilename)
      if not os.path.isfile(full_logfilename):
        cgiutils.html_end(session,
           error="Login archive filename is not a file.")
        return
      # clone the logins table definition object before
      # overriding the logins.file attribute, to avoid poisoning use of this
      # object attribute elsewhere
      clogins=tablcgi.table_def(logins)
      clogins.file=full_logfilename
      lins=tablcgi.table(clogins,dir=dl)
      lins.tab2html(skip_cols=['cookie'])
    cgiutils.html_end(session)
  elif option == "gbips":
    gbis=tablcgi.table(gbips,dir=dl)
    gbis.tab2html()
    cgiutils.html_end(session)
  elif option == "lacs":
    lac=tablcgi.table(lacs,dir=dl)
    lac.tab2html()
    cgiutils.html_end(session)
  elif option == "vover":
    over=tablcgi.table(ovrid,dir=dl)
    over.tab2html()
    cgiutils.html_end(session)
  elif option == "vmaint":
    smain=tablcgi.table(smaint,dir=dl)
    smain.tab2html()
    cgiutils.html_end(session)
  elif option == "delete":
    if "del_ovid" not in woi.keys():
      cgiutils.html_end(session,error="Delete attempted but no Override ID.")
      return
    # validate del_ovid field
    delov=cgiutils.stoi(woi,"del_ovid")
    if type(delov) == type(None): # true if not an integer
      cgiutils.html_end(session,error="Invalid Override id")
      return
    ovs=tablcgi.table(ovrid,dir=dl)
    ovidx=ovs.find(delov)
    if ovidx < 0:
      cgiutils.html_end(session,
         error="Delete attempted but Override ID not found.")
      return
    ovs.delrow(delov)
    cgiutils.html_end(session,received=1)
    return

  elif option == "aover":
    if not woi.has_required(["accrej","ovac","ipaddr"]):
      cgiutils.html_end(session,
         error="Missing value/s. Complete all form fields.")
      return
    # check field formats
    ovac=woi.firstval("ovac")
    if not cgiutils.is_valid(ovac,max_leng=8,allowed='^[\w]+$'):
      cgiutils.html_end(session,error="Invalid override account id/no.")
      return
    accrej=woi.firstval("accrej")
    if not cgiutils.is_valid(accrej,max_leng=6,allowed='^accept$|^reject$'):
      cgiutils.html_end(session,
         error="This must be either an accept or reject.")
      return
    ipaddr=woi.firstval("ipaddr")
    print("ipaddr: '"+ipaddr+"'") # debug
    if not ( cgiutils.is_ipv4addr(ipaddr) or cgiutils.is_ipv6addr(ipaddr)) :
      cgiutils.html_end(session,error="Invalid IP address.")
      return
    if not woi.has_required(["expiry"]):
      expiry=None # will default to 24 hours
    else:
      expiry=woi.firstval("expiry")
    if expiry and not cgiutils.is_valid(expiry,max_leng=20,
      allowed='^[0-9]{1,2} [A-Za-z]{3,10} [0-9]{4,4}$'):  
      cgiutils.html_end(session,
         error="Unparseable expiry date format error 1.")
      return
    # convert expiry date into float format
    import time
    posted=int(time.time()) # when posted
    if expiry:
      try:
        to=time.strptime(expiry,"%d %b %Y")
        expirytime=int(time.mktime(to))
      except(ValueError,OverflowError):
        cgiutils.html_end(session,
           error="Unparseable expiry date format error 2.")
        return
      if expirytime < posted:
        cgiutils.html_end(session,
           error="expiry date/time before current date/time.")
        return
    else:
      expirytime=posted+(24*60*60) # default expiry of override in 24 hours 
    orow={
        "ar":accrej,
        "ac_id":ovac,
        "ip":ipaddr,
        "when":posted,
        "expiry":expirytime }
    ovs=tablcgi.sq_table(ovrid,dir=dl)
    ovid=ovs.get_nextnum()
    ovs.addrow(orow)
    print("<p> Override: "+str(ovid)+" successfully input.</p>")
    cgiutils.html_end(session,received=1)
  return

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in security.cgi main()")
    traceback.print_exc()

