#!/usr/bin/env python
#
#  table_details.py: details of tables used within LETS CGI 
# simple accounts management system. These details are coded 
# in this file and imported into programs which access
# tables, enabling table metadata and definitions, together
# with table/column specific presentation code to be edited in one place.

# trivial table definitions class used to simplify naming
class table_def:
  pass

# code for printing human readable date/time from internal format 

def print_url(url):
  ''' function used for url output '''
  import cgiutils
  if url == 'None':
    return ''
  elif cgiutils.is_url(url):
    return '<a href="'+url+'">link</a>\n'
  else:
    return ''

def show_picture(picfile):
  if picfile == 'None':
    return ''
  else:
    return '<img src="'+picfile+'" width=100 height=120 alt="" />\n' 

# code for printing human readable date/time from internal format 

def print_date_time(timeval):
  ''' function used for ack tables date output '''
  import time
  time_struct=time.localtime(timeval)
  return str(time.asctime(time_struct))+'\n'

def print_date(timeval):
  ''' function used for wantoffer tables date output '''
  import time
  time_struct=time.localtime(timeval)
  # output time in DD Mmm YYYY format e.g 12 Jan 2007
  return str(time.strftime("%d %b %Y",time_struct))+'\n'


def print_skill(level):
  ''' code for printing skill level '''
  if level == 'P':
    return "Professional\n"
  elif level == 'S':
    return "Semi skilled\n"
  elif level == 'B':
    return "Beginner\n"
  else: 
    return "N/A\n"

def digest_cookie(cookie):
  ''' code for displaying last part of a md5 digested cookie, so 
      the real cookie is not visible '''
  import md5
  ho=md5.md5(cookie)
  return ho.hexdigest()[-16:] # prints last 16 hex digits 

# globals to help speed comparison functions
import re
cre_int=re.compile('^[0-9]+$')

def cmp(x, y):
    """
    Replacement for built-in function cmp that was removed in Python 3
    Compare the two objects x and y and return an integer according to
    the outcome. The return value is negative if x < y, zero if x == y
    and strictly positive if x > y.
    """
    return (x > y) - (x < y)

def cmp_acid(row1,row2):
  ''' comparison function to enable pretty sorting tables keyed on 
      ac_id field. This can be of the form "admin" (string) or "123"
      (integer value stored as a string). Int comparison is used
      if the ac_id's are both integers stored as strings.'''
  ac1=row1["ac_id"] 
  ac2=row2["ac_id"] 
  imo1=cre_int.match(ac1)
  imo2=cre_int.match(ac2)
  if imo1 and imo2:
    return cmp(int(ac1),int(ac2))
  else: 
    return cmp(ac1,ac2)

def cmp_categories(row1,row2):
  ''' comparison function to enable integer sorting of category sort 
      field stored as decimal string '''
  return cmp(int(row1["sort"]),int(row2["sort"])) 


# data for members table
mtab=table_def()
mtab.meta={
  "keys":["ac_id","pin","name","email","active","phone1","phone2","address","postcode","district","url","pic"],
  "colheads":["A/C","PIN","Name","email","Active","Phone 1","Phone 2","Address","Postcode","District","URL","Photo"],
  "formats":["%s","%s","%s","%s","%s","<pre>%s</pre>","<pre>%s</pre>",
             "<pre>%s</pre>","<pre>%s</pre>","%s",print_url,show_picture]}
mtab.file="members.pkl"
mtab.index_col="ac_id"

# data for applications table
apptab=table_def()
apptab.meta={
  "keys":["token","confirmed","handle","name","email","phone1","phone2","address","postcode","district","other","time"],
  "colheads":["Token","Confirmed","Handle","Name","email","Phone 1","Phone 2","Address","Postcode","District","Other Info","Date/Time Applied"],
  "formats":["%s","%s","%s","%s","%s","<pre>%s</pre>","<pre>%s</pre>",
      "<pre>%s</pre>","<pre>%s</pre>","%s","<pre>%s</pre>",print_date_time ]}
apptab.file="applications.pkl"
apptab.index_col="email"

# data for contacts view table
ctab=table_def()
ctab.meta={
  "keys":["ac_id","name","email","active","phone1","phone2","url","pic"],
  "colheads":["A/C","Name","email","Active","Phone 1","Phone 2","Homepage","Photo"],
  "formats":["%s","%s","%s","%s","<pre>%s</pre>","<pre>%s</pre>",print_url,show_picture ]}
ctab.file=None
ctab.index_col="ac_id"

# data for address labels view table. 
# Label comprises name, address and postcode
latab=table_def()
latab.meta={
  "keys":["0","1","2","3"],
  "colheads":["Label 1","Label 2","Label 3","Label 4"],
  "formats":["%s","%s","%s","%s" ]}
latab.file=None
latab.index_col="0"

# data for balances table
# limit column added by RK 7 Feb 2010
# limit to be converted to float if not "None" 
btab=table_def()
btab.meta={
  "keys":["ac_id","cfwd","balance","in","out","turnover","oldturn","limit"],
  "colheads":["A/C","CFWD","Balance","In","Out","Turnover","Previous Turnover","Limit"],
  "formats":["%s","%.2f","%.2f","%.2f","%.2f","%.2f","%.2f","%s"]}
btab.file="balances.pkl"
btab.index_col="ac_id"

# data for acknowledgements table

atab=table_def()
atab.meta={
  "keys":["ack_id","from_id","from_name","to_id","to_name","system",
  "amount","details","time"],
  "colheads":["ack_id","from_A/C","from_name","to_A/C","to_name","System",
  "Amount","Details","Date/Time Recorded"],
  "formats":["%s","%s","%s","%s","%s","%s","%.2f","%s",print_date_time]}
atab.file="acks.pkl"
atab.sqfile="acks_sqk.pkl"
atab.index_col="ack_id"

# data for summary table
stab=table_def()
stab.meta={
  "keys":["ac_id","name","cfwd","balance","in","out","turnover","oldturn"],
  "colheads":["A/C","Name","CFWD","Balance","In","Out","Recent Turnover","Past Turnover"],
  "formats":["%s","%s","%.2f","%.2f","%.2f","%.2f","%.2f","%.2f"]}
stab.file="turnover.pkl"
stab.index_col="ac_id"

# data for wantsoffers table
wotab=table_def()
wotab.meta={
  "keys":["wo_id","ac_id","categ","wo","skill","desc","posted","expiry"],
  "colheads":["W/O ID","A/C","Category","W/O","Skill","Description","Posted","Expires"],
  "formats":["%s","%s","%s","%s",print_skill,"%s",print_date,print_date]}
wotab.file="wantsoffers.pkl"
wotab.sqfile="wantsoffers_sqk.pkl"
wotab.index_col="wo_id"

# metadata for wantsoffers view 
woview=table_def()
woview.meta={
  "keys":["wo_id","ac_id","categ","wo","skill","desc",
          "posted","expiry","district", "name","email","phone"],
  "colheads":["W/O ID","A/C","Category","W/O","Skill","Description",
              "Posted","Expires","District","Name","Email","Phone"],
  "formats":["%s","%s", "%s","%s",print_skill,"%s",
             print_date,print_date,"%s","%s","%s","%s"]}
woview.file=None
woview.sqfile=None
woview.index_col="wo_id"

# data for categories table
cattab=table_def()
cattab.meta={
  "keys":["sort","category","desc"],
  "colheads":["Sort","Category","Description"],
  "formats":["%d","%s","%s"] }
cattab.file="categories.pkl"
cattab.index_col="category"

# data for logins table
logins=table_def()
logins.meta={
  "keys":["lid","ac_id","cookie","ac_exists","goodlogin","reason","badpin","ip","when"],
  "colheads":["Login ID","Account","Cookie Digest","A/C exists","Good Login","Reject Reason",
              "Bad Pin Digest","IP Address","Date/Time"],
  "formats":["%s","%s",digest_cookie,"%s","%s","%s","%s","%s",print_date_time] }
logins.file="logins.pkl"
logins.sqfile="logins_sqk.pkl"
logins.index_col="lid"

# data for good/bad IP address login attempts table
gbips=table_def()
gbips.meta={
  "keys":["ip","glh","gld","glw","glm","gly","blh","bld","blw","blm","bly"],
  "colheads":["IP Address","Good/Hour","Good/Day","Good/Week","Good/Month",
               "Good/Year","Bad/Hour","Bad/Day","Bad/Week","Bad/Month","Bad/Year"],
  "formats":["%s","%d","%d","%d","%d","%d","%d","%d","%d","%d","%d"] }
gbips.file="gbips.pkl"
gbips.index_col="ip"

# data for good/bad account login attempts table
lacs=table_def()
lacs.meta={
  "keys":["ac_id","glh","gld","glw","glm","gly","blh","bld","blw","blm","bly"],
  "colheads":["Account","Good/Hour","Good/Day","Good/Week","Good/Month",
               "Good/Year","Bad/Hour","Bad/Day","Bad/Week","Bad/Month","Bad/Year"],
  "formats":["%s","%d","%d","%d","%d","%d","%d","%d","%d","%d","%d"] }
lacs.file="lacs.pkl"
lacs.index_col="ac_id"

# data for security overrides used to cancel or force auto lockouts 
ovrid=table_def()
ovrid.meta={
  "keys":["ovid","ar","ac_id","ip","when","expiry"],
  "colheads":["Override ID","Accept/Reject","Account",
                            "IP Address","Overridden","Expires"],
  "formats":["%s","%s","%s","%s",print_date_time,print_date_time] }
ovrid.file="overrides.pkl"
ovrid.sqfile="overrides_sqk.pkl"
ovrid.index_col="ovid"

# data for security table maintenance control
# intended as a single record so key is arbitrary

smaint=table_def()
smaint.meta={
  "keys":["lrotate","nrotate","lgbr","ngbr"],
  "colheads":["Last Rotation","Next Rotation","Last G/B Rebuild","Next G/B Rebuild"],
  "formats":[print_date_time,print_date_time,print_date_time,print_date_time] }
smaint.file="smaint.pkl"
smaint.index_col="lrotate"

# data for audited actions log

aactions=table_def() 
aactions.meta={
  "keys":["aaid","ac_id","action","when","whom","auth","details"],
  "colheads":["Action ID","Who","Did What","When","For Whom","Valid","Details"],
  "formats":["%d","%s","%s",print_date_time,"%s","%s","%s"] }
aactions.file="aactions.pkl"
aactions.sqfile="aactions_sqk.pkl"
aactions.index_col="aaid"

# data for authorisations table
authtab=table_def()
authtab.meta={
  "keys":["authac","auth","acbe","whom"],
  "colheads":["Key","Authority","Account Authorised","On Behalf Of"],
  "formats":["%s","%s","%s","%s"] }
authtab.file="authorisations.pkl"
authtab.index_col="authac"

