#!/usr/bin/env python

# change next line to debug=1 to debug script
debug=0

""" tdirectory.cgi This file handles trading directory options 
    Richard Kay last changed: 01 Aug 2008
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import mtab,wotab,cattab,woview
v=cgiutils.get_vars('site.cfg')
dl=v['data_location']
sec_vars=cgiutils.get_vars('secure.cfg')

def remove_expired(members):
  '''removes expired directory entries from table
     prior to display. An entry expires for 1 of
     3 reasons:
     i. entry times out.
     ii. member ac_id becomes inactive.
     iii. member ac_id no longer in system.  '''
  wos=tablcgi.table(wotab,dir=dl,save_asap=0)
  import time
  timenow=time.time()
  delrows=[] # list of keys to delete
  for wrow in wos.data:
    wo_id=wrow["wo_id"]
    expiry=wrow["expiry"]
    if expiry < timenow:
      delrows.append(wo_id)
    ac_id=wrow["ac_id"]
    mindex=members.find(ac_id)
    if mindex == -1: # want/offer for nonmember
      delrows.append(wo_id)
#   comment out following 4 rows if inactive wants/offers should not go 
    else: 
      active=members.data[mindex]["active"]
      if not active in ['Y','P'] : # delete want/offer for inactive member 
        delrows.append(wo_id)
  # remove outdated rows from wantsoffers file
  for wo_id in delrows:
    wos.delrow(wo_id)
  wos.save()

def main():
  session=False
  cgiutils.html_header(title="LETS Directory Access",bgcolor='"#DDFFFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  valid,option=cgiutils.get_option(woi,buttons=[
      "add","viewdata","vwo","delete","vcats","catman"])
  if not valid:
    return # error message already sent
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="Invalid or stale or logged out session.")
    return
  # 
  members=tablcgi.table(mtab,dir=dl)
  if option == "add":
    # make category selection HTML string
    categtab=tablcgi.table(cattab,dir=dl)
    selectcategories=cgiutils.make_select(categtab,instruction=
      "trading category required",
      cgiparameter="category")
    # send the wantoffer form
    print(cgiutils.send_form("wantoffer.form",[v['registry'],ac_id,
      cookie,ua,selectcategories,None],ua=ua))
    cgiutils.html_end(session,want_form=1)
  elif option == "delete":
    if "del_woid" not in woi.keys():
      cgiutils.html_end(session,
         error="Delete attempted but no Want/Offer ID.")
      return
    # validate del_woid field
    delwo=cgiutils.stoi(woi,"del_woid")
    if type(delwo) == type(None): # true if not an integer
      cgiutils.html_end(session,error="Invalid Want/Offer id")
      return
    wos=tablcgi.table(wotab,dir=dl)
    woidx=wos.find(delwo)
    if woidx < 0:
      cgiutils.html_end(session,
         error="Delete attempted but Want/Offer ID not found.")
      return
    wo_ac=wos.data[woidx]["ac_id"]
    if ac_id == wo_ac:
      cgiutils.auth(session,ac_id,"WODEL","deleting own w/o: "+str(delwo),
                    whom=wo_ac,auditonly=True )
    elif cgiutils.auth(session,ac_id,"WODEL",str(delwo),whom=wo_ac):
      pass # OK was authorised
    else: 
      return # not authorised
    wos.delrow(delwo)
    cgiutils.html_end(session,received=1)
    return
  elif option == "viewdata":
    # just displays table data not including names
    # remove expired entries
    remove_expired(members) 
    # output to browser
    wos=tablcgi.table(wotab,dir=dl)
    wos.sort(key="wo")
    wos.sort(key="categ")
    wos.tab2html()
    cgiutils.html_end(session)
  elif option == "vwo":
    # full listing includes view of member district+names+phone+email
    # remove expired entries
    remove_expired(members)
    # check if all or just one category required
    if "category" not in woi.keys():
      category='all'
    else:
      category=woi.firstval("category")
    if "whom" not in woi.keys():
      allusers=True
    else:
      allusers=False
      whom=woi.firstval("whom")
      if not cgiutils.is_valid(whom,allowed='^\w+$'):
        cgiutils.html_end(session,error="Invalid enquiry account identifier.")
        return
    wos=tablcgi.table(wotab,dir=dl)
    wov=tablcgi.table(woview)
    # adjust view meta data to allow for row coloration
    wov.meta["keys"].append("bgcolor")
    wov.meta["colheads"].append("Bgc")
    wov.meta["formats"].append("%s")
    # copy directory records required full view 
    for row in wos.data:
      if (( category == 'all' or row["categ"] == category ) and 
          ( allusers or row["ac_id"] == whom )) :
        mindex=members.find(row["ac_id"])
        district=members.data[mindex]["district"]
        name=members.data[mindex]["name"]
        email=members.data[mindex]["email"]
        phone=members.data[mindex]["phone1"]
        wodrow={}
        for key in ["wo_id","ac_id","categ","wo","skill",
                    "desc","posted","expiry"]:
          wodrow[key]=row[key]
        wodrow["district"]=district
        wodrow["name"]=name
        wodrow["email"]=email
        wodrow["phone"]=phone
        active=members.data[mindex]["active"]
        wodrow["bgcolor"]=cgiutils.get_color(active,
                          default_color='"#FFFFFF"')
        wov.addrow(wodrow)
    # output to browser
    wov.sort(key="ac_id")
    wov.sort(key="wo")
    wov.sort(key="categ")
    cgiutils.print_colortable()
    wov.tab2html(skip_cols=["bgcolor"],color_row_key="bgcolor")
    cgiutils.html_end(session)
  elif option == "vcats":
    # view category table 
    cats=tablcgi.table(cattab,dir=dl)
    import table_details
    cats.sort(cmpfunc=table_details.cmp_categories)
    cats.tab2html()
    cgiutils.html_end(session)
  elif option == "catman":
    print(cgiutils.send_form("tcategories.form",[v['registry'],ac_id,
      cookie,ua]))
    cgiutils.html_end(session,received=1)
  return

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in tdirectory.cgi main()")
    traceback.print_exc()

