#!/usr/bin/env python
#
''' test.cgi
    collects the privileged utilities needed for testsuite to
    function into a single program. The same functionality
    was previously provided by make_clean.cgi, get_nextackid.cgi
    and get_userpin.cgi. Leaving this program live and
    executable would massively compromise a production system
    so don't do that. Only let this one be run in testing mode.
    See also that whether make_clean.py leaves this program
    executable or not depends upon tsmode parameter to 
    the make_clean.main() call ''' 

import sys
print("Content-type: text/plain\n") # don't need html for these functions
# change line to debug script
debug=False
if debug:
  sys.stderr=sys.stdout

import tablcgi
import cgiutils
import webinput
woi=webinput.Webinput()

v=cgiutils.get_vars('site.cfg')
dl=v['data_location']
from table_details import atab,mtab,apptab

def get_nextackid():
  ''' Gets next ackid for automated testing purposes.''' 
  acks=tablcgi.sq_table(atab,dir=dl)  
  next_ackid=acks.get_nextnum()
  print(next_ackid)

def get_userpin():
  ''' Gets user pin for automated testing purposes.'''
  members=tablcgi.table(mtab,dir=dl)
  if not woi.has_required(["ac_id"]):
    print("no account")
    return
  ac_id=woi.firstval("ac_id")
  if not cgiutils.is_valid(ac_id,allowed='^\w+$'):
    print("invalid account")
    return
  index=members.find(ac_id)
  if index == -1 :
    print("no such account")
    return
  pin=members.data[index]["pin"]
  print(pin)
  return

def make_clean():
  ''' wraps make_clean.py functionality to reset system
      to a pristine state, either for testing or secure but
      not both.  '''
  print('test.cgi make_clean keys: ' +str(woi.keys()))
  if 'testing' in woi.keys():
    mode='testing'
  else:
    mode='secure'
    # problems picking up key temp fix
    mode='testing'
  import make_clean # functionality is provided by make_clean.py
  make_clean.main(tsmode=mode)

def get_confirmcode():
  ''' Gets application confirmation code for automated testing purposes.'''
  applications=tablcgi.table(apptab,dir=dl)
  if not woi.has_required(["email"]):
    print("no email address")
    return
  email=woi.firstval("email")
  if not cgiutils.is_email(email):
    print("invalid email address")
    return
  index=applications.find(email)
  if index == -1 :
    print("no such application")
    return
  ccode=applications.data[index]["token"]
  print(ccode) 
  return

def main():
  keys=woi.keys()
  if 'make_clean' in keys:
    make_clean()
  elif 'get_userpin' in keys:
    get_userpin()
  elif 'get_nextackid' in keys:
    get_nextackid()
  elif 'get_confirmcode' in keys:
    get_confirmcode()
  else:
    print("error: test.cgi run without required key") 

if __name__ == "__main__":
  try:
    main()
  except:
    import traceback
    print("error detected in test.cgi main()")
    traceback.print_exc()
