#!/usr/bin/env python
''' test1.py part of Test suite for PyLETS CGI 

    test1.py - test4.py are designed to be used
    as part A of the suite.
    
    Purpose of tests: 
    Features in entry.cgi
    a. error handling,
    b. ability to send requested PIN by email and
    c. ability to output various forms.

    Prerequisites: 
    a. blank system
    b. admin_has_superpowers=yes 
    Postrequisite: account for joe must exist
'''

def main(change_expected=False):
  print("\nTest Case 1: entry.cgi errors, PIN send and form o/p\n")

  import testlib
  testlib.modify_config_var('test_configs/secure.cfg',
      'admin_has_superpowers','yes')
  # get test configuation variables in dictionary v
  v=testlib.get_vars()
  tv=testlib.get_vars(cfile='test.cfg')
  v['pin']=testlib.run_make_clean()
  ackf_select=testlib.ackfiles_select() # need this to know accounts form
  emd=tv['email_domain'] # test email domain
  tests=[]

  # test 1: obtain entry form
  testno=1
  entryform=testlib.get_form('entry.form',vars=[v['registry']],ua='user')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'',
	      'expected':entryform,
	      'testno':testno,
	      'description':'test sending entry form' })

  # test 2: no account
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'this=that',
	      'expected':"<p>  You havn't input your account no. or identifier. </p>",
	      'testno':testno,
	      'description':'test no account field' })

  # test 3: invalid account id : contains non word character
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=non*word',
	      'expected':"Invalid account number/identifier.",
	      'testno':testno,
	      'description':'non word char in account field' })

  # test 4: invalid option : not on list of submit buttons 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=word&this=that',
	      'expected':"Invalid submit button value.",
	      'testno':testno,
	      'description':'invalid option, not on list of submit buttons' })

  # test 5: invalid PIN  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=word&user=user&pin=abcd',
	      'expected':"Invalid PIN format.",
	      'testno':testno,
	      'description':'invalid PIN, not in range 10000000 - 99999999' })

  # test 6: unknown account  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=noac&user=user&pin=12345678',
	      'expected':"Incorrect PIN and/or account entered, or account inactive.",
	      'testno':testno,
	      'description':'unknown account' })

  # test 7: wrong PIN entered  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=admin&admin=admin&pin=%s' % str(int(v['pin']) - 1),
	      'expected':"Incorrect PIN and/or account entered, or account inactive.",
	      'testno':testno,
	      'description':'wrong PIN entered' })

  # test 8: no PIN entered and request for emailed PIN not made 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=admin&admin=admin',
	      'expected':"No PIN entered.",
	      'testno':testno,
	      'description':'no PIN entered or requested' })

  # test 9: PIN request with incorrect admin email  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=admin&sendpin=sendpin&EMAIL=n@a.zzzz',
	      'expected':"Incorrect email address, can't send PIN.",
	      'testno':testno,
	      'description':'PIN request but incorrect email address' })

  # test 10: valid PIN request with correct admin email  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=admin&sendpin=sendpin&EMAIL=%s' % v['admin_e'],
	      'expected':"The PIN for the account has been sent as requested.",
	      'testno':testno,
	      'description':'valid PIN request with correct email address' })

  # test 11: accounts option, send accounts form  
  testno+=1
  ua='admin'
  accountsform=testlib.get_form('accounts.form',vars=
    [v['registry'],'admin','VARIABLE_COOKIE',ua,None,None,ackf_select],ua=ua)
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'mainmenu.cgi',
              'data':'ac_id=admin&accounts=accounts&pin=%s&ua=admin' % v['pin'],
	      'expected':accountsform, 
	      'testno':testno,
	      'description':'accounts option, send the accounts form' })
  
  # test 12: cdirectory option, send cdirectory form  
  testno+=1
  cdirform=testlib.get_form('cdirectory.form',vars=
    [v['registry'],'admin','VARIABLE_COOKIE','admin',None],ua='admin')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'mainmenu.cgi',
              'data':'ac_id=admin&cdirectory=cdirectory&pin=%s&ua=admin'
                     % v['pin'],
  	      'expected':cdirform,
  	      'testno':testno,
  	      'description':'Contact directory option, send the cdirectory form' })
  
  # test 13: tdirectory option, send trading directory form
  select='''<SELECT NAME="category">
<OPTION VALUE="all">all
</SELECT>
'''
  testno+=1
  tdirform=testlib.get_form('tdirectory.form',vars=
    [v['registry'],'admin','VARIABLE_COOKIE','admin',select,None],ua='admin')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'mainmenu.cgi',
              'data':'ac_id=admin&tdirectory=tdirectory&pin=%s&ua=admin' % v['pin'],
  	      'expected':tdirform,
  	      'testno':testno,
  	      'description':'Trading directory option, send the tdirectory form' })
  
  # test 14: admin option, send admin form  
  testno+=1
  adminform=testlib.get_form('admin.form',vars=
    [v['registry'],'admin','VARIABLE_COOKIE','admin'],ua='admin')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'mainmenu.cgi',
                'data':'ac_id=admin&admin=admin&pin=%s&ua=admin' % v['pin'],
  	      'expected':adminform,
  	      'testno':testno,
  	      'description':'admin option send the admin form' })
  
  # test 15: login option, send mainmenu form  
  testno+=1
  mainmenuform=testlib.get_form('mainmenu.form',vars=
    [v['registry'],'admin','VARIABLE_COOKIE',ua,None],ua='admin')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=admin&admin=admin&pin=%s' % v['pin'],
	      'expected':mainmenuform,
	      'testno':testno,
	      'description':'admin login option, send the mainmenu form' })

  # test 16: add account invalid request reserved name 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=writeoff&pin=%s&ua=admin&add=add&user_name=Write Off&email=writeoff@example.net&start_bal=0.0&phone1=0123 456 7899&phone2=0781876666&address=33 Boggy St\nCoventry&postcode=CV2 3TZ&district=Radford&limit=None' % v['pin'],
              'expected':"Account id requested is a reserved name",
              'testno':testno,
              'description':'invalid add user reserved account name' })

  # test 17: add account valid request user joe needed for test2.py 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=joe&pin=%s&ua=admin&add=add&user_name=Joe&email=nullj@%s&start_bal=0.0&phone1=0123 456 7899&phone2=0781876666&address=33 Boggy St\nCoventry&postcode=CV2 3TZ&district=Radford&limit=None' % (v['pin'],emd),
              'expected':"<p>new member added",
              'testno':testno,
              'description':'valid add user: Joe needed for test2.py' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

if __name__ == "__main__":
  main()
