#!/usr/bin/env python
''' test10.py of Test suite for PyLETS CGI

    test9 - test13 are designed to be runnable as a unit
    from a clean system.
    
    Purpose of tests:
    a. viewing contact data active users as user
    b. viewing contact data all users as user
    c. try unauthorised viewing contacts as user
    d. various errors attemping to view contacts as user
    e. various errors attemping to view labels as admin 

    Prerequisites: test set 9 must be completed to add jill
    ben and jack's accounts and for jill's PIN to be known.
    Jack's account must be inactive, the others active. 
'''

def main(change_expected=False):
  print("\nTest Case 10: viewing contact directory operations as non admin user.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')  
  tests=[]
  
  # need jill's PIN first
  jpin=testlib.get_userpin('jill')
  testno=0
  testcase=10

  # test 1: viewing contact data active users as user

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&active=active'
              % jpin,
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view active member contact details as user' })

  # test 2: viewing contact data all users as user

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&all=all'
              % jpin,
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view all member contact details as user' })


  # test 3: try unauthorised viewing contacts as user

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&addr=addr'
              % jpin,
              'expected':'Unauthorised attempt to: FULLADDRVIEW by account: jill logged.',
              'testno':testno,
              'description':'unauthorised try to see postal details as user' })
 
  # test 4: try unauthorised viewing labels as user

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&labels=labels&lcols=2&labelsa=all&noemail=include'
              % jpin,
              'expected':'Only admin may view address labels.',
              'testno':testno,
              'description':'unauthorised try to see post labels as user' })
 
  # test 5: try incorrect button pressed as user

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&button=button'
              % jpin,
              'expected':'Invalid submit button value.',
              'testno':testno,
              'description':'invalid submit button option' })
 
  # test 6: viewing contact data active users as user wrong account

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=jillian&pin=%s&ua=user&active=active'
              % jpin,
              'expected':'Invalid or stale or logged out session.',
              'testno':testno,
              'description':'view member contact details wrong account' })

  # test 7: viewing contact data active users as user wrong pin

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&active=active'
              % str(int(jpin)+1),
              'expected':'Invalid or stale or logged out session.',
              'testno':testno,
              'description':'view member contact details wrong pin' })

  # test 8: no label columns as admin 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&labelsa=all&noemail=include'
              % v['pin'],
              'expected':'Missing value/s. Complete all form fields.',
              'testno':testno,
              'description':'no label columns number' })
 
  # test 9: no labels active or all as admin 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&lcols=3&noemail=include'
              % v['pin'],
              'expected':'Missing value/s. Complete all form fields.',
              'testno':testno,
              'description':'no labels active or all option' })
 
  # test 10: invalid no label columns as admin 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&lcols=7&labelsa=all&noemail=include'
              % v['pin'],
              'expected':'Incorrect value for label columns.',
              'testno':testno,
              'description':'invalid label columns number' })
 
  # test 11: invalid labels active or all as admin 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&lcols=3&labelsa=none&noemail=include'
              % v['pin'],
              'expected':'Incorrect value for active or all.',
              'testno':testno,
              'description':'invalid labels active or all option' })
 
  # test 12: invalid labels noemail without or include 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&lcols=3&labelsa=all&noemail=maybe'
              % v['pin'],
              'expected':'Incorrect value for members without email only',
              'testno':testno,
              'description':'invalid labels without email or include option' })
 
  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

 
if __name__ == "__main__":
  main() 
