#!/usr/bin/env python
''' test11.py of Test suite for PyLETS CGI

    test9 - test13 are designed to be runnable as a unit
    from a clean system.
    
    Purpose of tests:
    
    a. view logins.pkl log as admin 
    b. generate some bad login activity   
    c. view gbips.pkl good/bad IPs as admin    
    d. view lacs.pkl good/bad A/C logins as admin
    e. check override lockout on ben     
    f. generate some very bad login activity to lock testing ip 
    g. check i/p gets locked out using admin even with good password

    Prerequisites: test set 9 and 10 must be completed to add jill
    ben and jack's accounts and for jill's PIN to be known.
    Jack's account must be inactive, the others active, and for
    relevant login activity to have taken place. 
'''
# change to 1 to debug or 0 normal 
debug=0

def main(change_expected=False):
  print("\nTest Case 11: viewing security logging and locking operations.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')  
  tests=[]
  testno=0
  testcase=11
  
  # need jill and ben's PINs 
  jpin=testlib.get_userpin('jill')
  bpin=testlib.get_userpin('ben')
 
  # test 1: viewing logins.pkl log as admin 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&logins=logins&logfilename=logins.pkl'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view logins.pkl log as admin' })

  # test 2: generate some bad login activity   
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
                'data':'account=jill&pin=%s&user=user'
              % '99999999',
              'expected':'Incorrect PIN and/or account entered, or account inactive.',
              'testno':testno,
              'description':'generate bad login as user' })
 
  # test 3: viewing logins.pkl log including bad login as admin 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&logins=logins&logfilename=logins.pkl'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view logins.pkl log as admin including bad login' })
    

  # test 4: view gbips.pkl good/bad IPs as admin    
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&gbips=gbips'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view gbips.pkl log as admin' })

  # test 5: view lacs.pkl good/bad A/C logins as admin    

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&lacs=lacs'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view lacs.pkl log as admin' })

  # test 6: add override reject on ben 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&aover=aover&accrej=reject&ovac=ben&ipaddr=%s'
              % (v['pin'],tv['ipv4or6']),
              'expected':'Override: 1 successfully input.',
              'testno':testno,
              'description':'add reject override for ben ' })
    
  # test 7: check ben cant login   

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=ben&user=user&pin=%s' % bpin,
              'expected':'Incorrect PIN and/or account entered, or account inactive.',
              'testno':testno,
              'description':'check ben cant login after override reject' })
 
  # test 8: remove override lockout on ben 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&del_ovid=1' % v['pin'],
              'expected':'Thanks for your input which has been received',
              'testno':testno,
              'description':'delete override for ben' })
    
  # test 9: check ben can login    

  testno+=1
  mainmenuform=testlib.get_form('mainmenu.form',vars=
    [v['registry'],'ben','VARIABLE_COOKIE','user',None],ua='user')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=ben&user=user&pin=%s' % bpin,
              'expected':mainmenuform,
              'testno':testno,
              'description':'ben can now login OK' })

  # test 10 - 15: generate some very bad login activity to lock testing ip
  # This assumes security.cfg biph:5 (max bad IP/hour before lockout = 5)
  # the first has to be done separately due to different error message.

  nexttest=testno+1 
  for i in range(nexttest,nexttest+5): 
    testno+=1
    tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
                'data':'account=jill&pin=%s&user=user'
              % '99999999',
              'expected':'Incorrect PIN and/or account entered, or account inactive.',
              'testno':testno,
              'description':'generate repeated bad login as jill' })

  # test 16: view logins.pkl log as admin, good password but lockout 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&logins=logins&logfilename=logins.pkl'
              % v['pin'],
              'expected':'Invalid or stale or logged out session.',
              'testno':testno,
              'description':'login as admin good password but lockout' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

 
if __name__ == "__main__":
  main() 
