#!/usr/bin/env python
''' test12.py of Test suite for PyLETS CGI

    test9 - test13 are designed to be runnable as a unit
    from a clean system.
    
    Purpose of tests:
    
    a. add override for admin using master_key.py 
    b. check override record added using master_key.py 
    c. check lockout credited to goat account not admin
    d. admin adds override for ben various errors 
    e. admin adds override for ben (needed for test13)
    f. admin adds override for jill
    g. admin deletes override for jill 
    h. admin views overrides for admin and ben 

    Prerequisites: test set 9 - 11 must be completed to add jill
    ben and jack's accounts and for jill's PIN to be known.
    Jack's account must be inactive, the others active, and for
    relevant login activity and locking to have taken place.

    Before anything else can happen, a hack is needed using
    the master_key.py script runnable only directly on
    the server. Making this doable from a different test client
    requires a cgi script on the lines of make_clean.cgi to
    run master_key from the client. 
'''

# change to 1 to debug or 0 normal 
debug=0

def main(change_expected=False):
  print("\nTest Case 12: undoing lockout. checking override and goat account .\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')  
  tests=[]
  testno=0
  testcase=12

  # system command line needed to override admin account to accept 
  # admin logins from address running this test software
  # note that this operation should only ever be considered
  # as a command line by someone who can login to the server
  #  
  master_key_command='(cd .. ; ./master_key.py -o '+tv['ipv4or6']+' )'
    
  # test 1: add and check override for admin using master_key.py 

  import os
  os.system(master_key_command)
 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&vover=vover'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'add and check master_key override added' })

  # test 2: check lockout credited to goat account not admin

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&lacs=lacs'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'check lockout login logs added to goat a/c' })

  # test 3: admin adds override for ben missing values 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&aover=aover'
              % v['pin'],
              'expected':'Missing value/s. Complete all form fields.',
              'testno':testno,
              'description':'add override for ben missing values' })

  # test 4: admin adds override for ben invalid accept/reject 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&aover=aover&accrej=neither&ovac=ben&ipaddr=%s'
              % (v['pin'],tv['ipv4or6']),
              'expected':' This must be either an accept or reject.',
              'testno':testno,
              'description':'add override for ben invalid accept/reject' })

  # test 5: admin adds override for ben invalid  ip address 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&aover=aover&accrej=accept&ovac=ben&ipaddr=%s'
              % (v['pin'],'123.999.107.23'),
              'expected':'Invalid IP address.',
              'testno':testno,
              'description':'add override for ben invalid ip' })

  # test 6: admin adds override for ben invalid account 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&aover=aover&accrej=accept&ovac=ben#&ipaddr=%s'
              % (v['pin'],tv['ipv4or6']),
              'expected':'Invalid override account id/no.',
              'testno':testno,
              'description':'add override for ben invalid account' })

  # test 7: admin adds valid override for ben 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&aover=aover&accrej=accept&ovac=ben&ipaddr=%s'
              % (v['pin'],tv['ipv4or6']),
              'expected':'Override: 3 successfully input.',
              'testno':testno,
              'description':'add valid override for ben ' })

  # test 8: admin adds override for jill

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&aover=aover&accrej=accept&ovac=jill&ipaddr=%s'
              % (v['pin'],tv['ipv4or6']),
              'expected':'Override: 4 successfully input.',
              'testno':testno,
              'description':'add valid override for jill' })

  # test 9: admin deletes override for jill invalid overide id 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&del_ovid=6' % v['pin'],
              'expected':'Delete attempted but Override ID not found.',
              'testno':testno,
              'description':'delete override invalid override id' })

  # test 10: admin deletes override for jill invalid overide id format 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&del_ovid=xxxx' % v['pin'],
              'expected':'Invalid Override id',
              'testno':testno,
              'description':'delete override invalid override id format' })

  # test 11: admin deletes override for jill 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&del_ovid=4' % v['pin'],
              'expected':'Thanks for your input which has been received',
              'testno':testno,
              'description':'delete override for jill' })

  # test 12 admin views overrides for admin and ben 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&vover=vover'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'admin views overrides for admin and ben' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

 
if __name__ == "__main__":
  main() 
