#!/usr/bin/env python
''' test13.py of Test suite for PyLETS CGI

    test9 - test13 are designed to be runnable as a unit
    from a clean system.
    
    Purpose of tests:
    
    a. manipulate values in smaint table to force log rotation
    b. generate new login and view this login record only
    c. check selection option of new and rotated log in security form
    d. view older rotated log
    e. check correct recompilation of good/bad table by ip
    f. check correct recompilation of good/bad table by account 

    Prerequisites: test set 9 - 12 must be completed to add jill
    ben and jack's accounts and for jill's PIN to be known.
    Jack's account must be inactive, the others active, and for
    relevant login activity and locking to have taken place.

'''

# change to 1 to debug or 0 normal 
debug=0

def main(change_expected=False):
  print("\nTest Case 13: Confirming log rotation and good/bad table rebuild.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  # need ben's PIN first
  bpin=testlib.get_userpin('ben')
 
  tests=[]
  testno=0
  testcase=13
  # perform log maintenance 
  testlib.logmtce()
    
  # test 1: login ben new login log
  testno+=1
  mainmenuform=testlib.get_form('mainmenu.form',vars=
    [v['registry'],'ben','VARIABLE_COOKIE','user',None],ua='user')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=ben&user=user&pin=%s' % bpin,
              'expected':mainmenuform,
              'testno':testno,
              'description':'login ben using new log' })

  # test 2: viewing logins.pkl log as admin

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&logins=logins&logfilename=logins.pkl'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view logins.pkl log as admin' })

  # test 3: check selection option of new and rotated log in security form

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&secure=secure' % v['pin'],
              'expected':'''<SELECT NAME="logfilename">
<OPTION VALUE="">choose current file or archive
<OPTION VALUE="logins.pkl">logins.pkl
<OPTION VALUE="logins.0.pkl">logins.0.pkl
</SELECT>''',
              'testno':testno,
              'description':'check html selection of rotated logfile option' })

  # test 4: view older rotated log

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&logins=logins&logfilename=logins.0.pkl'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view logins.0.pkl log as admin' })

  # test 5: view older rotated log

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&logins=logins&logfilename=logins.2.pkl'
              % v['pin'],
              'expected':'Login archive filename is not a file.',
              'testno':testno,
              'description':'view non-existent login log as admin' })

  # test 6: check correct recompilation of good/bad table by ip

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&gbips=gbips'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view rebuilt gbips.pkl log' })


  # test 7: check correct recompilation of good/bad table by account 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'security.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&lacs=lacs'
              % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view rebuilt lacs.pkl log' })


  # do all stored tests
  testlib.run_tests(tests,stepped=debug,change_expected=change_expected)

 
if __name__ == "__main__":
  main() 
