#!/usr/bin/env python
''' test14.py of Test suite for PyLETS CGI

    test14 - test15 are designed to be runnable as a unit
    from a clean test system installation.
    
    Purpose of tests:
    a. adding 3 new accounts to system, jack, jill and ben
    b. adding want/offer categories for second and food.
    c. add want to system for jack
    d. add 2 offer to system for jill
    e. add want to system for ben 
    f. add want/offers various errors
    g. display wants and offers data 
    h. display wants and offers data including contact details
    i. make jack inactive
    j. remove ben from system
    k. remove 1 offer for jill
    l. remove offer various errors
    m. display pruned wants and offers data 

    prerequisites: system must
    not already contain accounts for jack or jill. 
'''

def main(change_expected=False):
  print("\nTest Case 14: wants and offers and trading directory listing.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars()
  tv=testlib.get_vars(cfile='test.cfg')
  emd=tv['email_domain']
  v['pin']=testlib.run_make_clean()
  next_ack_id=testlib.get_nextackid()
  
  tests=[]
  testcase=14
  testno=0
  
  # test 1: add account jack valid request 1
   
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&add=add&user_name=Jack&email=nullj@%s&start_bal=0.0&phone1=01234 56789&phone2=0781 987654&address=7 Drury Lane,\nLondon&postcode=SW1 2WW&district=Earlsdon&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
              'testno':testno,
  	      'description':'valid add user: Jack' })
  
  # test 2: add account jill valid request 2 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jill&pin=%s&ua=admin&add=add&user_name=Jill&email=nullk@%s&start_bal=0.0&phone1=012345 6897&phone2=None&address=2 Tuppence Rd,Coventry&postcode=CV2 2JX&district=Foleshill&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
              'testno':testno,
  	      'description':'valid add user: Jill' })
  
  # test 3: add account ben valid request 3 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=ben&pin=%s&ua=admin&add=add&user_name=Ben&email=nullb@%s&start_bal=0.0&phone1=012345 6897&phone2=None&address=2 Broad Lane,Coventry&postcode=CV4 3XY&district=Stoke Heath&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
              'testno':testno,
  	      'description':'valid add user: Ben' })
 
  # test 4: valid add trading category food 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&add=add&category=food&sort=100&desc=Food, produce, cooked and baked foods' % v['pin'],
  	      'expected':"new category added",
              'testno':testno,
  	      'description':'valid add category: food' })

  # test 5: valid add trading category food 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&add=add&category=second&sort=200&desc=second hand and reusable items' % v['pin'],
  	      'expected':"new category added",
              'testno':testno,
  	      'description':'valid add category: second hand goods' })

 
  # test 6: add want to system for jack
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=U&category=second&day=31&month=Dec&year=2030&desc=Dalek must be in good condition&who=jack' % v['pin'],
  		'expected':'<p> Want or offer: 1 successfully input.</p>',
  		'testno':testno,
  		'description':'add want to system for Jack' })
  
  # test 7 add offer to system for jill
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=offer&skill=U&category=second&day=31&month=Dec&year=2030&desc=Dalek with slightly erratic exterminator&who=jill' % v['pin'],
  		'expected':'<p> Want or offer: 2 successfully input.</p>',
  		'testno':testno,
  		'description':'add offer to system for Jill' })
  
  # test 7: add 2nd offer to system for jill
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=offer&skill=B&category=second&day=31&month=Dec&year=2030&desc=Dalek repairs&who=jill' % v['pin'],
  		'expected':'<p> Want or offer: 3 successfully input.</p>',
  		'testno':testno,
  		'description':'add another offer to system for Jill' })
  
  # test 9 add want to system for ben 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=P&category=food&day=31&month=Dec&year=2030&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'<p> Want or offer: 4 successfully input.</p>',
  		'testno':testno,
  		'description':'add want to system for Ben' })
  
  # test 10 add want/offers nonexistent ac_id 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=notaccount&pin=%s&ua=user&wo=want&skill=P&category=food&day=31&month=Dec&year=2030&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'<p>  Invalid user account id/no. </p>',
  		'testno':testno,
  		'description':'nonexistent ac_id' })
  
  # test 11 add want/offers nonexistent who 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=P&category=food&day=31&month=Dec&year=2030&desc=help catering for party&who=weed' % v['pin'],
  		'expected':'<p>  Trying to input want/offer for nonmember. </p>',
  		'testno':testno,
  		'description':'nonexistent account for whom w/o posted by admin' })
  
  # test 12 add want/offers neither want nor offer 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=something&skill=P&category=food&day=31&month=Dec&year=2030&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'<p>  This must be either a want or an offer. </p>',
  		'testno':testno,
  		'description':'add directory entry neither want nor offer' })
  
  # test 13 add want/offers skill level invalid 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=X&category=food&day=31&month=Dec&year=2030&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'either P (Professional) or S (Semi skilled) or B (Beginner) or U (Unknown',
  		'testno':testno,
  		'description':'add directory entry invalid skill type' })
  
  # test 14 add want/offers category not in list 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=P&category=XXXXX&day=31&month=Dec&year=2030&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'Unknown want or offer category.',
  		'testno':testno,
  		'description':'add directory unknown category' })
  
  # test 15 add want/offers expiry date invalid test 1 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=P&category=food&day=xx&month=013&year=abcd&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'Unparseable expiry date format error 1.',
  		'testno':testno,
  		'description':'add directory unparseable date' })
  
  # test 16 add want/offers expiry date invalid test 2 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=P&category=food&day=30&month=Feb&year=2019&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'DATE does not seem to be a valid date.',
  		'testno':testno,
  		'description':'add directory unconvertable expiry date' })
  
  # test 17 add want/offers expiry date before date posted 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=P&category=food&day=1&month=Jan&year=1970&desc=help catering for party&who=ben' % v['pin'],
  		'expected':'expiry date/time before current date/time.',
  		'testno':testno,
  		'description':'add directory expiry date before posting date' })
  
  # test 18 add want/offers invalid description 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=P&category=food&day=31&month=Dec&year=2030&desc=help catering for [party]&who=ben' % v['pin'],
  		'expected':'Invalid character in or too long description.',
  		'testno':testno,
  		'description':'add directory invalid description' })
  
  # test 19 display want/offers data 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&viewdata=viewdata' % v['pin'],
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'display directory wants/offers data' })
  
  # test 20 display want/offers all data including contact details 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&vwo=vwo&categ=all' % v['pin'],
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'display all wants/offers including contacts' })
  
  # test 21 display want/offers data food category only 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&vwo=vwo&category=food' % v['pin'],
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'display food wants/offers including contacts' })

  # test 22 display want/offers data posted for jill only 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&vwo=vwo&whom=jill' % v['pin'],
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'display jill wants/offers including contacts' })

  # test 8: make account jack inactive 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&status=status&pai=inactive' 
  	      % v['pin'],
  	      'expected':'<p>Thanks for your input which has been received and',
              'testno':testno,
  	      'description':'inactivate account: Jack' })
  
  # test 9: delete of ben account 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=ben&pin=%s&ua=admin&delete=delete' % v['pin'],
                'expected':"Thanks for your input which has been received and",
  		'testno':testno,
                'description':'delete of user ben' })
  
  # test 83 delete one of Jill want/offers  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&del_woid=3' % v['pin'],
  		'expected':'<p>Thanks for your input which has been received', 
  		'testno':testno,
  		'description':'successful delete of a wants/offer record' })
  
  # test 84 delete want/offer non existent woid  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&del_woid=3' % v['pin'],
  		'expected':'Delete attempted but Want/Offer ID not found.',
  		'testno':testno,
  		'description':'attempted delete of want/offer nonexistent woid' })
  
  # test 85 delete want/offer invalid woid  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&del_woid=xxxx' % v['pin'],
  		'expected':'Invalid Want/Offer id',
  		'testno':testno,
  		'description':'attempted delete of want/offer invalid woid' })
  
  # test 86 display want/offers data 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&viewdata=viewdata' % v['pin'],
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'display directory wants/offers data' })
  
  # test 10: add account for adam
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
              'data':'ac_id=admin&user_ac_id=adam&pin=%s&ua=admin&add=add&user_name=Adam Ant&email=nulla@%s&start_bal=0.0&phone1=012345678&phone2=0782 987554&address=10 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
              'expected':"<p>new member added",
              'testno':testno,
              'description':'valid add user: adam' })

  # test 89 grant ADDWO authority for adam in respect of jill
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=adam&pin=%s&ua=admin&add=add&auth=WOADD&whom=jill' % v['pin'],
              'expected':"New authorisation WOADD added for adam",
              'testno':testno,
              'description':'authorise user adam to WOADD for jill' })

  # test 90 grant WODEL authority for adam in respect of jill
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=adam&pin=%s&ua=admin&add=add&auth=WODEL&whom=jill' % v['pin'],
              'expected':"New authorisation WODEL added for adam",
              'testno':testno,
              'description':'authorise user adam to WODEL for jill' })
 
  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)


if __name__ == "__main__":
  main()
