#!/usr/bin/env python
''' test15.py of Test suite for PyLETS CGI

    test14 - test15 are designed to be runnable as a unit
    from a clean system.
    
    Purpose of tests:
    a. adding want/offer as user and as user who is admin (adam)
    b. various errors adding want/offer as user
    c. deleting want/offer as user
    d. various errors deleting want/offer as user
    e. viewing wants/offers data as user
    f. viewing wants/offers including contacts as user
    g. displaying categories as user
    h. requesting categories management form as admin
    i. invalid add category, sort and description
    j. invalid modify category: sort and description
    k. valid modify category: sort and description
    l. checking modify sort results in changed order of table view 
    m. invalid delete category
    n. valid delete category

    Prerequisites: test set 14 must be completed to add 
    accounts, categories and a want/offer for jill before 
    jill's PIN is known. User a/c for adam and adam authorised
    ADDWO and DELWO . 
'''

def main(change_expected=False):
  print("\nTest Case 15: checking want/offer operations as non admin user.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')  
  emd=tv['email_domain']
  tests=[]
  testcase=15
 
  # need jill's and adam's PINs 
  jpin=testlib.get_userpin('jill')
  apin=testlib.get_userpin('adam')
  
  # test 1: adding want/offer as jill
  
  testno=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
                  'data':'ac_id=jill&pin=%s&ua=user&wo=offer&skill=U&category=second&day=31&month=Dec&year=2030&desc=Dalek parts slightly burned out but still serviceable.' % jpin,
                  'expected':'<p> Want or offer: 5 successfully input.</p>',
                  'testno':testno,
                  'description':'add offer to system by Jill' })
  
  # test 2: adding want/offer as jill incorrect pin
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
                  'data':'ac_id=jill&pin=%s&ua=user&wo=offer&skill=U&category=second&day=31&month=Dec&year=2030&desc=Dalek parts slightly burned out but still serviceable.' % str(int(jpin)+1),
                  'expected':'Invalid or stale or logged out session.',
                  'testno':testno,
                  'description':'add offer to system by Jill wrong PIN' })
  
  # test 3: adding want/offer as jill invalid expiry date
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
                  'data':'ac_id=jill&pin=%s&ua=user&wo=offer&skill=U&category=second&day=31&month=Deb&year=2030&desc=Dalek parts slightly burned out but still serviceable.' % jpin,
                  'expected':'31 Deb 2030 does not seem to be a valid date.',
                  'testno':testno,
                  'description':'add offer to system by Jill invalid expiry date' })

  # test 4: deleting want/offer as jill


  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&delete=delete&del_woid=2' % jpin,
                'expected':'<p>Thanks for your input which has been received',
                'testno':testno,
                'description':'successful delete by user of a wants/offer record' })

  # test 5: attempting deletion want/offer as jill non existent woid
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&delete=delete&del_woid=2' % jpin,
                'expected':'Delete attempted but Want/Offer ID not found.',
                'testno':testno,
                'description':'attempted delete by user nonexisting wants/offer' })

  # test 6: attempting deletion want/offer as jill invalid woid
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&delete=delete&del_woid=abc' % jpin,
                'expected':'Invalid Want/Offer id',
                'testno':testno,
                'description':'attempted delete by user invalid want/offer id' })

  # test 6 display want/offers data as jill

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&viewdata=viewdata' % jpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'display directory wants/offers data by user' })

  # test 7 display want/offers data including contact details

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&vwo=vwo' % jpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'display wants/offers including contacts by user' })
  
  # test 8 display trading categories data as user 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&vcats=vcats' % jpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'display trading directory categories by user' })
  
  # test 9 request trading categories management form as admin 

  testno+=1
  tcategoriesform=testlib.get_form('tcategories.form',vars=
    [v['registry'],'admin','VARIABLE_COOKIE','admin'],ua='admin')
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&catman=catman' % v['pin'],
                'expected':tcategoriesform,
                'testno':testno,
                'description':'request trading directory categories form' })

  # test 10 invalid add category key already exists

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&add=add&category=second&sort=900&desc=odds and ends' % v['pin'],
                'expected':"Category exists. Can't add it.",
                'testno':testno,
                'description':'invalid add category key already exists' })

  # test 11 invalid add category sort code 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&add=add&category=misc&sort=abc&desc=odds and ends' % v['pin'],
                'expected':'Invalid sort code. Use numbers 0-9 only.',
                'testno':testno,
                'description':'invalid add category non numeric sort code' })


  # test 12 invalid add category invalid desc 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&add=add&category=misc&sort=500&desc=({[]})' % v['pin'],
                'expected':'Invalid category description.',
                'testno':testno,
                'description':'invalid add category illegal chars in description' })

  # test 13 invalid add category invalid chars in key

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&add=add&category=misc*&sort=500&desc=odds and ends' % v['pin'],
                'expected':'Invalid category. Use letters a-z only.',
                'testno':testno,
                'description':'invalid add category illegal chars in key' })


  # test 14 invalid modify category key doesnt exist 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&modify=modify&category=stuff&sort=500&desc=odds and ends' % v['pin'],
                'expected':"Category doesn't exist. Can't modify.",
                'testno':testno,
                'description':'invalid modify category does not exist' })

  # test 15 invalid modify category sort 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&modify=modify&category=second&sort=5xx' % v['pin'],
                'expected':"Invalid sort code. Use numbers 0-9 only.",
                'testno':testno,
                'description':'invalid modify category sort code' })

  # test 16 invalid modify category description

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&modify=modify&category=second&sort=500&desc=([{}])' % v['pin'],
                'expected':"Invalid category description.",
                'testno':testno,
                'description':'invalid modify category description' })

  # test 17 valid modify category sort 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&modify=modify&category=second&sort=50' % v['pin'],
                'expected':"1 field/s modified.",
                'testno':testno,
                'description':'valid modify category sort code' })

  # test 18 valid modify category description

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&modify=modify&category=second&desc=second hand, old, unwanted and reusable items' % v['pin'],
                'expected':"1 field/s modified.",
                'testno':testno,
                'description':'valid modify category description' })

  # test 19 checking modify sort results in changed order of table view 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&view=view' % v['pin'],
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'checking modify sort results in changed order of rows' })

  # test 20 invalid delete category does not exist

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&category=nothing' % v['pin'],
                'expected':"Category doesn't exist. Can't delete it.",
                'testno':testno,
                'description':'invalid delete category does not exist' })

  # test 21 invalid delete category want/offer record/s exist 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&category=second' % v['pin'],
                'expected':"Category currently used in want/offer table. Can't delete it.",
                'testno':testno,
                'description':'invalid delete category want or offers exist' })

  # test 22 valid delete category

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&delete=delete&category=food' % v['pin'],
                'expected':"<p>Thanks for your input which has been received",
                'testno':testno,
                'description':'invalid delete category want or offers exist' })

  # test 7: adding want/offer by adam administrating for jill
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
                  'data':'ac_id=adam&pin=%s&ua=admin&who=jill&wo=offer&skill=U&category=second&day=31&month=Dec&year=2030&desc=gardening tools' % apin,
                  'expected':'<p> Want or offer: 6 successfully input.</p>',
                  'testno':testno,
                  'description':
             'add offer to system for Jill by Adam with ADDWO authority' })
  
  # test 8: adam can DELWO for jill 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
             'data':'ac_id=adam&pin=%s&ua=user&delete=delete&who=jill&del_woid=6' % apin,
             'expected':'Thanks for your input which has been received',
             'testno':testno,
             'description':
             'Adam as DELWO authorised can remove want or offer for jill' })

  # test 9: add account joe valid request 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=joe&pin=%s&ua=admin&add=add&user_name=Joe&email=nullj@%s&start_bal=0.0&phone1=01234 56789&phone2=0781 987555&address=7 Drury Lane,\nLondon&postcode=SW1 2WW&district=Earlsdon&limit=None' % (v['pin'],emd),
              'expected':"<p>new member added",
              'testno':testno,
              'description':'valid add user: Joe' })

  # test 10: invalid add want/offer by adam who cant admin for joe
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
       'data':'ac_id=adam&pin=%s&ua=admin&who=joe&wo=offer&skill=U&category=second&day=31&month=Dec&year=2030&desc=gardening tools' % apin,
       'expected':'Unauthorised attempt to: WOADD by account: adam logged.',
       'testno':testno,
       'description':
       'invalid add offer by Adam with wrong specific WOADD authority' })
  
  # test 11: adding want/offer by admin for joe
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
        'data':'ac_id=admin&pin=%s&ua=admin&who=joe&wo=offer&skill=U&category=second&day=31&month=Dec&year=2030&desc=gardening tools' % v['pin'],
        'expected':'<p> Want or offer: 7 successfully input.</p>',
        'testno':testno,
        'description':
        'add offer to system for joe by admin' })

  # test 12: adam can not WODEL for joe
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tdirectory.cgi',
        'data':'ac_id=adam&pin=%s&ua=user&delete=delete&who=joe&del_woid=7' % apin,
        'expected':'Unauthorised attempt to: WODEL by account: adam logged.',
        'testno':testno,
        'description':
        'Adam as DELWO authorised but not specific to joe' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)
 
if __name__ == "__main__":
  main() 
