#!/usr/bin/env python
''' test16.py start of part E of test suite for PyLETS CGI

    test16 - test19 are designed to be runnable as a unit
    from a clean test system installation.
    
    Purpose of test 16:
    a. adding 4 new accounts for locksmith, alice, bob and chris to system
    b. granting locksmith ability to add authorisations  
    c. preparing for subsequent tests without effective admin account 
    prerequisites: system must start clean using make_clean
    postrequisites: secure.cfg variable admin_has_superpowers set to no 
'''

def main(change_expected=False):
  print("\nTest Case 16: add locksmith and users, remove admin superpowers.\n")
  
  import testlib 
  # prerequisite: admin_has_superpowers:yes 
  testlib.modify_config_var('test_configs/secure.cfg',
      'admin_has_superpowers','yes')
  # system has numeric accounts, own email prerequisites 
  testlib.modify_config_var('test_configs/newmember.cfg',
      'system_has_numeric_accounts','Y')
  testlib.modify_config_var('test_configs/newmember.cfg',
      'applicants_must_have_own_email','Y')
  # get site&test configuation variables in dictionaries v and tv
  v=testlib.get_vars()
  tv=testlib.get_vars(cfile='test.cfg')
  emd=tv['email_domain']
  v['pin']=testlib.run_make_clean()
  next_ack_id=testlib.get_nextackid()
  
  tests=[]
  testcase=16
  testno=0  

  # test 1: add account lock valid request 1 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=lock&pin=%s&ua=admin&add=add&user_name=Lock Smith&email=nulll@%s&start_bal=0.0&phone1=01274 56789&phone2=0781 987654&address=11 Drury Lane,\nLondon&postcode=SW1 2WW&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Lock Smith' })
  
  # test 2: add account alice valid request 2 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=alice&pin=%s&ua=admin&add=add&user_name=Alice&email=nulla@%s&start_bal=0.0&phone1=032345 6897&phone2=None&address=5 Tuppence Rd,Coventry&postcode=CV3 2JX&district=Foleshill&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Alice' })
  
  # test 3: add account bob valid request 3 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin&add=add&user_name=Bob Builder&email=nullb@%s&start_bal=0.0&phone1=032375 6897&phone2=None&address=45 Holyhead Rd,Coventry&postcode=CV5 2JX&district=Keresley&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Bob' })
  
  # test 4: add account chris valid request 4 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&add=add&user_name=Chris Carpenter&email=nullc@%s&start_bal=0.0&phone1=032545 8897&phone2=None&address=5 Halfpenny Rd,Coventry&postcode=CV1 2JX&district=Foleshill&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Chris' })
   
  # test 5: authorise Lock Smith to grant authorisations 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=lock&pin=%s&ua=admin&add=add&auth=AUTHADD&whom=admin'
% v['pin'],
              'expected':"New authorisation AUTHADD added for lock",
              'testno':testno,
              'description':'authorise Lock Smith to add authorisations' })
 
  # test 6: authorise Lock Smith to view authorisations 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=lock&pin=%s&ua=admin&add=add&auth=AUTHVIEW&whom=admin'
% v['pin'],
              'expected':"New authorisation AUTHVIEW added for lock",
              'testno':testno,
              'description':'authorise Lock Smith to view authorisations' })
    
  # test 7: get new account application form based upon numeric account nos
  testno+=1
  handleopt=testlib.cgiutils.get_handleopt()
  applicform=open('../forms/application.form').read()
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'',
              'expected':applicform % (v['registry'],handleopt),
              'testno':testno,
              'description':'test sending application form numeric acs' })

  # test 8: input new account application duplicate email 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'submit=submit&email1=%s&user_name=Joe Grundy&phone1=0234 56789&phone2=01798642468&district=Burbia&address=1 Pig Lane,\nAmbridge&postcode=BS3 4AM&other=Very good\nReferences' % v['admin_e'],
              'expected':'An account with the same email address exists',
              'testno':testno,
              'description':'test sending application duplicate email' })
 
  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

  # modify secure.cfg admin has superpowers - needed for test17.py 
  testlib.modify_config_var('test_configs/secure.cfg',
      'admin_has_superpowers','no')
  # applicants don't need own email for test17.py
  testlib.modify_config_var('test_configs/newmember.cfg',
      'applicants_must_have_own_email','N')

if __name__ == "__main__":
  main()
