#!/usr/bin/env python
''' test17.py part 2 of section E of test suite for PyLETS CGI

    test16 - test19 are designed to be runnable as a unit
    
    Purpose of tests:
    a. Locksmith authorises chris to MEMBADD 
    b. chris adds member dave
    c. Locksmith authorises dave to CATEGADD and dave does this 
    d. Locksmith authorises alice to WOADD and WODEL
    e. Locksmith authorises bob to PAY and REPUD and MEMBSTATUS
    f. Locksmith authorises alice to AUDITVIEW
    g. alice authorised to ACCEPTREJECT and to APPLICATIONSVIEW
    h. admin can do none of the above
    i. none of the above can be done prior to authorisation
    j. account authorised can do the above post authorisation
    k. application can be input 
    l. application can be confirmed 
    m. application can be approved 
    n. application can be rejected 
    o. doing a couple of payments which can be aged ready for yearend
    prerequisites: secure.cfg variable admin_has_superpowers set to no 
'''

def main(change_expected=False):
  print("\nTest Case 17: locksmith functions without admin superpowers.\n")
  
  import testlib 

  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  next_ack_id=testlib.get_nextackid()
  lockpin=testlib.get_userpin('lock')
  alicepin=testlib.get_userpin('alice')
  bobpin=testlib.get_userpin('bob')
  chrispin=testlib.get_userpin('chris')
  te1=tv['test_email1']
  te2=tv['test_email2']
  emd=tv['email_domain']
  tests=[]
  testcase=17

  # test 1: chris cant MEMBADD

  testno=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=chris&user_ac_id=dave&pin=%s&ua=admin&add=add&user_name=Dave Dentist&email=nulld@%s&start_bal=0.0&phone1=042385 7897&phone2=0783484532&address=56 Holyhead Rd,Coventry&postcode=CV4 2XX&district=Keresley&limit=None' % (chrispin,emd),
              'expected':'Unauthorised attempt to: MEMBADD by account: chris logged.',
              'testno':testno,
              'description':'invalid add user by unauthorised chris' })

  # test 2: admin cant MEMBADD

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&add=add&user_name=Dave Dentist&email=null@%s&start_bal=0.0&phone1=042385 7897&phone2=0783484532&address=56 Holyhead Rd,Coventry&postcode=CV4 2XX&district=Keresley&limit=None' % (v['pin'],emd),
              'expected':'Unauthorised attempt to: MEMBADD by account: admin logged.',
              'testno':testno,
              'description':'invalid add user by disabled admin' })
   
  # test 3: authorise chris to MEMBADD
 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=chris&pin=%s&ua=admin&add=add&auth=MEMBADD&whom=admin'
% lockpin,
              'expected':"New authorisation MEMBADD added for chris",
              'testno':testno,
              'description':'locksmith authorises chris to add members' })
 
  # test 4: authorised chris can MEMBADD 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=chris&user_ac_id=dave&pin=%s&ua=admin&add=add&user_name=Dave Dentist&email=nulld@%s&start_bal=0.0&phone1=042385 7897&phone2=0783484532&address=56 Holyhead Rd,Coventry&postcode=CV4 2XX&district=Keresley&limit=None' % (chrispin,emd),
              'expected':'new member added',
              'testno':testno,
              'description':'valid add user by authorised chris' })

  # test 5: bob authorised by locksmith to CATEGADD

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=bob&pin=%s&ua=admin&add=add&auth=CATEGADD&whom=admin' % lockpin,
              'expected':"New authorisation CATEGADD added for bob",
              'testno':testno,
              'description':'locksmith authorises bob to add categories' })

  # test 6: bob authorised by locksmith to MEMBSTATUS 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=bob&pin=%s&ua=admin&add=add&auth=MEMBSTATUS&whom=admin' % lockpin,
              'expected':"New authorisation MEMBSTATUS added for bob",
              'testno':testno,
              'description':'locksmith authorises bob to add categories' })

  # test 7: valid add trading category by bob 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'tcategories.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&add=add&category=second&sort=100&desc=Second hand goods' % bobpin,
              'expected':"new category added",
              'testno':testno,
              'description':'valid add category by bob' })

  # test 8: alice cant WOADD 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&wo=want&skill=U&category=second&day=31&month=Dec&year=2030&desc=Dalek must be in good condition&who=bob' % alicepin
,
                'expected':'Unauthorised attempt to: WOADD by account: alice logged.',
                'testno':testno,
                'description':'invalid add want by unauthorised alice' })

  # test 9: admin cant WOADD

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&wo=want&skill=U&category=second&day=31&month=Dec&year=2030&desc=Dalek must be in good condition&who=bob' % v['pin']
,
                'expected':'Unauthorised attempt to: WOADD by account: admin logged.',
                'testno':testno,
                'description':'invalid add want by disabled admin' })

  # test 10: alice authorised by locksmith to WOADD

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=WOADD&whom=admin' % lockpin,
              'expected':"New authorisation WOADD added for alice",
              'testno':testno,
              'description':'locksmith authorises alice to add wants/offers' })

  # test 11: alice authorised can WOADD 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'wantoffer.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&wo=want&skill=U&category=second&day=31&month=Dec&year=2030&desc=Dalek must be in good condition&who=bob' % alicepin
,
                'expected':'<p> Want or offer: 1 successfully input.</p>',
                'testno':testno,
                'description':'add want to system for bob' })
  
  # test 12: bob unauthorised cant PAY 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=chris&to_name=Chris&amount=12.0&details=jam and bread' % bobpin,
              'expected':'Unauthorised attempt to: PAY by account: bob logged.',
              'testno':testno,
              'description':'bob unauthorised cant PAY from alice to chris' })
  
  # test 13: admin disabled cant PAY 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=chris&to_name=Chris&amount=12.0&details=jam and bread' % v['pin'],
              'expected':'Unauthorised attempt to: PAY by account: admin logged.',
              'testno':testno,
              'description':'admin disabled cant PAY from alice to chris' })

  # test 14: Locksmith authorises bob to PAY 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=bob&pin=%s&ua=admin&add=add&auth=PAY&whom=admin' % lockpin,
              'expected':"New authorisation PAY added for bob",
              'testno':testno,
              'description':'locksmith authorises bob to enter payments' })

  # test 15: bob authorised can PAY 

  testno+=1
  next_ack_id=1
  expected14='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=chris&to_name=Chris&amount=12.0&details=dalek parts' % bobpin,
              'expected':expected14,
              'testno':testno,
              'description':'valid bob helps alice to pay chris' })

  # test 16: invalid repudiation by unauthorised bob

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=chris&to_name=Chris&amount=12.0&details=test repudiation 1&repudiate=on&ack_id=%d' % (bobpin,next_ack_id),
              'expected':
                'Unauthorised attempt to: REPUD by account: bob logged.',
              'testno':testno,
              'description':'invalid repudiation by unauthorised bob' })

  # test 17: invalid repudiation by disabled admin

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=chris&to_name=Chris&amount=12.0&details=test repudiation 1&repudiate=on&ack_id=%d' % (v['pin'],next_ack_id),
              'expected':
                'Unauthorised attempt to: REPUD by account: admin logged.',
              'testno':testno,
              'description':'invalid repudiation by disabled admin' })

  # test 18: Locksmith authorises bob to REPUD

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=bob&pin=%s&ua=admin&add=add&auth=REPUD&whom=admin' % lockpin,
              'expected':"New authorisation REPUD added for bob",
              'testno':testno,
              'description':'locksmith authorises bob to repudiate payments' })

  # test 19: valid repudiation by bob

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=chris&to_name=Chris&amount=12.0&details=test repudiation 1&repudiate=on&ack_id=%d' % (bobpin,next_ack_id),
              'expected':
                'Acknowledgment: %d successfully input' % (-1*next_ack_id),
              'testno':testno,
              'description':'valid repudiation by bob now authorised' })

  # test 20: Alice unauthorised tries to view audit log

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&audit=audit' % alicepin,
                'expected':
                'Unauthorised attempt to: AUDITVIEW by account: alice logged.',
                'testno':testno,
                'description':'alice unauthorised cant view audit log' })

  # test 21: Locksmith authorises alice to AUDITVIEW 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=AUDITVIEW&whom=admin' % lockpin,
              'expected':"New authorisation AUDITVIEW added for alice",
              'testno':testno,
              'description':'locksmith authorises alice to view audit log' })

  # test 22: Locksmith authorises alice to ACCEPTREJECT 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=ACCEPTREJECT&whom=admin' % lockpin,
              'expected':"New authorisation ACCEPTREJECT added for alice",
              'testno':testno,
              'description':'alice auth to approve/reject applications' })

  # test 23: Locksmith authorises alice to APPLICATIONSVIEW 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=APPLICATIONSVIEW&whom=admin' % lockpin,
              'expected':"New authorisation APPLICATIONSVIEW added for alice",
              'testno':testno,
              'description':'alice auth to view applications' })

  # test 24: Alice views audit log

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&audit=audit' % alicepin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'alice views contents of audit log' })

  # test 25: Locksmith views authorisations

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&pin=%s&ua=admin&view=view' % lockpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'locksmith views authorisations' })

  # test 26: bob unauthorised cant view Alice's statement

  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&statement=statement&whom=alice' 
  	      % bobpin,
                'expected':'Unauthorised attempt to: STATEMENT by account: bob logged',
  		'testno':testno,
  		'description':'bob unauthorised cant view alice statement' })

  # test 27: Locksmith authorises bob to view alice's statement 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=bob&pin=%s&ua=admin&add=add&auth=STATEMENT&whom=alice' % lockpin,
              'expected':"New authorisation STATEMENT added for bob",
              'testno':testno,
              'description':'locksmith authorises bob to view alice statement' })

  # test 28: bob authorised can view Alice's statement

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&statement=statement&whom=alice' % bobpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'bob authorised can view alice statement'
 })

  # test 29: bob unauthorised can't view chris's statement
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&statement=statement&whom=chris' % bobpin,
                'expected':'Unauthorised attempt to: STATEMENT by account: bob logged',
  		'testno':testno,
  		'description':'bob unauthorised cant view chris statement' })

  # test 30: application 1 can be input 
  testno+=1
  confirm_form=open('../forms/confirmation.form').read()
  agreement_url=v['membership_agreement_url']
  registry=v['registry']
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'submit=submit&email1=%s&user_name=Joe Grundy&phone1=0234 56789&phone2=01798642468&district=Burbia&address=1 Pig Lane,\nAmbridge&postcode=BS3 4AM&other=Very good\nReferences' % te1,
              'expected':confirm_form % (registry,agreement_url,te1),
              'testno':testno,
              'description':'test application 1 can be input' })

  # test 31: application 2 can be input 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'submit=submit&email1=%s&user_name=Bill Sykes&phone1=0234 56799&phone2=01798642469&district=Burbia&address=3 Pig Lane,\nAmbridge&postcode=BS3 4AP&other=Even better\nreferences\nfor a known crook' % te2,
              'expected':confirm_form % (registry,agreement_url,te2),
              'testno':testno,
              'description':'test application 2 can be input' })
 
  # test 32: alice pays bob. Payments will be aged for yearend processing  

  testno+=1
  next_ack_id=3
  expected31='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=alice&pin=%s&ua=user&from_id=alice&from_name=Alice Wonder&to_id=bob&to_name=Bob Builder&amount=8.5&details=flower arranging' % alicepin,
              'expected':expected31,
              'testno':testno,
              'description':'alice pays bob to be aged for later yearend' })

  # test 33: bob pays chris. Payments will be aged for yearend processing  

  testno+=1
  next_ack_id=4
  expected33='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=bob&pin=%s&ua=user&from_id=bob&from_name=Bob Builder&to_id=chris&to_name=Chris Carpenter&amount=6.0&details=fencing parts' % bobpin,
              'expected':expected33,
              'testno':testno,
              'description':'bob pays chris to be aged for yearend processing' })

  # test 34: Locksmith authorises alice to WRITEOFF 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=WRITEOFF&whom=admin' % lockpin,
              'expected':"New authorisation WRITEOFF added for alice",
              'testno':testno,
              'description':'alice auth to perform writeoffs' })

  # test 35: Locksmith authorises alice to MEMBVIEW 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=MEMBVIEW&whom=admin' % lockpin,
              'expected':"New authorisation MEMBVIEW added for alice",
              'testno':testno,
              'description':'alice auth to view member records' })

  # test 36: Locksmith authorises alice to MEMBADD 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=MEMBADD&whom=admin' % lockpin,
              'expected':"New authorisation MEMBADD added for alice",
              'testno':testno,
              'description':'alice auth to add members' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

  # postrequisites 
  testlib.modify_config_var('test_configs/newmember.cfg',
      'system_has_numeric_accounts','N')
  testlib.hack_ackdates(3600*24*365*3) # age the transactions by 3 years

  # modify secure.cfg to regrant admin_has_superpowers=yes. 
  # probably need this to be restored after test18.py 
  #testlib.modify_config_var('test_configs/secure.cfg',
  #    'admin_has_superpowers','yes')

if __name__ == "__main__":
  main()
