#!/usr/bin/env python
''' test18.py Part E of Test suite for PyLETS CGI 

    test16.py - test19.py are designed to be used
    as a set.
    
    Purpose of tests:
    Account applications and approvals, various errors  
'''

def main(change_expected=False):
  print("\nTest Case 18: applications and approval/rejection. \n")

  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  # need alice's PIN
  apin=testlib.get_userpin('alice')
  bobpin=testlib.get_userpin('bob')
  lockpin=testlib.get_userpin('lock')
  te1=tv['test_email1']
  te2=tv['test_email2']
  ccapp1=testlib.get_confirmcode(te1)
  ccapp2=testlib.get_confirmcode(te2)
  confirm_form=open('../forms/confirmation.form').read()
  agreement_url=v['membership_agreement_url']
  registry=v['registry']

  tests=[]
  testcase=18
  testno=0

  # test 1: application 1 agreement has to be read to be confirmed

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'token=%s&email=%s&read=no&accept=accept' % (ccapp1,te1),
              'expected':'You must read the agreement before accepting it',
              'testno':testno,
              'description':'application 1 agreement must be read' })

  # test 2: application 1 doesn't say if agreement read or not 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'token=%s&email=%s&accept=accept' % (ccapp1,te1),
              'expected':'Missing fields for accept agreement. Complete all fields.',
              'testno':testno,
              'description':'doesnt say agreement read or not' })

  # test 3: application cant be confirmed with wrong token

  testno+=1
  ccwrong=str( int(ccapp1) - 1) # token sent -1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'token=%s&email=%s&read=yes&accept=accept' % (ccwrong,te1),
              'expected':'You must input the token sent to your email to continue.',
              'testno':testno,
              'description':'application cant be confirmed wrong token' })

  # test 4: application 1 can be confirmed

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'token=%s&email=%s&read=yes&accept=accept' % (ccapp1,te1),
              'expected':'Your application has been input for consideration',
              'testno':testno,
              'description':'application 1 can be confirmed' })

  # test 5: application 2 can be confirmed

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'token=%s&email=%s&read=yes&accept=accept' % (ccapp2,te2),
              'expected':'Your application has been input for consideration',
              'testno':testno,
              'description':'application 2 can be confirmed' })

  # test 6: alice can view applications

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'approval.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&view=view' % apin,
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'alice can view 2 applications' })
  
  # test 7: application 1 cant be approved using invalid account id 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'approval.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&active=active&email=%s&account=123a.y' % (apin,te1),
              'expected':'Invalid account identifier.',
              'testno':testno,
              'description':'application 1 not approved with invalid account' })

  # test 8: application 1 cant be approved using reserved account id 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'approval.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&active=active&email=%s&account=writeoff' % (apin,te1),
              'expected':'Account id requested is a reserved name.',
              'testno':testno,
              'description':'application 1 not approved with reserved account' })

  # test 9: application 1 can be approved for active membership

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'approval.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&active=active&email=%s&account=123' % (apin,te1),
              'expected':'new member added',
              'testno':testno,
              'description':'application 1 can be approved as active' })

  # test 10: application 2 can be rejected

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'approval.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&reject=reject&email=%s&account=123&reason=I just dont like you so go away.' % (apin,te2),
              'expected':'The rejection reason as input has been emailed to: %s' % te2,
              'testno':testno,
              'description':'application 2 can be rejected' })

  # test 11: application 2 can be reinput with handle

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'submit=submit&email1=%s&user_name=Billy Alias&phone1=0234 56799&phone2=01798642469&district=Burbia&address=3 Pig Lane,\nAmbridge&postcode=BS3 4AP&other=So ive changed my name will that do&handle=crook bogus badguy' % te2,
              'expected':confirm_form % (registry,agreement_url,te2),
              'testno':testno,
              'description':'test application 2 reinput with handle request' })

  # test 12: joe pays alice. Payments will be aged for yearend processing

  testno+=1
  next_ack_id=5
  expected12='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&from_id=123&from_name=Joe Grundy&to_id=alice&to_name=Alice Wonder&amount=11&details=editing' % bobpin, 
              'expected':expected12,
              'testno':testno,
              'description':'joe pays alice to be aged for later yearend' })

  # test 13: make account joe inactive

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=bob&user_ac_id=123&pin=%s&ua=admin&status=status&pai=inactive'
              % bobpin,
              'expected':'<p>Thanks for your input which has been received and',
              'testno':testno,
              'description':'inactivate account: Joe' })

  # test 14: make account chris inactive

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=bob&user_ac_id=chris&pin=%s&ua=admin&status=status&pai=inactive'
              % bobpin,
              'expected':'<p>Thanks for your input which has been received and',
              'testno':testno,
              'description':'inactivate account: Joe' })

  # test 15: Locksmith authorises alice to do yearend processing
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=lock&acbe=alice&pin=%s&ua=admin&add=add&auth=YEAREND&whom=admin' % lockpin,
              'expected':"New authorisation YEAREND added for alice",
              'testno':testno,
              'description':'locksmith authorises alice to yearend process' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)
  testlib.hack_ackdates(3600*24*365*2) # age the transactions by 2 years
 
if __name__ == "__main__":
  main()
