#!/usr/bin/env python
''' test19.py Part E of Test suite for PyLETS CGI 

    test16.py - test19.py are designed to be used
    as a set.
    
    Purpose of tests:
    Account applications and approvals, yearend, dead account
    identification and removal and writeoff processing  
'''

def main(change_expected=False):
  print("\nTest Case 19: application approval, yearend and writing off. \n")

  import testlib
  # restore config
  # modify secure.cfg admin has superpowers to normal value 
  testlib.modify_config_var('test_configs/secure.cfg',
      'admin_has_superpowers','yes')
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  # need alice's PIN
  alicepin=testlib.get_userpin('alice')
  bobpin=testlib.get_userpin('bob')
  lockpin=testlib.get_userpin('lock')
  te2=tv['test_email2']
  emd=tv['email_domain']
  ccapp2=testlib.get_confirmcode(te2)
  confirm_form=open('../forms/confirmation.form').read()
  agreement_url=v['membership_agreement_url']
  registry=v['registry']
  date3yago=testlib.get_datestr(secs=3600*24*365*-3) # format DD Mmm YYYY
  tests=[]
  testcase=19
  testno=0

  # test 1: application 2 can be confirmed

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'application.cgi',
              'data':'token=%s&email=%s&read=yes&accept=accept' % (ccapp2,te2),
              'expected':'Your application has been input for consideration',
              'testno':testno,
              'description':'application 2 can be confirmed' })

  # test 2: application 2 can be approved for provisional membership

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'approval.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&provis=provis&email=%s&account=crook' % (alicepin,te2),
              'expected':'new member added',
              'testno':testno,
              'description':'application 2 can be approved as provisional' })

  # test 3: alice can view all new and old members 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&all=all' % alicepin,
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'alice can see contact list with all members' })

  # test 4: alice pays crook. Payment will be fresh for yearend processing

  testno+=1
  next_ack_id=6
  expected4='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=bob&pin=%s&ua=admin&from_id=alice&from_name=Alice Wonder&to_id=crook&to_name=Billy Alias&amount=3.5&details=cowboy job' % bobpin,
              'expected':expected4,
              'testno':testno,
              'description':'alice pays crook recent transaction' })

  # test 5: alice performs yearend processing 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
              'data':'ac_id=alice&pin=%s&ua=admin&yearend=yearend&date=%s' % (alicepin,date3yago),
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst', 
              'testno':testno,
              'description':'alice performs yearend processing rotating acks' })

  # test 6: make account dave inactive

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=bob&user_ac_id=dave&pin=%s&ua=admin&status=status&pai=inactive' % bobpin,
              'expected':'<p>Thanks for your input which has been received and',
              'testno':testno,
              'description':'inactivate account: dave' })
  
  # test 7: alice views dead inactive accounts 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&viewinact=viewinact&date=%s' % (alicepin,date3yago),
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst', 
              'testno':testno,
              'description':'alice views dead inactive accounts ' })

  # test 8: alice views system balance, should be zero  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&balance=balance' % alicepin,
              'expected':'Accounts balance to within a penny. No correction possible.',
              'testno':testno,
              'description':'alice views system zero balance ' })

  # test 9: alice tries to create writeoff account but cant due to 0 balance 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&writeoff=writeoff' % alicepin,
              'expected':'Accounts balance to within a penny. No correction possible.',
              'testno':testno,
              'description':'alice fails to create writeoff account due to 0 balance' })


  # test 10: alice adds member with non zero balance unbalancing system 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=alice&user_ac_id=ethel&pin=%s&ua=admin&add=add&user_name=Poly Ethylene&email=nulle@%s&start_bal=15.0&phone1=053385 8877&phone2=0784484568&address=63 Holyhead Rd,Coventry&postcode=CV5 2XZ&district=Keresley&limit=None' % (alicepin,emd),
              'expected':'new member added',
              'testno':testno,
              'description':'valid add user non-zero start balance' })

  # test 11: alice views system balance, should not be zero  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&balance=balance' % alicepin,
              'expected':'The overall balance of the system is: 15.00',
              'testno':testno,
              'description':'alice views system non zero balance ' })

  # test 12: alice creates writeoff account and can due to system imbalance 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&writeoff=writeoff' % alicepin,
              'expected':'Thanks for your input which has been',
              'testno':testno,
              'description':'alice creates writeoff account to correct imbalance' })

  # test 13: dead inactives removed transferring balances to writeoff 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&clearout=clearout&date=%s' % (alicepin,date3yago),
              'expected':'2 long inactive account deletions',
              'testno':testno,
              'description':'clearout 2 dead inactive accounts removed' })

  # test 14: get balance of writeoff account 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=alice&enquiry_id=writeoff&pin=%s&ua=user'
              % alicepin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':"balance view of writoff account" })
 
  # test 15: alice adds system account member 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=alice&user_ac_id=system&pin=%s&ua=admin&add=add&user_name=System Account&email=None&start_bal=0.0&phone1=None&phone2=None&address=Nowhere Near&district=unknown&postcode=ZZ99 9ZZ&limit=None' % alicepin,
              'expected':'new member added',
              'testno':testno,
              'description':'valid add user system account' })
  
  # test 16: writeoff account cant be removed with non zero balance  

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=writeoff&pin=%s&ua=admin&delete=delete' % v['pin'],
                'expected':"Account has non-zero balance. Unwise to delete it. Clear it first.",
                'testno':testno,
                'description':'cant delete writeoff account with balance' })

  # test 17: balance in writeoff account transferred into system account  

  testno+=1
  next_ack_id=7
  expected17='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=system&from_name=System Account&to_id=writeoff&to_name=Writeoff Account&amount=9.0&details=cost of writing off dead accounts' % v['pin'],
              'expected':expected17,
              'testno':testno,
              'description':'system makes writeoff payment' })

  # test 18: writeoff account can be removed with zero balance  

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=writeoff&pin=%s&ua=admin&delete=delete' % v['pin'],
                'expected':"Thanks for your input which has been received",
                'testno':testno,
                'description':'can delete writeoff account with zero balance' })

  # test 19: obtain summary showing all accounts 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=alice&pin=%s&ua=user&all=all' % alicepin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'valid summary showing all users' })

  # test 20: alice views system balance, should be zero again 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'yearend.cgi',
                'data':'ac_id=alice&pin=%s&ua=admin&balance=balance' % alicepin,
              'expected':'Accounts balance to within a penny. No correction possible.',
              'testno':testno,
              'description':'alice views restored overall zero balance ' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

if __name__ == "__main__":
  main()
