#!/usr/bin/env python
''' test2.py Part 2 of Test suite for PyLETS CGI 
    
    Purpose of tests:
    a. handling errors using admin.cgi
    b. adding 3 accounts to system
    c. ability to modify account.

    Prerequisites: system must not already contain
    accounts for alice, bob or chris. System must
    contain account for joe
'''
def main(change_expected=False):  
  print("\nTest Case 2: admin.cgi errors, add/mod accounts.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  testcase=2
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  joepin=testlib.get_userpin('joe')
  emd=tv['email_domain'] # test email domain  
  tests=[]
  
  # test 1: no admin form data
  testno=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'',
  	      'expected':'Missing authentication details.',
  	      'testno':testno,
  	      'description':'no admin form data' })
  
  # test 2: no account
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin',
  	      'expected':"Missing account identifier or PIN.",
  	      'testno':testno,
  	      'description':'test no account or pin field' })
  
  # test 3: invalid use by non admin account 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
          'data':'ac_id=joe&pin=%s&ua=user&user_ac_id=bob&add=add' % joepin,
  	      'expected':'You must login as admin to do this.',
  	      'testno':testno,
  	      'description':'use of admin.cgi by non admin user' })
  
  # test 4: invalid PIN format 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
          'data':'ac_id=admin&pin=abcd&ua=admin&user_ac_id=bob&add=add',
  	      'expected':"Invalid PIN format.",
  	      'testno':testno,
  	      'description':'invalid PIN, not in range 10000000 - 99999999' })
  
  # test 5: invalid user account format 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
          'data':'ac_id=admin&pin=%s&ua=admin&user_ac_id=b#ob&add=add' % v['pin'],
  	      'expected':"Invalid user account id/no.",
  	      'testno':testno,
  	      'description':'invalid user account ID format' })
  
  # test 6: wrong admin PIN entered  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
         'data':'ac_id=admin&pin=%s&ua=admin&user_ac_id=bob&add=add' 
              % str(int(v['pin'])-1),
  	      'expected':"Invalid or stale or logged out session.",
  	      'testno':testno,
  	      'description':'wrong admin PIN entered' })
  
  # test 7: no valid submit button used  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin' % v['pin'],
  	      'expected':"Invalid submit button value.",
  	      'testno':testno,
  	      'description':'no valid submit button used' })
  
  # test 8: add account with missing field  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin&add=add&user_name=Bob' % v['pin'],
  	      'expected':"Missing field for add member. Complete all fields.",
  	      'testno':testno,
  	      'description':'attempt to add user but missing field/s' })
  
  # test 9: add account with invalid email  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin&add=add&user_name=Bob&email=bob@b@bob.bob&start_bal=0.0&phone1=01234&phone2=12345&address=1 Pig Lane\nLeicester&postcode=LE1 2BC&district=Stoke Aldermoor&limit=None' % v['pin'],
  	      'expected':"Invalid email address.",
  	      'testno':testno,
  	      'description':'add user with invalid email' })
  
  # test 10: add account with invalid user name  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin&add=add&user_name=Bo$b&email=bob@bob.bob&start_bal=0.0&phone1=01234&phone2=12345&address=1 Pig Lane\nLeicester&postcode=LE1 2BC&district=Stoke Aldermoor&limit=None' % v['pin'],
  	      'expected':"Invalid name.",
  	      'testno':testno,
  	      'description':'add user with invalid name' })
  
  # test 11: add account which already exists  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=admin&pin=%s&ua=admin&add=add&user_name=Bob&email=bob@bob.bob&start_bal=0.0&phone1=01234&phone2=12345&address=1 Pig Lane\nLeicester&postcode=LE1 2BC&district=Stoke Aldermoor&limit=None' % v['pin'],
  	      'expected':"Account number/id exists. Can't add account.",
  	      'testno':testno,
  	      'description':'add account which already exists' })
  
  # test 12: add account non-float starting balance  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin&add=add&user_name=Bob&email=bob@bob.bob&start_bal=abc&phone1=01234&phone2=12345&address=1 Pig Lane\nLeicester&postcode=LE1 2BC&district=Stoke Aldermoor&limit=None' % v['pin'],
  	      'expected':"Invalid starting balance.",
  	      'testno':testno,
  	      'description':'add user with non-numeric start balance' })
  
  # test 13: add account with out of range starting balance  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin&add=add&user_name=Bob&email=bob@bob.bob&start_bal=1000001.0&phone1=01234&phone2=12345&address=1 Pig Lane\nLeicester&postcode=LE1 2BC&district=Stoke Aldermoor&limit=None' % v['pin'],
  	      'expected':"starting balance too high or low.",
  	      'testno':testno,
  	      'description':'add user with out of range start balance' })
  
  # test 14: add account valid request 1 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=alice&pin=%s&ua=admin&add=add&user_name=Alice&email=nulla@%s&start_bal=0.0&phone1=012345&phone2=123456789&address=1 Pig Land\nLeicester&postcode=LE1 2BC&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Alice' })
  
  # test 15: add account valid request 2 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=bob&pin=%s&ua=admin&add=add&user_name=Bob&email=nullb@%s&start_bal=0.0&phone1=0123 456 7890&phone2=0781 876543&address=12 Bog St\nCoventry&postcode=CV2 3TY&district=Foleshill&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Bob' })
  
  # test 16: add account valid request 3 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&add=add&user_name=Chris&email=nullc@%s&start_bal=5.0&phone1=0123 456 7890&phone2=0781 888543&address=12 Bog St\nCoventry&postcode=CV2 3TY&district=Earlsdon&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Chris' })
  
  # test 17: mod account which does not exist 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=xyzqh&pin=%s&ua=admin&modify=modify&user_name=Chris&email=nullc@%s&start_bal=0.0' % (v['pin'],emd),
  	      'expected':"Account id/no. doesn't exist. Can't modify.",
  	      'testno':testno,
  	      'description':'mod user: account does not exist' })
  
  # test 18: mod account with invalid email address 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&user_name=Chris&email=chris@co.net.&start_bal=0.0' % v['pin'],
  	      'expected':"Invalid email address.",
  	      'testno':testno,
  	      'description':'mod user invalid email address' })
  
  # test 19: mod account with invalid user name 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&user_name=Chris$topher&email=nullc@%s&start_bal=0.0' % (v['pin'],emd),
  	      'expected':"Invalid name.",
  	      'testno':testno,
  	      'description':'mod user: invalid name' })
  
  # test 20: mod account with non-numeric start balance 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&user_name=Christopher&email=nullc@%s&start_bal=abc' % (v['pin'],emd),
  	      'expected':"Invalid starting balance.",
  	      'testno':testno,
  	      'description':'mod user: non numeric start balance' })
  
  # test 21: mod account with start balance out of range 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&user_name=Christopher&email=nullc@%s&start_bal=-1000005.0' % (v['pin'],emd),
  	      'expected':"starting balance too high or low.",
  	      'testno':testno,
  	      'description':'mod user: too low start balance' })
  
  # test 22: mod account with no modification data provided 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify' % v['pin'],
  	      'expected':"No modified field values submitted.",
  	      'testno':testno,
  	      'description':'mod user: no changed fields provided' })
  
  # test 23: mod account with invalid phone1 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&phone1=ABC123' % v['pin'],
  	      'expected':"Invalid phone1.",
  	      'testno':testno,
  	      'description':'mod user invalid phone 1' })
  
  # test 24: mod account with invalid phone2 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&phone2=ABC*123' % v['pin'],
  	      'expected':"Invalid phone2.",
  	      'testno':testno,
  	      'description':'mod user invalid phone 2' })
  
  # test 25: mod account with invalid postal address 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&address=ABC*123' % v['pin'],
  	      'expected':"Invalid address.",
  	      'testno':testno,
  	      'description':'mod user invalid postal address' })
  
  # test 26: mod account with invalid postcode 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&postcode=ABC*123' % v['pin'],
  	      'expected':"Invalid postcode.",
  	      'testno':testno,
  	      'description':'mod user invalid postcode' })
  
  # test 27: mod account with invalid district 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&district=123ABC' % v['pin'],
  	      'expected':"Invalid district use letters a-z and space only.",
  	      'testno':testno,
  	      'description':'mod user invalid district' })
  
  # test 28: mod account with invalid limit 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&limit=nothing' % v['pin'],
  	      'expected':"Invalid limit. Must be numeric or None .",
  	      'testno':testno,
  	      'description':'mod user invalid limit wrong string for None' })
  
  # test 29: mod account with too high limit 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&limit=100000000' % v['pin'],
  	      'expected':"Debit limit is too high or low.",
  	      'testno':testno,
  	      'description':'mod user invalid limit out of range' })

  # test 30: mod account valid request 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&user_name=Christopher&email=nullc@%s&start_bal=0.0&limit=-300.00' % (v['pin'],emd),
  	      'expected':"4 field/s modified.",
  	      'testno':testno,
  	      'description':'mod user: change 4 fields' })
 
  # test 31: view audit log after previous tests including mod user 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
              'data':'ac_id=admin&pin=%s&ua=admin&audit=audit' % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
              'testno':testno,
              'description':'view audit log showing previous activity' })
 
  # test 32: authorise chris to add members 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=chris&pin=%s&ua=admin&add=add&auth=MEMBADD&whom=admin' % v['pin'],
  	      'expected':"New authorisation MEMBADD added for chris",
  	      'testno':testno,
  	      'description':'authorise user chris to MEMBADD' })
  
  # test 33: authorise chris to del members 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=chris&pin=%s&ua=admin&add=add&auth=MEMBDEL&whom=admin' % v['pin'],
  	      'expected':"New authorisation MEMBDEL added for chris",
  	      'testno':testno,
  	      'description':'authorise user chris to MEMBDEL' })
  
  # test 34: authorise chris to mod members 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=chris&pin=%s&ua=admin&add=add&auth=MEMBMOD&whom=admin' % v['pin'],
  	      'expected':"New authorisation MEMBMOD added for chris",
  	      'testno':testno,
  	      'description':'authorise user chris to MEMBMOD' })
  
  # test 35: send balance form to joe user login 
  testno+=1
  ua='user'
  balanceform=open('../forms/balance.form').read()
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
              'data':'ac_id=joe&balance=balance&pin=%s&ua=user' % joepin,
              'expected':balanceform % (v['registry'],
                        'joe','VARIABLE_COOKIE',ua),
              'testno':testno,
              'description':'send joe the balance enquiry form' })
  
  # test 36: joe cant use admin login without authorisations 
  testno+=1
  errormes='No administrative authorisations. Please login as user.'
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=joe&pin=%s&admin=admin' % joepin,
              'expected':errormes,
              'testno':testno,
              'description':'joe cant login as admin without authorisation' })
 
  # test 37: add account invalid phone2 because duplicated 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=ethel&pin=%s&ua=admin&add=add&user_name=Alice&email=nulla@%s&start_bal=0.0&phone1=012345&phone2=123456789&address=1 Pig Land\nLeicester&postcode=LE1 2BC&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"phone2 123456789 belongs to alice",
  	      'testno':testno,
  	      'description':'invalid add user someone elses phone2' })
  
  # test 38: mod account with invalid phone2 because duplicated 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&phone2=123456789' % v['pin'],
  	      'expected':"phone2 123456789 belongs to alice",
  	      'testno':testno,
  	      'description':'mod user invalid someone elses phone 2' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)
 
if __name__ == "__main__":
  main() 
