#!/usr/bin/env python
''' test3.py Part 3 of Test suite for PyLETS CGI 
    
    Purpose of tests:
    a. handling more errors using admin.cgi
    b. making account inactive 
    c. making account active,
    d. ability to view account
    e. ability to delete account
    f. setup accounts for test4.py testing of tithes etc.

    Prerequisites: system must not already contain
    account for dave. Must contain accounts for alice, bob, joe and
    chris, chris is authorised to MEMBADD, MEMBDEL, MEMBMOD .
    Postrequisites: accounts for faith, hope and charity must
    exist. Tithes variables set.
'''
def main(change_expected=False):
  print("\nTest Case 3: admin.cgi active/inactive/del accounts.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  chrispin=testlib.get_userpin('chris')
  alicepin=testlib.get_userpin('alice') 
  joepin=testlib.get_userpin('joe')
  emd=tv['email_domain'] # test email domain 
  #
  tests=[]
  testcase=3
  
  # test 1: check chris as authorised can add an account for dave

  testno=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=chris&user_ac_id=dave&pin=%s&ua=admin&add=add&user_name=Dave&email=nulld@%s&start_bal=0.0&phone1=012345678&phone2=0781 987654&address=10 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (chrispin,emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid chris authorised can add user: Dave' })
  
  # test 2 check alice as not authorised cant add an account for ethel

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=alice&user_ac_id=ethel&pin=%s&ua=admin&add=add&user_name=Ethel&email=nulle@%s&start_bal=0.0&phone1=012345678&phone2=0781 999654&address=10 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (alicepin,emd),
  	      'expected':'Unauthorised attempt to: MEMBADD by account: alice logged.',
  	      'testno':testno,
  	      'description':'check alice unauthorised cant add an account' })
  
  # test 3 check alice as not authorised cant del an account for dave 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=alice&user_ac_id=dave&pin=%s&ua=admin&delete=delete' % alicepin,
  	      'expected':'Unauthorised attempt to: MEMBDEL by account: alice logged.',
  	      'testno':testno,
  	      'description':'check alice unauthorised cant delete an account' })

  # test 4 check chris as authorised can modify daves account

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=chris&user_ac_id=dave&pin=%s&ua=admin&modify=modify&email=nulld@%s&phone1=123456789&phone2=0781 987655&district=Earlsdon' % (chrispin,emd),
  	      'expected':"4 field/s modified",
  	      'testno':testno,
  	      'description':'valid chris authorised can modify user: Dave' })
  
  # test 5 check chris as authorised can delete daves account

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=chris&user_ac_id=dave&pin=%s&ua=admin&delete=delete' % chrispin,
  	      'expected':'Thanks for your input which has been received',
  	      'testno':testno,
  	      'description':'check chris authorised can delete daves account' })

  # test 6: inactivate non-exising account for dave 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&status=status&pai=inactive&user_name=Dave' % v['pin'],
  	      'expected':"Account id/no. doesn't exist. Can't make inactive.",
  	      'testno':testno,
  	      'description':'inactivate non-existing account' })
  
  # test 7: activate non-exising account for dave 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&status=status&pai=active&user_name=Dave' % v['pin'],
  	      'expected':"Account id/no. doesn't exist. Can't make active.",
  	      'testno':testno,
  	      'description':'activate non-existing account' })
  
  # test 8: re-add account for dave 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&add=add&user_name=Dave&email=nulld@%s&start_bal=0.0&phone1=012345678&phone2=0781 987654&address=10 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Dave' })
  
  # test 9: inactivate exising account for dave 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&status=status&pai=inactive&user_name=Dave' % v['pin'],
  	      'expected':"Thanks for your input which has been received and",
  	      'testno':testno,
  	      'description':'valid inactivate existing account' })
  
  # test 10: activate exising account for dave 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&status=status&pai=active&user_name=Dave' % v['pin'],
  	      'expected':"Thanks for your input which has been received and",
  	      'testno':testno,
  	      'description':'valid activate existing account' })
  
  # test 11: valid view existing account
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&view=view&user_name=Dave' % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'valid view existing account' })
  
  # test 12: attempt invalid delete of account for admin member  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=admin&pin=%s&ua=admin&delete=delete' % v['pin'],
  	      'expected':"Account id/no. has member record but no balance record.",
  	      'testno':testno,
  	      'description':'invalid delete of admin user' })
  
  # test 13: attempt valid delete of account with zero balance  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&delete=delete' % v['pin'],
  	      'expected':"Thanks for your input which has been received and",
  	      'testno':testno,
  	      'description':'valid delete of user with zero balance' })
  
  # test 14: attempt invalid delete of account which does not exist  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&delete=delete' % v['pin'],
  	      'expected':"Account id/no. doesn't exist. Can't delete.",
  	      'testno':testno,
  	      'description':'invalid delete non-existing user' })
  
  # test 15: attempt invalid view of account which does not exist  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=dave&pin=%s&ua=admin&view=view' % v['pin'],
  	      'expected':"Account id/no. doesn't exist. Can't view it.",
  	      'testno':testno,
  	      'description':'invalid view non-existing user' })
  
  # test 16: attempt invalid view of account for admin member  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=admin&pin=%s&ua=admin&view=view' % v['pin'],
  	      'expected':"Account id/no. has member record but no balance record.",
  	      'testno':testno,
  	      'description':'invalid view of account for admin user' })
  
  # test 17: add account for hope 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=hope&pin=%s&ua=admin&add=add&user_name=Hope&email=nullh@%s&start_bal=0.0&phone1=012345678&phone2=0781 987654&address=12 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Hope' })
  
  # test 18: add account for faith 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=faith&pin=%s&ua=admin&add=add&user_name=Faith&email=nullf@%s&start_bal=0.0&phone1=012345678&phone2=0792 987654&address=14 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Faith' })
  
  # test 19: add account for charity 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=charity&pin=%s&ua=admin&add=add&user_name=Charity&email=nullc@%s&start_bal=0.0&phone1=012345678&phone2=0788 997654&address=16 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Charity' })
 
  # test 20: check joe account can login and view balance when active  

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=joe&enquiry_id=charity&pin=%s&ua=user'
              % joepin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':"joe requests charity's balance" })

  # test 21: inactivate joe account   
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=joe&pin=%s&ua=admin&status=status&pai=inactive' % v['pin'],
  	      'expected':"Thanks for your input which has been received and",
  	      'testno':testno,
  	      'description':'valid inactivate existing joe account' })

  # test 22: confirm joe account can not login when inactive 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=joe&enquiry_id=charity&pin=%s&ua=user'
              % joepin,
                'expected':'Invalid or stale or logged out session.',
                'testno':testno,
                'description':"inactive joe cant get charity's balance" })

  # test 23: attempt valid delete of joe account with zero balance  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=joe&pin=%s&ua=admin&delete=delete' % v['pin'],
  	      'expected':"Thanks for your input which has been received and",
  	      'testno':testno,
  	      'description':'valid delete of joe with zero balance' })

  # test 24: attempt reset PIN of charity account
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=charity&pin=%s&ua=admin&rpin=rpin' % v['pin'],
  	      'expected':"PIN emailed for account: charity to address:",
  	      'testno':testno,
  	      'description':'valid reset of charity account pin' })
 
  # test 25: invalid attempted reset PIN of deleted joe account
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=joe&pin=%s&ua=admin&rpin=rpin' % v['pin'],
  	      'expected':"Account id/no. doesn't exist. Can't reset PIN.",
  	      'testno':testno,
  	      'description':'invalid reset of pin for non existent account' })
 
 
  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

  # postrequisite
  testlib.modify_config_var('test_configs/tithes.cfg','tithes','y')
  testlib.modify_config_var('test_configs/tithes.cfg','untithed_from_acs','alice,bob,chris,charity')
  testlib.modify_config_var('test_configs/tithes.cfg','untithed_to_acs','alice,bob,chris,charity')
  testlib.modify_config_var('test_configs/tithes.cfg','to_ac','charity')
 
if __name__ == "__main__":
  main() 
