#!/usr/bin/env python
''' test4.py Part 4 of Test suite for PyLETS CGI 
    
    Purpose of tests:
    a. handling various errors using ack.cgi
    b. acknowledgement handling
    c. repudiation handling
    d. ablity to send ack1 and ack2 forms
    e. various errors with ack.cgi
    f. tithed payments involving users hope and faith who 
       pay tithes to charity account.
    g. effect of limits preventing chris from paying below his
       debit limit

    Prerequisites: 
    System must contain accounts for alice, bob and chris in list of
    untithed accounts to maintain older test validity. Chris must be
    debit limited.
    System must contain accounts for hope, faith and charity.
    hope and faith not in list of tithed accounts, tithing to charity which 
    is in list of untithed accounts.  
    Must not contain account axxxx.

'''
def main(change_expected=False):
  print("\nTest Case 4: ack.cgi errors and acknowledgements.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  next_ack_id=testlib.get_nextackid()
  # need alices pin to attempt unauthorised repudiation 
  alicepin=testlib.get_userpin('alice')  
  tests=[]
  testcase=4
  
  # test 1: no authentication details
  testno=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'',
                'expected':"Missing authentication details.",
  	        'testno':testno,
                'description':'no authentication details' })
  
  # test 2: send ack1 form  
  testno+=1
  ack1form=open('../forms/ack1.form').read()
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
              'data':'ack=ack&ac_id=admin&pin=%s&ua=admin' % v['pin'],
  	      'expected':ack1form % (v['registry'],'admin','VARIABLE_COOKIE',
                'admin',''),
  	      'testno':testno,
  	      'description':'send ack1 form' })

  # test 3: send ack2 form  
  testno+=1
  ack2form=open('../forms/ack2.form').read()
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&to_id=bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':ack2form % (v['registry'],'admin',
                'VARIABLE_COOKIE','admin','Bob','bob','Alice',
                'alice','10.0','test ack 1'),
  	      'testno':testno,
  	      'description':'send ack2 form' })
  
  # test 4: process valid untithed ack  
  
  testno+=1
  expected2='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':expected2,
  	      'testno':testno,
  	      'description':'process valid acknowledgement no tithe' })
  
  # test 5: process valid repudiation stage 1 (send repud2.form)
  
  testno+=1
  repud2form=open('../forms/repud2.form').read()
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
              'data':'ac_id=admin&pin=%s&ua=admin&ack_id=%d&repudiate=on' % (v['pin'], next_ack_id),
  	      'expected':repud2form % (v['registry'],'admin',
                      'VARIABLE_COOKIE',
                      'admin',str(next_ack_id),
                      "Bob","bob","Alice","alice","10.0",
  		      "REPUDIATION ack_id: "+str(next_ack_id)),
  	      'testno':testno,
  	      'description':'process valid repudiation stage 1' })
  
  # test 6: missing ack details 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob' % v['pin'],
  	      'expected':'Missing value/s. Complete all form fields',
  	      'testno':testno,
  	      'description':'missing acknowledgement details' })
  
  # test 7: invalid user ac_id  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=ad^min&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'Invalid user account id/no',
  	      'testno':testno,
  	      'description':'invalid ac_id' })
  
  # test 8: invalid PIN format  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % '12X4',
  	      'expected':'Invalid PIN format',
  	      'testno':testno,
  	      'description':'invalid PIN format' })
  
  # test 9: invalid user submitting 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=axxxx&pin=%s&ua=user&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'Invalid or stale or logged out session.',
  	      'testno':testno,
  	      'description':'nonexistent submitting ac_id' })
  
  # test 10: incorrect PIN  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % str(int(v['pin'])+1),
  	      'expected':'Invalid or stale or logged out session.',
  	      'testno':testno,
  	      'description':'wrong PIN' })
  
  # test 11: invalid repudiation request   
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1&repudiate=off&ack_id=%d'
  	      % (v['pin'],next_ack_id),
  	      'expected':'Invalid repudiation request field',
  	      'testno':testno,
  	      'description':'invalid repudiation request' })
  
  # test 12: invalid repudiation ack_id   
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1&repudiate=on&ack_id=x' 
  	      % v['pin'],
  	      'expected':'Invalid repudiation ack_id',
  	      'testno':testno,
  	      'description':'Invalid repudiation ack_id' })
  
  # test 13: repudiation ack_id not found  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1&repudiate=on&ack_id=%d' 
  	      % (v['pin'],next_ack_id+1),
  	      'expected':'repudiation ack_id not found',
  	      'testno':testno,
  	      'description':'repudiation ack_id not found' })
  
  # test 14: invalid from account id/no field  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=a*lice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'Invalid from account id/no',
  	      'testno':testno,
  	      'description':'Invalid from account id/no field' })
  
  # test 15: nonexistent From Account no./id field  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=axxxx&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'From Account no./id not found',
  	      'testno':testno,
  	      'description':'nonexistent From Account no./id field' })
  
  # test 16: from a/c has member record but no balance record 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=admin&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'from a/c has member record but no balance record',
  	      'testno':testno,
  	      'description':'from a/c has member record but no balance record' })
  
  # test 17: invalid to account id/no field  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=a?x&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'Invalid to account id/no',
  	      'testno':testno,
  	      'description':'invalid to account id/no field' })
  
  # test 18: To Account no./id not found 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=axxxx&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'To Account no./id not found',
  	      'testno':testno,
  	      'description':'To Account no./id not found' })
  
  # test 19: To a/c has member record but no balance record 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=admin&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'To a/c has member record but no balance record',
  	      'testno':testno,
  	      'description':'To a/c has member record but no balance record' })
  
  # test 20: invalid amount field  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.x&details=test ack 1' % v['pin'],
  	      'expected':'Invalid amount, not numeric.',
  	      'testno':testno,
  	      'description':'non numeric amount field' })
  
  # test 21: Invalid amount, too large  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=1000.1&details=test ack 1' % v['pin'],
  	      'expected':'Invalid amount, zero, negative or too large.',
  	      'testno':testno,
  	      'description':'too large amount' })
  
  # test 22: Invalid amount, negative 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=-1.0&details=test ack 1' % v['pin'],
  	      'expected':'Invalid amount, zero, negative or too large.',
  	      'testno':testno,
  	      'description':'negative amount' })
  
  # test 23: Invalid details 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test # ack 1' % v['pin'],
  	      'expected':'Invalid details',
  	      'testno':testno,
  	      'description':'Invalid details' })
  
  # test 24: Invalid From name 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=A@lice&to_id=bob&to_name=Bob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'Invalid From name',
  	      'testno':testno,
  	      'description':'Invalid From name' })
  
  # test 25: Invalid To name 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=B*ob&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'Invalid To name',
  	      'testno':testno,
  	      'description':'Invalid To name' })
  
  # test 26: attempt unauthorised repudiation by user  
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=alice&pin=%s&ua=user&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test repudiation 1&repudiate=on&ack_id=%d' 
  	      % (alicepin,next_ack_id),
  	      'expected':'Unauthorised attempt to: REPUD by account: alice logged.',
  	      'testno':testno,
  	      'description':'unauthorised repudiation by user' })

  # test 27: valid repudiation 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=alice&from_name=Alice&to_id=bob&to_name=Bob&amount=10.0&details=test repudiation 1&repudiate=on&ack_id=%d' 
  	      % (v['pin'],next_ack_id),
  	      'expected':'Acknowledgment: %d successfully input' 
  	      % (-1*next_ack_id),
  	      'testno':testno,
  	      'description':'valid repudiation' })
  
  # test 28: process valid tithed ack  
  
  testno+=1
  next_ack_id+=2 # every successful ack.addrow adds 1 to this - including the repudiation
  expected27='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=hope&from_name=Hope&to_id=faith&to_name=Faith&amount=50.0&details=test tithed ack' % v['pin'],
  	      'expected':expected27,
  	      'testno':testno,
  	      'description':'process valid tithed acknowledgement' })

  # test 29: view summary as alice showing results of transactions 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=alice&pin=%s&ua=user&all=all'
              % alicepin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
                'testno':testno,
                'description':'summary showing tithed and untithed accounts' })

  # test 30: modify limit for chris
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=chris&pin=%s&ua=admin&modify=modify&user_name=Christopher Wren&limit=-200.00' % v['pin'],
              'expected':"2 field/s modified.",
              'testno':testno,
              'description':'mod chris: change 2 fields name and limit' })

  # test 31: process valid untithed ack within chris limit 
  
  testno+=1
  next_ack_id+=2
  expected30='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=chris&from_name=Chris&to_id=bob&to_name=Bob&amount=100.0&details=test ack from chris valid' % v['pin'],
  	      'expected':expected30,
  	      'testno':testno,
  	      'description':'process valid ack within chris limit' })

  # test 32: process invalid ack outside chris limit 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&from_id=chris&from_name=Chris&to_id=bob&to_name=Bob&amount=300.0&details=test over limit ack' % v['pin'],
  	      'expected':'Payment blocked due to insufficient funds.',
  	      'testno':testno,
  	      'description':'payment taking account below limit blocked' })

  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)
if __name__ == "__main__":
  main() 
