#!/usr/bin/env python
''' test5.py part of part B of test suite for PyLETS CGI

    test5 - test8 are designed to be runnable as a unit
    from a clean test system installation.
    
    Purpose of tests:
    a. adding 2 new accounts to system
    b. make 1 account inactive
    c. make invalid payment to inactive account
    d. make account active
    e. make valid payment between accounts
    f. checking account balances as admin

    prerequisites: system must
    not already contain accounts for jack or jill (use make_clean ).
    postrequisites: secure.cfg variable who_can_ack set to ac_can_pay
 
'''
def main(change_expected=False):
  print("\nTest Case 5: make payments, check balances as admin.\n")
  
  import testlib 
  # prerequisite: turn tithing off
  testlib.modify_config_var('test_configs/tithes.cfg','tithes','n')

  # get test configuation variables in dictionary v
  v=testlib.get_vars()
  tv=testlib.get_vars(cfile='test.cfg')
  emd=tv['email_domain']
  v['pin']=testlib.run_make_clean()
  next_ack_id=testlib.get_nextackid()
  
  tests=[]
  testcase=5
  testno=0  

  # test 1: add account jack valid request 1 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&add=add&user_name=Jack&email=nullj@%s&start_bal=0.0&phone1=01234 56789&phone2=0781 987654&address=7 Drury Lane,\nLondon&postcode=SW1 2WW&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Jack' })
  
  # test 2: add account jill valid request 2 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jill&pin=%s&ua=admin&add=add&user_name=Jill&email=nullk@%s&start_bal=0.0&phone1=012345 6897&phone2=None&address=2 Tuppence Rd,Coventry&postcode=CV2 2JX&district=Hillfields&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Jill' })
  
  # test 3: make account jill inactive 

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jill&pin=%s&ua=admin&status=status&pai=inactive' 
  	      % v['pin'],
  	      'expected':'<p>Thanks for your input which has been received and',
  	      'testno':testno,
  	      'description':'inactivate account: Jill' })
  
  # test 4: process invalid ack due to to account being inactive
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&from_id=jack&from_name=Jack&to_id=jill&to_name=Jill&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':'<p>  To Account is inactive. </p>',
  	      'testno':testno,
              'description':'acknowledgement to inactive account' })
  
  # test 5: make account jill active 
  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jill&pin=%s&ua=admin&status=status&pai=active' 
  	      % v['pin'],
  	      'expected':'<p>Thanks for your input which has been received and',
  	      'testno':testno,
  	      'description':'re-activate account: Jill' })
  
  # test 6: process valid ack
  
  testno+=1
  expected2='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
  	      'data':'ac_id=admin&pin=%s&ua=admin&from_id=jack&from_name=Jack&to_id=jill&to_name=Jill&amount=10.0&details=test ack 1' % v['pin'],
  	      'expected':expected2,
  	      'testno':testno,
  	      'description':'process valid acknowledgement' })
  
  # test 7: get balance as admin

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jill&pin=%s&ua=admin&view=view' 
  	      % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'get balance as admin' })
 
  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

  # modify secure.cfg who_can_ack to ac_can_pay - needed for test6.py 
  testlib.modify_config_var('test_configs/secure.cfg','who_can_ack','ac_can_pay')

if __name__ == "__main__":
  main()
