#!/usr/bin/env python
''' test6.py of Test suite for PyLETS CGI

    test5 - test8 are designed to be runnable as a unit
    from a clean system.
    
    Purpose of tests:
    a. checking summary as user
    b. checking account balance as user
    c. checking account balance as user wrong account/pin
    d. repudiating the payment
    e. checking account balances are correct following repudiation
    f. checking jill can pay jack with who_can_ack ac_can_pay 
    f. checking jill can not pay from jack to herself with who_can_ack ac_can_pay 
    g. deleting user account
    h. obtaining account statement
    i. obtaining ledger as admin 
    j. obtaining ledger as non admin must fail
    k. re-adding jack's account for further tests
    l. adding adam's account for further tests

    Prerequisites: test set 5 must be completed to add jill's
    and jack's accounts before jill's PIN is known etc.
    secure.cfg variable who_can_ac set to ac_can_pay
    Postrequisites:  
    secure.cfg variable who_can_ack set to admin_alone 
'''

def main(change_expected=False):
  print("\nTest Case 6: check summary, balances, statement, repudiate, ledger.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  emd=tv['email_domain']
  next_ack_id=testlib.get_nextackid()
  repud_ackid=next_ack_id-1 # ID of last ack entered
  
  tests=[]
  testcase=6
  
  # test 1: obtain summary as jill
  
  # need jill's PIN first
  jpin=testlib.get_userpin('jill')

  testno=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&all=all' 
  	      % jpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'valid summary for jack and jill' })
  
  # test 2: get balance as user 

  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=jill&enquiry_id=jack&pin=%s&ua=user' 
  	      % jpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  	        'description':"jill requests jack's balance" })
  
  # test 3: checking account balance as user wrong account

  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=jillian&enquiry_id=jack&pin=%s&ua=user' 
  	      % jpin,
  	      'expected':'Invalid or stale or logged out session.',
  	      'testno':testno,
  	      'description':"wrong account requests jack's balance" })

  # test 4: checking account balance as user wrong pin

  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=jill&enquiry_id=jack&pin=%s&ua=user' 
  	      % str(int(jpin)+1),
  	      'expected':'Invalid or stale or logged out session.',
  	      'testno':testno,
  	      'description':"jill wrong PIN requests jack's balance" })

  # test 5: invalid attempt to delete account with non-zero balance 
  
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
  		'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&delete=delete' % v['pin'],
  		'expected':"Account has non-zero balance. Unwise to delete it. Clear it first.",
  		'testno':testno,
  		'description':'invalid delete a/c non-zero balance' })
  
  # test 6: repudiating the payment
  
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
  		'data':'ac_id=admin&pin=%s&ua=admin&from_id=jack&from_name=Jack&to_id=jill&to_name=Jill&amount=10.0&details=test repudiation 1&repudiate=on&ack_id=%d' 
  		% (v['pin'],repud_ackid),
  		'expected':'Acknowledgment: %d successfully input' % (-1*repud_ackid),
  		'testno':testno,
  		'description':'valid repudiation' })
  
  
  # test 7: check jack balance after repudiating payment
  
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=jill&enquiry_id=jack&pin=%s&ua=user' 
  	      % jpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  	        'description':"check jack's balance" })
  
  # test 8: check jill balance after repudiating payment
  
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'balance.cgi',
                'data':'ac_id=jill&enquiry_id=jill&pin=%s&ua=user' 
  	      % jpin,
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':"check jill's balance" })
  
  # test 8 checking jill can herself pay jack with who_can_ack ac_can_pay 

  testno+=1
  expected1='<p> Acknowledgment: %d successfully input.</p>' % (next_ack_id+1)
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
              'data':'ac_id=jill&pin=%s&ua=user&from_id=jill&from_name=Jill&to_id=jack&to_name=Jack&amount=10.0&details=test ack made by jill herself' % jpin,
              'expected':expected1,
              'testno':testno,
              'description':'jill can acknowledge with who_can_ack as ac_can_pay' })

  # test 9 admin pays from jack to jill to zero account again

  testno+=1
  expected2='<p> Acknowledgment: %d successfully input.</p>' % (next_ack_id+2)
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
              'data':'ac_id=admin&pin=%s&ua=admin&from_id=jack&from_name=Jack&to_id=jill&to_name=Jill&amount=10.0&details=repay ack made by jill herself' % v['pin'],
              'expected':expected2,
              'testno':testno,
              'description':'jack pays jill to zero a/c again' })

  # test 10 jill cant initiate payment from jack to jill  

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
              'data':'ac_id=jill&pin=%s&ua=user&from_id=jack&from_name=Jack&to_id=jill&to_name=Jill&amount=10.0&details=invalid test ack made by jill to herself' % jpin,
              'expected':'You can only pay out of your own account.',
              'testno':testno,
              'description':'jill cant pay herself from other a/c' })

  # test 9: attempt valid delete of account with zero balance

  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
  		'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&delete=delete' % v['pin'],
  		'expected':"Thanks for your input which has been received and",
  		'testno':testno,
  		'description':'valid delete of user with zero balance' })
  
  # test 10: obtain Jill's statement
 
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=jill&pin=%s&ua=user&statement=statement' 
  	      % jpin,
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'valid statement for jill' })
  
  # test 11: obtain acnowledgements ledger success 
  
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&ledger=ledger' 
  	      % v['pin'],
                'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  		'testno':testno,
  		'description':'valid ledger report by admin' })
  
  # test 12: obtain acnowledgements ledger fail as non admin 
  
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
               'data':'ac_id=jill&ledger=ledger&pin=%s&ua=user' 
 	      % jpin,
                'expected':'Unauthorised attempt to: LDGRVIEW by account: jill logged.',
        	'testno':testno,
                'description':'invalid ledger request by non-admin' })
 
  # test 13: add account for jack again

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
              'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&add=add&user_name=Jack&email=nullj@%s&start_bal=0.0&phone1=01234 56789&phone2=0781 987654&address=7 Drury Lane,\nLondon&postcode=SW1 2WW&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
              'expected':"<p>new member added",
              'testno':testno,
              'description':'valid re-add user: Jack' })

  # test 16 check adam can not login until added

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'entry.cgi',
              'data':'account=adam&user=user&pin=12345678',
              'expected':"Incorrect PIN and/or account entered, or account inactive.",
              'testno':testno,
              'description':'unknown account' })

  # test 17: add account for adam
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
              'data':'ac_id=admin&user_ac_id=adam&pin=%s&ua=admin&add=add&user_name=Adam Ant&email=nulla@%s&start_bal=0.0&phone1=012345678&phone2=0781 987764&address=10 Broad Lane\nCoventry&postcode=CV4 9PG&district=Stoke Aldermoor&limit=None' % (v['pin'],emd),
              'expected':"<p>new member added",
              'testno':testno,
              'description':'valid add user: adam' })

  # test 18: authorise adam to input payment for jack 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'authorise.cgi',
                'data':'ac_id=admin&acbe=adam&pin=%s&ua=admin&add=add&auth=PAY&whom=jack'
% v['pin'],
              'expected':"New authorisation PAY added for adam",
              'testno':testno,
              'description':'authorise user adam to PAY for jack' })
 
  # test 19: obtain Jill's statement by admin
 
  testno+=1 
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'accounts.cgi',
           'data':'ac_id=admin&pin=%s&ua=admin&whom=jill&statement=statement' 
  	      % v["pin"],
           'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
           'testno':testno,
           'description':'valid statement for jill obtained by admin' })
  
  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)

  # postrequisites

  # modify secure.cfg who_can_ack to ac_can_pay - needed for test7.py
  testlib.modify_config_var('test_configs/secure.cfg','who_can_ack','admin_plus')
 
if __name__ == "__main__":
  main() 
