#!/usr/bin/env python
''' test7.py Part B of Test suite for PyLETS CGI 

    test5.py - test8.py are designed to be used
    as a set.
    
    Purpose of tests: 
    a. check jill can't pay jack with who_can_ack admin_alone
    b. check adam can make jack pay to jill with ack for jack auth 

    Prerequisite: 
    secure.cfg variable who_can_ack set to admin_alone
    adam must exist and have been granted ACK for adam authorisation
    Postrequisite: 
    secure.cfg variable who_can_ack set to all_can_record
'''

def main(change_expected=False):
  print("\nTest Case 7: ack security admin_alone and who_can_admin settings\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars(savedpin=1)
  tv=testlib.get_vars(cfile='test.cfg')
  # need jill's PIN 
  jpin=testlib.get_userpin('jill')
  apin=testlib.get_userpin('adam')
  next_ack_id=testlib.get_nextackid()

  testcase=7
  tests=[]
  testno=0

  # test 1 checking jill can not herself pay jack with who_can_ack admin_plus

  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
              'data':'ac_id=jill&pin=%s&ua=user&from_id=jill&from_name=Jill&to_id=jack&to_name=Jack&amount=10.0&details=test ack made by jill herself' % jpin,
              'expected':'Unauthorised attempt to: PAY by account: jill logged.',
              'testno':testno,
              'description':'jill can not acknowledge with who_can_ack as admin_plus' })

  # test 2 checking adam can make jack pay to jill with ACK for jack auth

  testno+=1
  expected1='<p> Acknowledgment: %d successfully input.</p>' % next_ack_id
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'ack.cgi',
              'data':'ac_id=adam&pin=%s&ua=admin&from_id=jack&from_name=Jack&to_id=jill&to_name=Jill&amount=10.0&details=test ack made by adam for jack to jill' % apin,
              'expected':expected1,
              'testno':testno,
              'description':'adam can make jack pay to jill with ACK for jack auth' })


  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)
  # postrequisite 
  testlib.modify_config_var('test_configs/secure.cfg','who_can_ack','all_can_record')

if __name__ == "__main__":
  main()
