#!/usr/bin/env python
''' test9.py of Test suite for PyLETS CGI

    test9 - test13 are designed to be runnable as a unit
    from a clean test system installation.
    
    Purpose of tests:
    a. adding 3 new accounts to system, jack, jill and ben
    b. make jack inactive
    c. view active member contact details
    d. view all member contact details
    e. view full contact records by admin 
    f. view post labels 2 cols active by admin
    g. view post labels 3 cols all records by admin  
    h. various view contacts/label errors input by admin

    prerequisites: system to be started clean by this test set. 
'''

def main(change_expected=False):
  print("\nTest Case 9: members contact information display viewed by admin.\n")
  
  import testlib
  # get test configuation variables in dictionary v
  v=testlib.get_vars()
  tv=testlib.get_vars(cfile='test.cfg')
  emd=tv['email_domain']
  v['pin']=testlib.run_make_clean()
  next_ack_id=testlib.get_nextackid()
  
  tests=[]
  testcase=9
  testno=0
  
  # test 1: add account jack valid request 1 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&add=add&user_name=Jack&email=nullj@%s&start_bal=0.0&phone1=01234 56789&phone2=0781 987654&address=7 Drury Lane,\nLondon&postcode=SW1 2WW&district=Bell Green&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Jack' })
  
  # test 2: add account jill valid request 2 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jill&pin=%s&ua=admin&add=add&user_name=Jill&email=nullk@%s&start_bal=0.0&phone1=012345 6897&phone2=None&address=2 Tuppence Rd,Coventry&postcode=CV2 2JX&district=Finham&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Jill' })
  
  # test 3: add account ben valid request 3 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=ben&pin=%s&ua=admin&add=add&user_name=Ben&email=nullb@%s&start_bal=0.0&phone1=012345 6897&phone2=None&address=2 Broad Lane,Coventry&postcode=CV4 3XY&district=Holbrooks&limit=None' % (v['pin'],emd),
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Ben' })
  
  # test 4: make account jack inactive 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=jack&pin=%s&ua=admin&status=status&pai=inactive' 
  	      % v['pin'],
  	      'expected':'<p>Thanks for your input which has been received and',
  	      'testno':testno,
  	      'description':'inactivate account: Jack' })

  # test 5: view active member contact details
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&active=active' 
  	      % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'view active member contact details as admin' })
  
  # test 6: view all member contact details
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&all=all' 
  	      % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'view all member contact details as admin' })
  
  # test 7: view full contact records by admin 
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&addr=addr' 
  	      % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'view full member contact details as admin' })
  
  # test 8: view post labels 2 cols active by admin
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&lcols=2&labelsa=active&noemail=include' 
  	      % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'view post labels 2 cols active' })


  # test 9: view post labels 3 cols all records by admin  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&lcols=3&labelsa=all&noemail=include' 
  	      % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'view post labels 3 cols all members' })
  
  # test 10: add account lisa without email valid request  
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'admin.cgi',
                'data':'ac_id=admin&user_ac_id=lisa&pin=%s&ua=admin&add=add&user_name=Lisa&email=None&start_bal=0.0&phone1=01234 56789&phone2=0781 987777&address=11 Drury Lane,\nLondon&postcode=SW1 2WW&district=Paradise&limit=None' % v['pin'],
  	      'expected':"<p>new member added",
  	      'testno':testno,
  	      'description':'valid add user: Lisa no email' })
  
  # test 11: view post labels 3 cols only members without email   
  testno+=1
  tests.append({'url':'http://'+tv['domain']+tv['testdir']+'cdirectory.cgi',
                'data':'ac_id=admin&pin=%s&ua=admin&labels=labels&lcols=3&labelsa=all&noemail=without' 
  	      % v['pin'],
              'resultfile':'test'+str(testcase)+'_'+str(testno)+'.tst',
  	      'testno':testno,
  	      'description':'view post labels members without email only' })
  
  # do all stored tests
  testlib.run_tests(tests,change_expected=change_expected)


if __name__ == "__main__":
  main()
