#!/usr/bin/env python

# change 0 to 1 to debug script

debug=0

""" wantoffer.cgi This file handles input from the wantoffer 
    form used to input wantoffer details. 
    Richard Kay last changed: 21 Mar 2008
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
from table_details import wotab,mtab,cattab

# who_can_post is controlled by editing secure.cfg file
# ac_can_post to allow account holders to post their own wants/offers
# admin_alone to allow only admin account to post wants/offers for other users 
# all_can_post to allow any account holders to post wants/offers for other users

sec_vars=cgiutils.get_vars('secure.cfg')
who_can_post=sec_vars['who_can_post']

if who_can_post not in ['ac_can_post','admin_alone','all_can_post']:
  raise ValueError("invalid value for who_can_post configured")

site_vars=cgiutils.get_vars('site.cfg')
dl=site_vars['data_location']

def conv_datetime(timeval):   
  import time
  time_struct=time.localtime(float(timeval))
  return time.asctime(time_struct)

def main():
  session=False
  cgiutils.html_header(title="LETS want and offer processing",
                       bgcolor='"#DDFFFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  # 
  if not woi.has_required(["wo","skill","category","desc"]):
    cgiutils.html_end(session,
       error="Missing value/s. Complete all form fields.")
    return
  # check if account is admin and who can post is admin_alone
  if who_can_post=='admin_alone' and ac_id != "admin":
    cgiutils.html_end(session,
       error="Only the administrator is allowed to do this.")
    return
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="Invalid or stale or logged out session.")
    return
  #
  # check if authorised user entered input for someone else
  members=tablcgi.table(mtab,dir=dl)
  has_who=woi.has_required(["who"])
  if ac_id == "admin" :
    if not has_who:
      cgiutils.html_end(session,
            error="Admin input of want/offer but don't know who for.")
      return
  if has_who:
    who=woi.firstval("who")
    if not cgiutils.is_valid(who,max_leng=8,allowed='^[\w]+$'):
      cgiutils.html_end(session,
         error="Invalid account id/no making want/offer .")
      return
    whomember=who in members.keys()
    if not whomember:
      cgiutils.html_end(session,
         error="Trying to input want/offer for nonmember.")
      return
  # validate other fields
  wo=woi.firstval("wo")
  if not cgiutils.is_valid(wo,max_leng=5,allowed='^want$|^offer$'):
    cgiutils.html_end(session,error="This must be either a want or an offer.")
    return
  skill=woi.firstval("skill")
  if not cgiutils.is_valid(skill,max_leng=1,allowed='^[PSBU]$'):
    cgiutils.html_end(session,error=
    "This must be either P (Professional) or S (Semi skilled)" + 
    " or B (Beginner) or U (Unknown/not applicable)")
    return
  category=woi.firstval("category")
  cattable=tablcgi.table(cattab,dir=dl)
  allowed_categories=[]
  for row in cattable.data:
    allowed_categories.append(row["category"])
  if category not in allowed_categories:
    cgiutils.html_end(session,error="Unknown want or offer category.")
    return
  if not woi.has_required(["day","month","year"]):
    expiry="31 Dec 2029"
  else:
    day=woi.firstval("day")
    month=woi.firstval("month")
    year=woi.firstval("year")
    expiry=day+' '+month+' '+year
    if expiry == '  ' :
      expiry="31 Dec 2029"
  if not cgiutils.is_valid(expiry,max_leng=20,
      allowed='^[0-9]{1,2} [A-Za-z]{3,10} [0-9]{4,4}$'):
    cgiutils.html_end(session,error="Unparseable expiry date format error 1.")
    return
  # convert expiry date into float format
  import time
  posted=int(time.time())
  try:
    to=time.strptime(expiry,"%d %b %Y")
    expirytime=int(time.mktime(to))
  except(ValueError,OverflowError):
    errmes=expiry+" does not seem to be a valid date."
    cgiutils.html_end(session,errmes)
    return
  desc=woi.firstval("desc")
  if expirytime < posted:
    cgiutils.html_end(session,
       error="expiry date/time before current date/time.")
    return
  if not cgiutils.is_valid(desc,max_leng=70,
    allowed=r'''^[\w \'\-\.\,\!\"\'\$\%\^\&\*\(\)\+\=\;\:\?\/\\\@\|]+$'''):
    cgiutils.html_end(session, 
       error="Invalid character in or too long description.")
    return
  # party making want/offer
  if ac_id == "admin" :
    party=who
  elif has_who and who == ac_id: # allow redundant input of own ac_id
    party=ac_id
  elif has_who: # have to authorise it later 
    party=who
  else:
    party=ac_id
  worow={
        "ac_id":party,
        "wo":wo,
        "categ":category,
        "skill":skill,
        "desc":desc,
        "expiry":expirytime,
	"posted":posted }
  # make audit row details date formats human readable
  auditrow=worow.copy()
  auditrow['expiry']=conv_datetime(worow['expiry']) 
  auditrow['posted']=conv_datetime(worow['posted']) 
  wos=tablcgi.sq_table(wotab,dir=dl)
  wo_id=wos.get_nextnum()
  sauditrow=cgiutils.mystr(auditrow) # ensures consistent ouput
  if who_can_post == 'all_can_post' :
    cgiutils.auth(session,ac_id,"WOADD",sauditrow,whom=party,
           auditonly=True)
  elif who_can_post == 'ac_can_post' and ac_id == party:
    cgiutils.auth(session,ac_id,"WOADD",sauditrow,whom=party,
           auditonly=True)
  elif cgiutils.auth(session,ac_id,"WOADD",sauditrow,whom=party,
           auditonly=False):
    pass # authorisation succeeded
  else:
    return # authorisation failed 
  wos.addrow(worow)
  print("<p> Want or offer: "+str(wo_id)+" successfully input.</p>") 
  cgiutils.html_end(session, received=1)
  return 

if __name__ == "__main__":
  try: 
    main()
  except:
    import traceback
    print("error detected in wantoffer.cgi main()")
    traceback.print_exc()

