#!/usr/bin/env python

# change next line to 1 to debug script

debug=0

""" yearend.cgi
    This program handles input from the yearend form. 
    
    This performs some long term system maintenance functions.
 
    a. System balancing operations via creation of a writeoff
    account with a starting balance computed so all account
    balances add up to zero.    
 
    b. Inactive account management to enable dead accounts
    to be identified and have residual balances transferred
    into the writeoff account.

    c. Year end processing enables transactions
    to be archived and carry forward and starting balances are
    updated. 
    Richard Kay 07 Apr 2010
"""

if debug:
  import sys
  sys.stderr=sys.stdout
  print("Content-type: text/plain\n")

import cgiutils
import webinput
woi=webinput.Webinput()
import tablcgi
import table_details
from table_details import atab,btab,mtab

v=cgiutils.get_vars('site.cfg')
dl=v['data_location']
sec_vars=cgiutils.get_vars('secure.cfg')
ack_generations=int(v['ack_generations'])

def rotate_ackfiles():
  ''' rotates archived ack files. Doesn't touch acks.pkl, just 
  increments number N for all files in range acks.N.pkl up to 
  the maximum number of generations. '''
  import os, os.path
  prefix="acks"
  generations=list(range(ack_generations))
  # list of number of log generations kept
  generations.reverse() # want to count down not up
  for gen in generations:
    thisfile=os.path.join(dl,prefix+'.'+str(gen)+'.pkl')
    nextfile=os.path.join(dl,prefix+'.'+str(gen+1)+'.pkl')
    if os.path.isfile(thisfile):
      os.rename(thisfile,nextfile)

def get_balance():
  ''' computes overall balance of system ''' 
  balances=tablcgi.table(btab,dir=dl)
  sum=0.0 # add every balance to this sum
  for brow in balances.data :
    bal=brow["balance"]
    sum+=bal
  return sum

def get_date(session,minsecs=3600*24*365):
  ''' obtains and validates date field which must be minsecs
  old at least to be valid.
  minsecs - forces error if date is later than now - minsecs 
  returns tuple containing:
  Boolean validity, 
  integer no. secs since epoch, 
  String formatted date
 '''
  if "date" not in woi.keys():
    cgiutils.html_end(session,error="A cutoff date is required.")
    return False,0,''
  strdate=woi.firstval("date") # format DD Mmm YYYY e.g. 25 Dec 2007
  if not cgiutils.is_valid(strdate,max_leng=20,
      allowed='^[0-9]{1,2} [A-Za-z]{3,10} [0-9]{4,4}$'):
    cgiutils.html_end(session,error="Unparseable expiry date format error 1.")
    return False,0,strdate
  # convert date into numeric format
  import time
  now=int(time.time())
  try:
    to=time.strptime(strdate,"%d %b %Y")
    datetime=int(time.mktime(to))
  except(ValueError,OverflowError):
    cgiutils.html_end(session,error="Unparseable expiry date format error 2.")
    return False,0,strdate
  if now < (datetime + minsecs) :
    cgiutils.html_end(session,error=
      "This facility not intended for accounts used that recently.")
    return False,0,strdate
  return True,datetime,strdate
     
def proc_inactive(session,ac_id,option):
    # for every inactive account creates row in a view table
    # giving date of last acknowledgement or an epoch date.
    # If an ack later than a cutoff date is found row
    # is removed. Purpose is to identify accounts dead for
    # more than a certain period e.g. 5 years.
    valid,cutoff,strdate=get_date(session,minsecs=3600*24*365)
    if not valid:
      return # error message already sent
    # create members table view object
    mtabvo=tablcgi.table_def(mtab)
    mtabvo.file=None
    mtabvo.meta["keys"].append("cfwd")
    mtabvo.meta["keys"].append("balance")
    mtabvo.meta["keys"].append("in")
    mtabvo.meta["keys"].append("out")
    mtabvo.meta["keys"].append("turnover")
    mtabvo.meta["keys"].append("limit")
    mtabvo.meta["keys"].append("last")
    mtabvo.meta["colheads"].append("CFWD")
    mtabvo.meta["colheads"].append("Balance")
    mtabvo.meta["colheads"].append("In")
    mtabvo.meta["colheads"].append("Out")
    mtabvo.meta["colheads"].append("Turnover")
    mtabvo.meta["colheads"].append("Limit")
    mtabvo.meta["colheads"].append("Last Transaction")
    mtabvo.meta["formats"].append("%.2f")
    mtabvo.meta["formats"].append("%.2f")
    mtabvo.meta["formats"].append("%.2f")
    mtabvo.meta["formats"].append("%.2f")
    mtabvo.meta["formats"].append("%.2f")
    mtabvo.meta["formats"].append("%s")
    mtabvo.meta["formats"].append(table_details.print_date_time)
    mtv=tablcgi.table(mtabvo)
    members=tablcgi.table(mtab,dir=dl)
    balances=tablcgi.table(btab,dir=dl)
    for mrow in members.data :
      row={}
      for key in ["ac_id","pin","name","email","active","phone1","phone2",
              "address","postcode","district","url","pic"]:
        row[key]=mrow[key]
      bidx=balances.find(mrow["ac_id"])
      if bidx == -1 : # admin member with no balance record
        continue # at next loop iteration
      brow=balances.data[bidx] 
      for key in ["cfwd","balance","in","out","turnover","limit"]:
        row[key]=balances.data[bidx][key]
      # initial last transaction time set to epoch i.e. earliest value
      row["last"]=0 
      if mrow["active"] == 'N': # we are only looking at inactives
        mtv.addrow(row)
    # now that we have a table of inactives, get their last transaction dates
    # To do this, we go through every archived ack. First get filenames
    ackfnames=cgiutils.list_ackfiles()
    for ackf in ackfnames : # process current acks and archived acks 
      atdo=tablcgi.table_def(atab) # clone ack file def object  
      atdo.file=ackf
      ackset=tablcgi.table(atdo,dir=dl)
      for ackrow in ackset.data :
        from_id=ackrow["from_id"]
        to_id=ackrow["to_id"]
        datetime=ackrow["time"]
        if from_id in mtv.keys() :
          fidx=mtv.find(from_id)
          fromtime=mtv.data[fidx]["last"]
          if datetime > fromtime: 
            mtv.data[fidx]["last"]=datetime # update table with latest ackdate
        if to_id in mtv.keys() :
          tidx=mtv.find(to_id)
          totime=mtv.data[tidx]["last"]
          if datetime > totime: 
            mtv.data[tidx]["last"]=datetime # update table with latest ackdate
    # Now remove all rows from mtv view table with last later 
    # than cutoff date/time, leaving just the dead inactives of interest
    # First get deletion keys.
    delkeys=[]
    for mrow in mtv.data :
      if mrow["last"] > cutoff: # transaction later than cutoff date
        delkeys.append(mrow["ac_id"])
    for dkey in delkeys:
      mtv.delrow(dkey)
    if option == "viewinact": 
      if not cgiutils.auth(session,ac_id,
           "MEMBVIEW","viewing dead inactives",whom="admin"):
        return
      mtv.tab2html(skip_cols=["pin"])
      cgiutils.html_end(session)
      return
    elif option == "clearout":
      if not cgiutils.auth(session,ac_id,
           "WRITEOFF","clearing out dead inactive accounts",whom="admin"):
        return
      widx=balances.find("writeoff") # writeoff account must exist
      if widx == -1 : 
        cgiutils.html_end(session,
           error="writeoff account must exist for this.")
        return
      # for each account transfer balance to writeoff a/c
      deletions=0
      # this next bit is a bit rough - an improvement would be to
      # use ack.cgi in client/server mode to transfer funds 
      # under greater control. At least it is audited. 
      for mrow in mtv.data :
        widx=balances.find("writeoff") # need to repeat indexing each time 
        delac=mrow["ac_id"] # a/c to be deleted
        bal=mrow["balance"] # balance to transfer to writeoff a/c
        details="Deleting a/c: %s writing off balance: %.2f" % (delac,bal) 
        cgiutils.auth(session,ac_id,
           "WRITEOFF",details,whom="admin",auditonly=True)
        balances.data[widx]["balance"]+=bal
        mrow["balance"]-=bal # might be useful if we have to debug this 
        members.delrow(delac) 
        members.save()
        balances.delrow(delac)
        balances.save() # note this changes the record widx indexes 
        deletions+=1
      print("<p> %d long inactive account deletions </p>" % deletions) 
      cgiutils.html_end(session,received=1)
      return

def yearend(session,ac_id):
    valid,arch_date,strdate=get_date(session,minsecs=3600*24*28)
    if not valid:
      return
    if not cgiutils.auth(session,ac_id,
         "YEAREND","rotating ack tables, archiving old acks",whom="admin"):
      return
    # move all files of form acks.N.pkl up by a generation
    rotate_ackfiles()
    #
    balance=tablcgi.table(btab,dir=dl,save_asap=0)
    # need atab clones for newer and archived acks
    acks=tablcgi.table(tablcgi.table_def(atab),
         dir=dl,read_locked=True,save_asap=0)
    archatab=tablcgi.table_def(atab)
    archatab.file="acks.0.pkl" # should be vacant due to rotation
    aack=tablcgi.table(archatab,dir=dl,save_asap=0)
    # deal with uninitialised oldturn data in balances if any
    for i in range(len(balance.data)):
      # this is only needed if upgrading from previous versions
      # which had no old_turnover data
      if 'oldturn' not in balance.data[i]:
        balance.data[i]['oldturn']=0.0
    # start list of ack_ids to be deleted from current ack file
    delrows=[]
    # process all acks
    for ack in acks.data:
      ack_id=ack["ack_id"]
      aindex=acks.find(ack_id)
      time=ack["time"]
      from_id=ack["from_id"]
      to_id=ack["to_id"]
      amount=ack["amount"]
      if time < arch_date:
        # time of ack is before the archive date/time.
        tindex=balance.find(to_id)
        findex=balance.find(from_id)
        if ack_id > 0: # normal ack
          # Add/Subtract amount to/from CFWD, turnover and archive the ack.
          print("<br>archiving ack id: %d aindex %d" % (ack_id,aindex))
          balance.data[tindex]["cfwd"]+=amount
          balance.data[tindex]["in"]-=amount
          balance.data[tindex]["turnover"]-=amount
          balance.data[tindex]["oldturn"]+=amount
          balance.data[findex]["cfwd"]-=amount
          balance.data[findex]["out"]-=amount
          balance.data[findex]["turnover"]-=amount
          balance.data[findex]["oldturn"]+=amount
        else: # this is a repudiation cancelling the normal one
          # reverse the above sums for CFWD etc. to nullify repudiated ack. 
          print("<br>archiving repudiation: %d aindex %d" % (ack_id,aindex))
          balance.data[tindex]["cfwd"]-=amount
          balance.data[tindex]["in"]+=amount
          balance.data[tindex]["turnover"]+=amount
          balance.data[tindex]["oldturn"]-=amount
          balance.data[findex]["cfwd"]+=amount
          balance.data[findex]["out"]+=amount
          balance.data[findex]["turnover"]+=amount
          balance.data[findex]["oldturn"]-=amount
        # archive the old ack
        delrows.append(ack_id) # add ack_id to list of rows to be removed
        aack.addrow(ack)
    # remove archived rows from acks file
    for ack_id in delrows:
      acks.delrow(ack_id)
    balance.save()
    aack.save()
    acks.save()
    # update carry forward date text file
    fp=open("cfwd_date.txt","w")
    fp.write("\n<p>Carried Forward Date:"+ strdate + "</p>\n")
    fp.close()
    cgiutils.html_end(session,received=1)
    return

def report_balance(session,ac_id):
    sum=get_balance()
    correction=-sum
    print("<p> The overall balance of the system is: %.2f <p>" % sum)
    if correction*correction < 0.0001: # one penny squared is 1/10000
      print("Accounts balance to within a penny. No correction possible.<p>")
    else:  
      print("To correct this a writeoff account with a starting ")
      print("balance of %.2f is required <p>" % correction)  
    cgiutils.html_end(session,received=1)
    return

def make_writeoffac(session,ac_id):
    sum=get_balance()
    correction=-sum
    if correction*correction < 0.0001: # one penny squared is 1/10000
      print("Accounts balance to within a penny. No correction possible.<p>")
      cgiutils.html_end(session,received=1)
      return
    newpin=cgiutils.make_pin(mini=10000000,maxi=99999999)
    mrow={}
    newac="writeoff" # name of account. Don't use this for a real A/C
    mrow["ac_id"]=newac
    mrow["name"]="Writing Off Account"
    mrow["email"]='None'
    mrow["pin"]=newpin
    mrow["phone1"]='None'
    mrow["phone2"]='None'
    mrow["address"]='unknown'
    mrow["postcode"]='None'
    mrow["district"]='unknown'
    mrow["active"]='Y' # active membership
    mrow["url"]='None'
    mrow["pic"]='None'
    brow={}
    brow["limit"]="None"
    brow["ac_id"]=newac
    brow["balance"]=correction
    brow["cfwd"]=0.0
    brow["in"]=0.0
    brow["out"]=0.0
    brow["turnover"]=0.0
    brow["oldturn"]=0.0
    balance=tablcgi.table(btab,dir=dl)
    has_already=balance.find(newac) # check if account already exists
    if has_already != -1 : 
      cgiutils.html_end(session,
         error="writeoff account exists can't create it.")
      return
    members=tablcgi.table(mtab,dir=dl)
    if not cgiutils.auth(session,ac_id,"WRITEOFF",str(mrow)+str(brow),
           whom="admin"):
      return
    members.addrow(mrow)
    members.save()
    balance.addrow(brow)
    balance.save()
    print("<p> Account writeoff added with balance %.2f ." % correction)
    print("<p> Use this to collect non-zero balances from long dead ")
    print("accounts prior to their removal.<p>")
    print("It is then recommended that you zero this account by ")
    print("making a payment between it and whichever account") 
    print("is to be written down or up in order to bring the system")
    print("back into zero balance. You should then remove the writeoff")
    print("account. <p>") 
    cgiutils.html_end(session,received=1)
    return

def main():
  session=False
  cgiutils.html_header(title="Year End and System Balancing",
                       bgcolor='"#FFFFFF"')
  valid,ac_id,pincookie,ua=cgiutils.get_hidden(woi)
  if not valid:
    return # error message already sent
  valid,option=cgiutils.get_option(woi,buttons=[
      "balance","writeoff","viewinact","clearout","yearend"])
  if not valid:
    return # error message already sent
  # check pin_or_cookie and account
  session=cgiutils.login(ac_id,pincookie,ua)
  valid=session.login()
  cookie=session.cookie
  if not valid:
    cgiutils.html_header(title="LETS Accounting and Directory System")
    cgiutils.html_end(session,
       error="Invalid or stale or logged out session.")
    return
  # 
  # only do work for an admin account
  if ua != 'admin': 
    cgiutils.html_end(session,error="Administrator login required.")
    return
  # 
  # If we've got this far, there should be a valid option and PIN OK
  if option == "balance": # need to report overall balance of system 
    report_balance(session,ac_id)
    return
  elif option == "writeoff": # create writeoff account to balance system 
    make_writeoffac(session,ac_id)
    return
  elif option in ["viewinact","clearout"]:
    proc_inactive(session,ac_id,option)
    return
  elif option == "yearend":
    yearend(session,ac_id)
    return
  
if __name__ == "__main__":
  try:
      main()
  except:
    import traceback
    print("error detected in yearend.cgi main()")
    traceback.print_exc()

